/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	CLXTRACT				*/
/*	Program to extract information from	*/
/*	logs and have them displayed via WEB	*/
/*						*/
/************************************************/
#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<errno.h>
#include	<string.h>
#include	"subrou.h"

#define	CLXTRACT	"clxtract"

typedef	enum	{
	s_date,		/*select date		*/
	s_orig,		/*select originator	*/
	s_recip,	/*select recipient	*/
	s_status,	/*select status		*/
	s_subject,	/*select subject	*/
	s_void		/*no select		*/
	}SRHENUM;


typedef	struct	{	/*search structur	*/
	SRHENUM field;	/*selected search field	*/
	char *search;	/*string to search	*/
	}SRHTYP;
/*

*/
/************************************************/
/*						*/
/*	Procedure to display the program usage	*/
/*						*/
/************************************************/
static void usage()

{
(void) fprintf(stderr,"%s Version %s\n",appname,rou_getvers());
(void) fprintf(stderr,"usage:  %s "
		      "[-b base] "
		      "[-d debug] "
		      "[-h ] "
		      "[-l days] "
		      "[-v ] "
		      "extension"
		      "username|\"\" "
		      "sort "
		      "search|\"\" "
		      "\n",appname);
(void) fprintf(stderr,"\t\t-b base   : application working directory\n");
(void) fprintf(stderr,"\t\t-d debug  : debug level\n");
(void) fprintf(stderr,"\t\t-h        : display usage\n");
(void) fprintf(stderr,"\t\t-l days   : how many days of hitory to extract\n");
(void) fprintf(stderr,"\t\t-v        : verbose debug\n");
(void) fprintf(stderr,"\t\textension : name extension to output file name\n");
(void) fprintf(stderr,"\t\tusername  : to extract, full log if \"\"\n");
(void) fprintf(stderr,"\t\tsort	     : display order (status,date,originator,"
				       "recipient subject)\n");
(void) fprintf(stderr,"\t\tsearch    : search for specific (status,...)\n");
(void) exit(-1);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a 	*/
/*	SRHTYP structure.			*/
/*						*/
/************************************************/
static SRHTYP *freesearch(SRHTYP *srh)

{
if (srh!=(SRHTYP *)0) {
  srh->search=rou_freestr(srh->search);
  (void) free(srh);
  srh=(SRHTYP *)0;
  }
return srh;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to convert a search parameter	*/
/*	to a SRHTYP structure			*/
/*						*/
/************************************************/
static SRHTYP *getsearch(char *search)

{
SRHTYP *srh;

srh=(SRHTYP *)0;
if ((search!=(char *)0)&&(strlen(search)>0)) {
  int pos;
  char *field;
  char *contents;

  field=(char *)calloc(strlen(search),sizeof(char));
  contents=(char *)calloc(strlen(search),sizeof(char));
  if (sscanf(search,"%[^=\n\r]=%n",field,&pos)==1) {
    int i;

    (void) strcpy(contents,search+pos);
    srh=(SRHTYP *)calloc(1,sizeof(SRHTYP));
    srh->field=s_void;
    for (i=0;((i<5)&&(srh->field==s_void));i++) {
      switch (i) {
	case 0	:	/*checking date		*/
	  if (strcasecmp(field,"date")==0) {
	    srh->field=s_date;
	    srh->search=strdup(contents);
	    }
	  break;
	case 1	:	/*checking originator		*/
	  if (strcasecmp(field,"originator")==0) {
	    srh->field=s_orig;
	    srh->search=strdup(contents);
	    }
	  break;
	case 2	:	/*checking recipient		*/
	  if (strcasecmp(field,"recipient")==0) {
	    srh->field=s_recip;
	    srh->search=strdup(contents);
	    }
	  break;
	case 3	:	/*checking subject		*/
	  if (strcasecmp(field,"subject")==0) {
	    srh->field=s_subject;
	    srh->search=strdup(contents);
	    }
	  break;
	case 4	:	/*checking status		*/
	  if (strcasecmp(field,"status")==0) {
	    srh->field=s_status;
	    srh->search=strdup(contents);
	    }
	  break;
	}
      }
    if (srh->field==s_void) {
      (void) alert(0,"%s Unable to scan search request <%s>",
		     "clxtract.c:getsearch,",search);
      srh=freesearch(srh);
      }
    }
  contents=rou_freestr(contents);
  field=rou_freestr(field);
  }
return srh;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to sort the collected data	*/
/*						*/
/************************************************/
static void dosort(char *sort,char *filetmp,char *fileout)

{
SRHTYP *srh;
char field[100];
char command[400];

(void) snprintf(field,sizeof(field),"%s=void",sort);
srh=getsearch(field);
(void) strcpy(field,"");
switch (srh->field) {
  case s_date		:
    (void) snprintf(field,sizeof(field),"%s","");
    break;
  case s_recip		:
    (void) snprintf(field,sizeof(field),"%s","| sort -k7,7f -s");
    break;
  case s_orig		:
    (void) snprintf(field,sizeof(field),"%s","| sort -k6,6f -s");
    break;
  case s_status		:
    (void) snprintf(field,sizeof(field),"%s","| sort -k3,3f -s");
    break;
  case s_subject	:
    (void) snprintf(field,sizeof(field),"%s","| sort -k8,10f -s");
    break;
  case s_void		:
    (void) strcpy(field,"");
    break;
  }
(void) snprintf(command,sizeof(command),"cat %s | sort -r -k1,2g %s > %s",
					 filetmp,field,fileout);
if (system(command)<0)
  (void) alert(0,"%s Unable execute command <%s>","clxtract.c:dosort,",command);
srh=freesearch(srh);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract information from	*/
/*	Clement logs.				*/
/*						*/
/************************************************/
static int extracting(int backlog,char *extension,char *username,
		      char *sort,SRHTYP *srh)

{
#define	OPEX	CLXTRACT":extracting,"


int status;
FILE *fout;
char ext[100];
char filename[300];
char filetmp[350];

status=0;
(void) snprintf(ext,sizeof(ext),".%s",extension);
(void) rou_mkname(filename,sizeof(filename),d_tmp,"",CLXTRACT,ext);
(void) snprintf(filetmp,sizeof(filetmp),"%s.tmp",filename);
if ((fout=fopen(filetmp,"w"))!=(FILE *)0) {
  time_t curtime;
  int i;
 
  curtime=rou_systime(); 
  for (i=0;i<backlog;i++) {
    time_t logtime;
    int logday;
    FILE *fin;
    char curdate[50];
    char filelog[300];

    logtime=curtime-(i*(3600*24));
    logday=rou_normdate(logtime);
    (void) strcpy(curdate,rou_getstrdate(logtime));
    (void) getjourname(filelog,sizeof(filelog),d_web,CLEMENT,logday);
    if ((fin=fopen(filelog,"r"))!=(FILE *)0) {
      int numline;
      char line[500];

      numline=1;
      while (fgets(line,sizeof(line),fin)!=(char *)0) {
	char *curline;
	char *heure;
	char *strstat;
	char *orig;
	char *recip;
	char *subject;
	u_long mtaille;
	char *ptr;
        int j;

	curline=strdup(line);
	heure=(char *)0;
	strstat=(char *)0;
	orig=(char *)0;
	recip=(char *)0;
	subject=(char *)0;
	ptr=curline;
	for (j=0;j<6;j++) {
	  char *nptr;

	  if ((nptr=strchr(ptr,' '))==(char *)0) {
            (void) alert(0,"%s Unable scan file <%s> line <%05d:%s>",
		           OPEX,filelog,numline,line);
	    (void) strcpy(line,"");
	    break;
	    }
	  *nptr='\000';
	  nptr++;
	  while (*nptr==' ')
	    nptr++; 
	  switch (j) {
	    case 0	:	/*getting the time		*/
	      heure=strdup(ptr);
	      break;
	    case 1	:	/*getting the status string	*/
	      strstat=strdup(ptr);
	      break;
	    case 2	:	/*getting the E-mail size	*/
	      mtaille=atol(ptr);
	      break;
	    case 3	:	/*getting the E-mail size unit	*/
	      if (strcmp(ptr,"Bts")==0)
	        break;
	      mtaille*=1024;
	      if (strcmp(ptr,"KBts")==0)
	        break;
	      mtaille*=1024;
	      if (strcmp(ptr,"MBts")==0)
	        break;
	      mtaille*=1024;
	      if (strcmp(ptr,"GBts")==0)
	        break;
	      break;
	    case 4	:	/*getting the originator	*/
	      orig=strdup(ptr);
	      break;
	    case 5	:	/*getting the recipient		*/
	      recip=strdup(ptr);
	      subject=strdup(nptr);
	      break;
	    }
	  ptr=nptr;
	  }
	for (j=0;(j<2)&&(line[0]!='\000');j++) {
	  switch (j) {
	    case 0	:	/*checking if user ok		*/
	      if ((username!=(char *)0)&&(strstr(recip,username)==(char *)0))
		(void) strcpy(line,"");	/*not select entry	*/
	      break;
	    case 1	:	/*checking if other select	*/
	      if (srh!=(SRHTYP *)0) {
		int found;

		found=true;
		switch(srh->field) {
		  case s_date	:
		    if (strcmp(curdate,srh->search)!=0)
		      found=false;
		    break;
		  case s_recip	:
		    if (strstr(recip,srh->search)==(char *)0)
		      found=false;
		    break;
		  case s_orig	:
		    if (strstr(orig,srh->search)==(char *)0)
		      found=false;
		    break;
		  case s_status	:
		    if (strcmp(strstat,srh->search)!=0)
		      found=false;
		    break;
		  case s_subject	:
		    if (strstr(subject,srh->search)==(char *)0)
		      found=false;
		    break;
		  case s_void	:
		    break;
		  }
		if (found==false)
		  (void) strcpy(line,"");
		}
	    }
	  }
	subject=rou_freestr(subject);
	orig=rou_freestr(orig);
	recip=rou_freestr(recip);
	strstat=rou_freestr(strstat);
	heure=rou_freestr(heure);
	curline=rou_freestr(curline);
	if (strlen(line)>0) {
	  (void) fprintf(fout,"%s %s",curdate,line);
	  }
	numline++;
	}
      (void) fclose(fin);
      }
    else {
      (void) alert(0,"%s Unable to open logfile <%s> (status=<%s>)",
		      OPEX,filelog,strerror(errno)); 
      }
    }
  (void) fclose(fout);
  (void) dosort(sort,filetmp,filename);
  (void) unlink(filetmp);
  }
else {
  (void) alert(0,"%s Unable to open result file <%s> (status=<%s>)",
		 OPEX,filename,strerror(errno)); 
  status=-1;
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Main Program				*/
/*						*/
/************************************************/
int main(int argc,char *argv[])

{
#define	BACKLOG	10

char c;
int status;
char *clxtract;
char *extension;
char *usager;
char *sort;
char *search;
int backlog;
SRHTYP *srh;
int i;

status=0;
clxtract=CLXTRACT;
extension=(char *)0;
usager=(char *)0;
sort=(char *)0;
search=(char *)0;
backlog=BACKLOG;
srh=(SRHTYP *)0;
base=strdup(base);
(void) openlog(clxtract,LOG_PID,0);
while ((c=getopt(argc,argv,"b:d:hl:v"))!=EOF) {
  switch(c) {
    case   'b'  :
      (void) free(base);
      base=strdup(optarg);
      break;
    case   'd'  :
      debug=atoi(optarg);
      break;
    case   'l'  :
      backlog=atoi(optarg);
      break;
    case   'v'  :
      verbose=true;
      break;
    case   'h'  :
    default     :
      (void) usage();
      break;
    }
  }
foreground=true;
for (i=0;i<4;i++) {
  if (argv[optind+i]==(char *)0)
    break;
  switch (i) {
    case  0	:	/*setting extension res	*/
      extension=argv[optind+i];
      break;
    case  1	:	/*setting the username	*/
      usager=argv[optind+i];
      break;
    case  2	:	/*setting the sort	*/
      sort=argv[optind+i];
      break;
    case  3	:	/*setting the search	*/
      search=argv[optind+i];
      break;
    }
  }
srh=getsearch(search);
status=extracting(backlog,extension,usager,sort,srh);
srh=freesearch(srh);
base=rou_freestr(base);
return status;
}
