/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	GESCNF:					*/
/*	Take Care of all config definition	*/
/*						*/
/************************************************/
#include	<malloc.h>
#include	<syslog.h>
#include	<stdio.h>
#include	<string.h>
#include	<math.h>
#include	<unistd.h>
#include	<stdlib.h>
#include	<ctype.h>
#include	<errno.h>
#include	"subrou.h"
#include	"unieml.h"
#include	"uninet.h"
#include	"gescnf.h"

/*smtp access to local mailer			*/
#define	LOCMAIL	"127.0.0.25,25"


#define	CNF	".conf"	/*config extension	*/

/*config file vocabulary			*/
#define	SSLCIPHERS	"SSLCIPHERS"		/*SSL ciphers definition sequence	*/
#define	TLSAUTH		"TLSAUTH"		/*Authentication Only under if in TLS	*/
#define	ADMIN		"ADMIN"
#define	SPMDSRV		"SPMDSRV"
#define	SMTPSRV		"SMTPSRV"
#define	RMPONDER	"RMPONDER"
#define	CNDOMAIN	"DOMAIN"
#define	BINDING		"BINDING"
#define	RELAY		"RELAY"
#define	CHECKER		"CHECKER"
#define	SENDER		"SENDER"
#define	CHKIDSRV	"CHKIDSRV"
#define	CHKIDMOD	"CHKIDMOD"
#define	DFLTCODE	"DFLTCODE"
#define	SPAMLIM		"SPAMLIM"
#define	QUALOC		"QUARANTINE_LOCAL"	/*Yes/No	*/
#define	QUAHEAD		"QUARANTINE_HEADER"	/*String	*/
#define	QUAEXTRA	"QUARANTINE_EXTRA"	/*Yes/No	*/
#define	CHKCODE		"CHKCODE"
#define	WEBGOOD		"WEBGOOD"
/*

*/
/************************************************/
/*						*/
/*	Procedure to split relai information	*/
/*						*/
/************************************************/
static int isyes(char *line)

{
int result;

result=false;
switch (line[0]) {
  case 'Y'	:
  case 'y'	:
    result=true;
    break;
  default	:
    result=false;
    break;
  }
return result;
}

/*

*/
/************************************************/
/*						*/
/*	Procedure to split relais information	*/
/*						*/
/************************************************/
static char **splitlist(char *info)

{
char **list;
char *myinfo;

list=(char **)0;
myinfo=(char *)0;
if (info!=(char *)0)
  myinfo=strdup(info);
info=myinfo;
while ((info!=(char *)0)&&(*info!='\000')) {
  char *ptr;

  if ((ptr=strchr(info,':'))!=(char *)0) {
    *ptr='\000';
    ptr++;
    }
  if (strlen(info)>0)
    list=(char **)rou_addveclst((void **)list,(void *)strdup(info));
  info=ptr;
  }
myinfo=rou_freestr(myinfo);
return list;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to us a file as domain list	*/
/*	Filename is ALWAYS understood within	*/
/*	the base directory			*/
/*						*/
/************************************************/
static char **getdomlist(char *domfile)

{
char **list;
FILE *fichier;
char filename[400];

list=(char **)0;
(void) memset(filename,'\000',sizeof(filename));
(void) snprintf(filename,sizeof(filename)-1,"%s/%s",base,domfile);
if ((fichier=fopen(filename,"r"))!=(FILE *)0) {
  char line[400];

  while (rou_getstr(fichier,line,sizeof(line),'#')!=(char *)0) {
    char **nlist;

    if ((nlist=splitlist(line))!=(char **)0) {
      int i;

      for (i=0;nlist[i]!=(char *)0;i++) {
        list=(char **)rou_addveclst((void **)list,(void *)nlist[i]);
        }
      (void) free(nlist);
      }
    }
  (void) fclose(fichier);
  }
else
  (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
		 "gescnf.c:getlist,",filename,strerror(errno));
return list;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to split domain infomation	*/
/*	according domain separator.		*/
/*						*/
/************************************************/
static DFSTYP **mulsplitlist(DFSTYP **dfs,char *doms,
			     char *webgood,float spamlim,
			     int qualoc,char *quahead,int quaextra)

{
char **nlist;

switch (doms[0]) {
  case '.'	:
  case '/'	:
    nlist=getdomlist(doms);
    break;
  default	:
    nlist=splitlist(doms);
    break;
  }
if (nlist!=(char **)0) {
  int i;

  for (i=0;nlist[i]!=(char *)0;i++) {
    dfs=(DFSTYP **)rou_addveclst((void **)dfs,
				 (void *)eml_mkdf(nlist[i],webgood,spamlim,
						  qualoc,quahead,quaextra));
    }
  nlist=rou_freestrlst(nlist);
  }
return dfs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add a default auxiliary	*/
/*	to do relaying.				*/
/*						*/
/************************************************/
static void addrelaying(CNFTYP *cnf)

{
AUXTYP *auxs;
DFSTYP *dfs;
char line[100];

auxs=(AUXTYP *)calloc(1,sizeof(AUXTYP));
(void) snprintf(line,sizeof(line),"%s,25",LOCALHOST);
auxs->sender=strdup(line);
auxs->relay=true;
dfs=eml_mkdf(RLAYDOM,(char *)0,NAN,true,"SPAM",true);
auxs->dfs=(DFSTYP **)rou_addveclst((void **)0,(void *)dfs);
cnf->auxs=(AUXTYP **)rou_addveclst((void **)cnf->auxs,(void *)auxs);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return an SKR location 	*/
/*	if exsiting.				*/
/*						*/
/************************************************/
static SKRTYP *findskr(SKRTYP **lst,CKPREF pref,char *hostname,char *chkcode)

{
SKRTYP *sk;

sk=(SKRTYP *)0;
if (lst!=(SKRTYP **)0) {
  int i;

  for (i=0;lst[i]!=(SKRTYP *)0;i++) {
    int found;

    found=false;
    if (lst[i]->pref!=pref)
      continue;
    if (strcmp(lst[i]->hostname,hostname)!=0)
      continue;
    if (pref==p_sender)
      found=true;
    else {
      if ((chkcode==(char *)0)&&(lst[i]->chkcode)==(char *)0)
        found=true;
      else {
        if ((chkcode!=(char *)0)&&(lst[i]->chkcode)!=(char *)0) 
          if (strcmp(lst[i]->chkcode,chkcode)==0) 
	    found=true;
        }
      }
    if (found==true) {
      sk=lst[i];
      break;
      }
    } 
  }
return sk;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add a sndr/chkr to a list	*/
/*						*/
/************************************************/
static SKRTYP **addskr(SKRTYP **lst,CKPREF pref,char *hostname,AUXTYP *aux)

{
if (hostname!=(char *)0) {
  SKRTYP *sk;

  if ((sk=findskr(lst,pref,hostname,aux->chkcode))==(SKRTYP *)0) {
    sk=(SKRTYP *)calloc(1,sizeof(SKRTYP));
    sk->pref=pref;
    sk->hostname=strdup(hostname); 
    sk->relay=aux->relay;
    if (aux->chkcode!=(char *)0)
      sk->chkcode=strdup(aux->chkcode); 
    lst=(SKRTYP **)rou_addveclst((void **)lst,(void *)sk);
    }
  if (aux->dfs!=(DFSTYP **)0) {
    int i;
  
    for (i=0;aux->dfs[i]!=(DFSTYP *)0;i++) {
      DFSTYP *d;
      DFSTYP *nd;

      d=aux->dfs[i];
      nd=eml_mkdf(d->domain,d->webgood,d->spamlim,
		  d->qualoc,d->quahead,d->quaextra);
      sk->dfs=(DFSTYP **)rou_addveclst((void **)sk->dfs,(void *)nd);
      }
    }
  if (aux->bindlst!=(char **)0) {
    int i;
  
    for (i=0;aux->bindlst[i]!=(char *)0;i++) {
      sk->bindlst=(char **)rou_addveclst((void **)sk->bindlst,
				         (void *)strdup(aux->bindlst[i]));
      }
    }
  }
return lst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to load the clement conf file	*/
/*	and extract domain, binding and checker	*/
/*	informations.				*/
/*						*/
/************************************************/
CNFTYP *cnf_readconfig()

{
#define	OPCN	"gescnf.c:getconfig,"

#define	DCHKIDSRV	"127.0.0.25,411,1"

typedef	enum	{
	c_admin,	/*clement admin			*/
	c_spmdsrv,	/*Spam daemon definition	*/	
	c_smtpsrv,	/*Local default SMTP srv	*/
	c_rmponder,	/*Mentor local transponder	*/
	c_domain,	/*Domain definition		*/
	c_binding,	/*Binding interface def		*/
	c_relay,	/*Relay accepted def		*/
	c_checker,	/*checker server def		*/
	c_sender,	/*local SMTP sender def		*/	
	c_chkidsrv,	/*local checkid server		*/
	c_chkidmod,	/*local checkid mode		*/
	c_chkcode,	/*checker access code		*/
	c_webgood,	/*Domain to set goodmail info	*/
	c_dfltcode,	/*chkr dflt access code		*/
	c_spamlim,	/*Trigger SPAM level		*/
	c_qualoc,	/*quarantine local (default)	*/
	c_quahead,	/*quarantaine subject addition	*/
	c_quaextra,	/*quarantaine subject extra info*/
	c_sslciphers,	/*SSL cipher sequence		*/
	c_tlsauth,	/*authentication on TLS only	*/
	c_unknown	/*Unknown keyword		*/
	}ENUCNF;

static VOCTYP vocable[]={
		{c_admin,ADMIN,(const void *)0},
		{c_spmdsrv,SPMDSRV,(const void *)0},
		{c_smtpsrv,SMTPSRV,(const void *)0},
		{c_rmponder,RMPONDER,(const void *)0},
		{c_domain,CNDOMAIN,(const void *)0},
		{c_binding,BINDING,(const void *)0},
		{c_relay,RELAY,(const void *)0},
		{c_checker,CHECKER,(const void *)0},
		{c_sender,SENDER,(const void *)0},
		{c_chkidsrv,CHKIDSRV,(const void *)0},
		{c_chkidmod,CHKIDMOD,(const void *)0},
		{c_dfltcode,DFLTCODE,(const void *)0},
		{c_chkcode,CHKCODE,(const void *)0},
		{c_webgood,WEBGOOD,(const void *)0},
		{c_spamlim,SPAMLIM,(const void *)0},
		{c_qualoc,QUALOC,(const void *)0},
		{c_quaextra,QUAEXTRA,(const void *)0},
		{c_quahead,QUAHEAD,(const void *)0},
		{c_sslciphers,SSLCIPHERS,(const void *)0},
		{c_tlsauth,TLSAUTH,(const void *)0},
		{c_unknown,(char *)0,(const void *)0}
		};

CNFTYP *cnf;
FILE *fichier;
char fname[400];

cnf=(CNFTYP *)0;
(void) rou_mkname(fname,sizeof(fname),d_etc,"",appname,CNF);
if ((fichier=fopen(fname,"r"))!=(FILE *)0) {
  AUXTYP *auxs;
  int linenum;
  char line[3000];

  auxs=(AUXTYP *)0;
  cnf=(CNFTYP *)calloc(1,sizeof(CNFTYP));
  cnf->chkidsrv=strdup(DCHKIDSRV);
  cnf->dfltcode=strdup("Unassigned");
  cnf->chkidmod=strdup("PAM");
  cnf->smtpsrv=strdup(LOCMAIL);
  cnf->spamlim=NAN;
  cnf->qualoc=true;
  cnf->quahead=strdup("SPAM");
  cnf->quaextra=true;
  linenum=0;
  while (rou_getstr(fichier,line,sizeof(line),(char)'\000')!=(char *)0) {
    VOCTYP *voc;
    char *ptr;

    linenum++;
    if (line[0]=='#')
      continue;
    if ((ptr=strchr(line,'='))!=(char *)0)
      *ptr=' ';
    if ((voc=rou_getvoca(vocable,line))!=(VOCTYP *)0) {
      switch (voc->code) {
        case c_admin		:	/*No Care	*/
	  cnf->adm=(char **)rou_addveclst((void **)cnf->adm,
					  (void *)strdup(line));
	  break;
        case c_spmdsrv		:
	  cnf->spmdsrv=rou_freestr(cnf->spmdsrv);
	  cnf->spmdsrv=strdup(line);
	  break;
        case c_smtpsrv		:
	  cnf->smtpsrv=rou_freestr(cnf->smtpsrv);
	  cnf->smtpsrv=strdup(line);
	  break;
        case c_rmponder		:
	  cnf->rmponder=rou_freestr(cnf->rmponder);
	  cnf->rmponder=strdup(line);
	  break;
        case c_relay		:
	  if (isdigit(line[0])==0) {
	    /*only one relay file name		*/
	    cnf->frelais=rou_freestrlst(cnf->frelais);
	    cnf->frelais=splitlist(line);
	    }
	  else
	    cnf->relais=net_addiplst(cnf->relais,line);
	  break;
        case c_domain		:
	  if (auxs!=(AUXTYP *)0)
	    auxs->dfs=mulsplitlist(auxs->dfs,line,(char *)0,cnf->spamlim,
				   cnf->qualoc,cnf->quahead,cnf->quaextra);
	  else
            (void) alert(0,"%s DOMAIN info <%s> is NOT within binding "
			   "definition (file=<%s>)",OPCN,line,fname);
	  break;
        case c_spamlim		:
	  if (auxs!=(AUXTYP *)0) {
	    if (auxs->dfs!=(DFSTYP **)0) {
	      int i;
	      float spamlim;

	      spamlim=atof(line);
	      for (i=0;auxs->dfs[i]!=(DFSTYP *)0;i++) {
		auxs->dfs[i]->spamlim=spamlim;
	        }
	      }
	    }
	  else
	    cnf->spamlim=atof(line);
	  break;
        case c_qualoc		:
	  if (auxs!=(AUXTYP *)0) {
	    if (auxs->dfs!=(DFSTYP **)0) {
	      int qualoc;
	      int i;

	      qualoc=isyes(line);
	      for (i=0;auxs->dfs[i]!=(DFSTYP *)0;i++) {
		auxs->dfs[i]->qualoc=qualoc;
	        }
	      }
	    }
	  else
	    cnf->qualoc=isyes(line);
	  break;
        case c_quaextra		:
	  if (auxs!=(AUXTYP *)0) {
	    if (auxs->dfs!=(DFSTYP **)0) {
	      int quaextra;
	      int i;

	      quaextra=isyes(line);
	      for (i=0;auxs->dfs[i]!=(DFSTYP *)0;i++) {
		auxs->dfs[i]->quaextra=quaextra;
	        }
	      }
	    }
	  else
	    cnf->quaextra=isyes(line);
	  break;
        case c_quahead		:
	  if (auxs!=(AUXTYP *)0) {
	    if (auxs->dfs!=(DFSTYP **)0) {
	      int i;

	      for (i=0;auxs->dfs[i]!=(DFSTYP *)0;i++) {
	        auxs->dfs[i]->quahead=rou_freestr(auxs->dfs[i]->quahead);
		auxs->dfs[i]->quahead=strdup(line);
	        }
	      }
	    }
	  else {
	    cnf->quahead=rou_freestr(cnf->quahead);
	    cnf->quahead=strdup(line);
	    }
	  break;
        case c_binding		:
	  auxs=(AUXTYP *)calloc(1,sizeof(AUXTYP));
	  auxs->bindlst=splitlist(line);
	  auxs->sender=strdup(cnf->smtpsrv);
	  auxs->chkcode=strdup(cnf->dfltcode);
	  cnf->auxs=(AUXTYP **)rou_addveclst((void **)cnf->auxs,(void *)auxs);
	  if (cnf->bindlst!=(char *)0)
            cnf->bindlst=rou_memstrcat(cnf->bindlst,":");
          cnf->bindlst=rou_memstrcat(cnf->bindlst,"%s",line);
	  break;
        case c_dfltcode		:
	  cnf->dfltcode=rou_freestr(cnf->dfltcode);
	  cnf->dfltcode=strdup(line);
	  break;
        case c_chkcode		:
	  if (auxs!=(AUXTYP *)0) {
	    auxs->chkcode=rou_freestr(auxs->chkcode);
	    auxs->chkcode=strdup(line);
	    }
	  else
            (void) alert(0,"%s CHKCODE info <%s> is NOT within binding "
			   "definition (file=<%s>)",OPCN,line,fname);
	  break;
        case c_webgood		:
	  if (auxs!=(AUXTYP *)0) {
	    if (auxs->dfs!=(DFSTYP **)0) {
	      int i;

	      for (i=0;auxs->dfs[i]!=(DFSTYP *)0;i++) {
	        auxs->dfs[i]->webgood=rou_freestr(auxs->dfs[i]->webgood);
		auxs->dfs[i]->webgood=strdup(line);
	        }
	      }
	    }
	  else
            (void) alert(0,"%s %s info <%s> is NOT within binding "
			   "definition (file=<%s>)",OPCN,WEBGOOD,line,fname);
	  break;
        case c_checker		:
	  if (auxs!=(AUXTYP *)0) {
	    auxs->checker=rou_freestr(auxs->checker);
	    auxs->checker=strdup(line);
	    }
	  else
            (void) alert(0,"%s CHECKER info <%s> is NOT within binding "
			   "definition (file=<%s>)",OPCN,line,fname);
	  break;
        case c_sender		:
	  if (auxs!=(AUXTYP *)0) {
	    auxs->sender=rou_freestr(auxs->sender);
	    auxs->sender=strdup(line);
	    }
	  else
            (void) alert(0,"%s SENDER info <%s> is NOT within domain "
			   "definition (file=<%s>)",OPCN,line,fname);
	  break;
        case c_chkidsrv		:
	  cnf->chkidsrv=rou_freestr(cnf->chkidsrv);
	  cnf->chkidsrv=strdup(line);
	  break;
        case c_chkidmod		:
	  cnf->chkidmod=rou_freestr(cnf->chkidmod);
	  cnf->chkidmod=strdup(line);
	  break;
        case c_sslciphers	:
	  cnf->SSLciphrs=rou_freestr(cnf->SSLciphrs);
	  cnf->SSLciphrs=strdup(line);
	  break;
        case c_tlsauth		:
	  cnf->TLSauth=atoi(line);
	  break;
	default		:
          (void) alert(0,"%s VOC->%d, (bug?) Unknown keyword <line %d:%s>",
			 OPCN,voc->code,linenum,line);
	  break;
	}
      }
    }
  (void) fclose(fichier);
  if ((cnf->frelais==(char **)0)&&(cnf->relais==(IPTYP **)0)) {
    cnf->relais=net_addiplst(cnf->relais,"127.0.0.1/32");
    }
  if (cnf->bindlst!=(char *)0)
    cnf->bindlst=rou_memstrcat(cnf->bindlst,":");
  cnf->bindlst=rou_memstrcat(cnf->bindlst,"%s%%%s",PROCHKID,cnf->chkidsrv);
  (void) addrelaying(cnf);
  }
else {
  (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
		 OPCN,fname,strerror(errno));
  }
return cnf;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to load the current config,	*/
/*	if the file temporary config file was	*/
/*	changed since last load, the config is	*/
/*	reloaded, otherwise config is kept and 	*/
/*	NULL is returned.			*/
/*						*/
/************************************************/
CNFTYP *cnf_getconfig()

{
static time_t lastload=(time_t)0;
static time_t lastcheck=(time_t)0;

CNFTYP *cnf;
time_t curtime;

cnf=(CNFTYP *)0;
curtime=rou_systime();
if ((lastcheck+10)<curtime) {
  char fname[300];

  (void) rou_mkname(fname,sizeof(fname),d_etc,"",appname,CNF);
  switch (rou_isnewfile(fname,lastload)) {
    case   -1 	:
      (void) alert(0,"gescnf.c:cnf_getconfig, Unable to stat file <%s>",fname);
      break;
    case    0	:
      break;
    case    1	:
      if ((cnf=cnf_readconfig())!=(CNFTYP *)0)
        cnf->lastload=curtime;
      lastload=curtime;
      break;
    }
  lastcheck=curtime;
  }
return cnf;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by	*/
/*	auxiliary information.			*/
/*						*/
/************************************************/
AUXTYP **cnf_freeauxs(AUXTYP **auxs)

{
if (auxs!=(AUXTYP **)0) {
  int i;

  for (i=0;auxs[i]!=(AUXTYP *)0;i++) {
    AUXTYP *dptr;

    dptr=auxs[i];
    dptr->dfs=eml_freedfslst(dptr->dfs);
    dptr->checker=rou_freestr(dptr->checker);
    dptr->sender=rou_freestr(dptr->sender);
    dptr->chkcode=rou_freestr(dptr->chkcode);
    dptr->bindlst=rou_freestrlst(dptr->bindlst);
    (void) free(dptr);
    }
  (void) free(auxs);
  auxs=(AUXTYP **)0;
  }
return auxs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by	*/
/*	domain information.			*/
/*						*/
/************************************************/
CNFTYP *cnf_freeconfig(CNFTYP *config)

{
if (config!=(CNFTYP *)0) {
  config->auxs=cnf_freeauxs(config->auxs);
  config->relais=net_freeiplst(config->relais);
  config->quahead=rou_freestr(config->quahead);
  config->adm=rou_freestrlst(config->adm);
  config->rmponder=rou_freestr(config->rmponder);
  config->dfltcode=rou_freestr(config->dfltcode);
  config->SSLciphrs=rou_freestr(config->SSLciphrs);
  config->chkidmod=rou_freestr(config->chkidmod);
  config->chkidsrv=rou_freestr(config->chkidsrv);
  config->frelais=rou_freestrlst(config->frelais);
  config->bindlst=rou_freestr(config->bindlst);
  config->smtpsrv=rou_freestr(config->smtpsrv);
  config->spmdsrv=rou_freestr(config->spmdsrv);
  (void) free(config);
  config=(CNFTYP *)0;
  }
return config;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to locate the auxiliary name	*/
/*	related to a specific domain.		*/
/*						*/
/************************************************/
AUXTYP *cnf_getaux(AUXTYP **auxs,char *domain,int sndr)

{
AUXTYP *aux;

aux=(AUXTYP *)0;
if (auxs!=(AUXTYP **)0) {
  int i;

  for (i=0;auxs[i]!=(AUXTYP *)0;i++) {
    if (eml_locatedfs(auxs[i]->dfs,domain)!=(DFSTYP *)0) {
      char *ptr;
 
      if (sndr==true)
        ptr=auxs[i]->sender;
      else
        ptr=auxs[i]->checker;
      if (ptr!=(char *)0)
        aux=auxs[i];
      break;
      }
    }
  }
return aux;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to make a merged list of 	*/
/*	sender/checker from the auxiliary list.	*/
/*						*/
/************************************************/
SKRTYP **cnf_getskrlst(AUXTYP **auxs)

{
SKRTYP **lst;

lst=(SKRTYP **)0;
if (auxs!=(AUXTYP **)0) {
  int i;

  for (i=0;auxs[i]!=(AUXTYP *)0;i++) {
    lst=addskr(lst,p_checker,auxs[i]->checker,auxs[i]);
    lst=addskr(lst,p_sender,auxs[i]->sender,auxs[i]);
    }
  }
return lst;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to free memory used by an	*/
/*	skr list				*/
/*						*/
/************************************************/
SKRTYP **cnf_freeskrlst(SKRTYP **skrlst)

{
if (skrlst!=(SKRTYP **)0) {
  int i;

  for (i=0;skrlst[i]!=(SKRTYP *)0;i++) {
    skrlst[i]->bindlst=rou_freestrlst(skrlst[i]->bindlst);
    skrlst[i]->dfs=eml_freedfslst(skrlst[i]->dfs);
    skrlst[i]->webgood=rou_freestr(skrlst[i]->webgood);
    skrlst[i]->chkcode=rou_freestr(skrlst[i]->chkcode);
    skrlst[i]->hostname=rou_freestr(skrlst[i]->hostname);
    (void) free(skrlst[i]);
    }
  (void) free(skrlst);
  skrlst=(SKRTYP **)0;
  }
return skrlst;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to locate an SKR reference	*/
/*	among an skr list			*/
/*						*/
/************************************************/
SKRTYP *cnf_getskr(SKRTYP **skrlst,char *hostname)

{
SKRTYP *found;

found=(SKRTYP *)0;
if (skrlst!=(SKRTYP **)0) {
  int i;

  for (i=0;skrlst[i]!=(SKRTYP *)0;i++) {
    if (strcmp(skrlst[i]->hostname,hostname)==0) {
      found=skrlst[i];
      break;
      }
    }
  }
return found;
}
