/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	GESQUE:					*/
/*	Take care on E-mail queueing/retreiving	*/
/*	process.				*/
/*						*/
/************************************************/
#include	<string.h>
#include	<signal.h>
#include	<malloc.h>
#include	<errno.h>
#include	<ctype.h>
#include	<syslog.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<pwd.h>
#include	<sys/stat.h>
#include	"subrou.h"
#include	"unilck.h"
#include	"unieml.h"
#include	"gesque.h"


/*Application base env variable	name		*/
#define	BASE	"BASE"	
/*Shell script to update user account within	*/
/*virtual domain.				*/
#define	GESPAS	"gespas"
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by one	*/
/*	aliases.				*/
/*						*/
/************************************************/
static ALITYP *freeonealiases(ALITYP *ali)

{
if (ali!=(ALITYP *)0) {
  ali->list=rou_freestrlst(ali->list);
  ali->name=rou_freestr(ali->name);
  (void) free(ali);
  ali=(ALITYP *)0;
  }
return ali;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract list of aliases	*/
/*	related ton one user.			*/
/*						*/
/************************************************/
static char **getonealiases(char *email)

{
char **list;
char *domain;
ALITYP **ali;

list=(char **)0;
domain=eml_strdomain(email);
if ((ali=que_getaliases(domain))!=(ALITYP **)0) {
  int i;
  char *name;
  char *ptr;

  name=strdup(email);
  if ((ptr=strchr(name,'@'))!=(char *)0)
    *ptr='\000';
  for (i=0;ali[i]!=(ALITYP *)0;i++) {
    if (strcasecmp(name,ali[i]->name)==0) {
      int j;
      ALITYP *pa;	

      pa=ali[i];
      for (j=0;pa->list[j]!=(char *)0;j++) {
	list=(char **)rou_addveclst((void **)list,(void *)strdup(pa->list[j]));
	}
      break;
      }
    }
  ali=que_freealiases(ali);
  (void) free(name);
  }
return list;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if an Email is	*/
/*	already within a list of E-mail.	*/
/*						*/
/************************************************/
static int presente(char **done,char *email)

{
int found;

found=false;
if (done!=(char **)0) {
  int i;

  for (i=0;done[i]!=(char *)0;i++) {
    if (strcasecmp(done[i],email)==0) {
      found=true;
      break;
      }
    }
  }
return found;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan alias list, removing	*/
/*	duplicate and avoiding aliases loop.	*/
/*						*/
/************************************************/
static int scanaliases(char *email,char ***done,char ***list)

{
char **loc;
int terminal;

terminal=true;
*done=(char **)rou_addveclst((void **)(*done),(void *)strdup(email));
if ((loc=getonealiases(email))!=(char **)0) {
  int i;

  terminal=false;
  for (i=0;loc[i]!=(char *)0;i++) {
    int toadd;

    toadd=false;
    if (presente(*done,loc[i])==true) {
      toadd=true;
      *list=(char **)rou_addveclst((void **)(*list),(void *)strdup(loc[i]));
      }    
    else {
      toadd=scanaliases(loc[i],done,list);
      }
    if (toadd==true) {
      *list=(char **)rou_addveclst((void **)(*list),(void *)strdup(loc[i]));
      }
    }
  loc=rou_freestrlst(loc);
  }
return terminal;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan aliases information	*/
/*						*/
/************************************************/
static void getaliases(ALITYP *ali,char *domain,char *list)

{
if (strlen(list)>0) {
  char *wrk;

  wrk=calloc(strlen(list)+strlen(domain)+10,sizeof(char));
  while (list!=(char *)0) {
    char *ptr;

    if ((ptr=strchr(list,','))!=(char *)0) {
      *ptr='\000';
       ptr++;
      }
    if (strlen(list)>0) {
      (void) strcpy(wrk,list);
      if (strchr(list,'@')==(char *)0) {
        (void) strcat(wrk,"@");
        (void) strcat(wrk,domain);
        }
      ali->list=(char **)rou_addveclst((void **)ali->list,(void *)strdup(wrk));
      }
    list=ptr;
    }
  (void) free(wrk);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to dump aliases information	*/
/*	within the aliases file.		*/
/*						*/
/************************************************/
static void dumpaliases(char *domain,ALITYP **lists)

{
#define OPDA	"gesque.c:dumpaliases,"

FILE *fichier;
char dir[300];
char filename[400];
char newname[400];

(void) rou_mkname(dir,sizeof(dir),d_spool,MAILDIR,domain,"");
(void) snprintf(filename,sizeof(filename),"%s/%s",dir,ALIASNAME);
(void) snprintf(newname,sizeof(newname),"%s.new",filename);
if ((fichier=fopen(newname,"w"))!=(FILE *)0) {
  if (lists!=(ALITYP **)0) {
    int i;

    for (i=0;lists[i]!=(ALITYP *)0;i++) {
      char *outline;
      char **list;
      int j;

      outline=rou_memstrcat((char *)0,"%s:\t",lists[i]->name);
      list=lists[i]->list;
      for (j=0;list[j]!=(char *)0;j++) {
	char *sep;

	sep=",";
	if (list[j+1]==(char *)0)
	  sep="";
        outline=rou_memstrcat(outline,"%s%s",list[j],sep);
	if (strlen(outline)>60) {
	  (void) fprintf(fichier,"%s\n",outline);
	  outline=rou_freestr(outline);
	  if (list[j+1]!=(char *)0)
            outline=rou_memstrcat((char *)0,"\t");
	  }
        }
      if (outline!=(char *)0) {
	(void) fprintf(fichier,"%s\n",outline);
	outline=rou_freestr(outline);
	}
      }
    }
  (void) fclose(fichier);
  (void) unlink(filename);
  if (link(newname,filename)<0)
    (void) alert(0,"%s Unable to link file <%s> to <%s> (error=<%s>)",
		    OPDA,newname,filename,strerror(errno));
  else
    (void) unlink(newname);
  }
else 
  (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
		  OPDA,newname,strerror(errno));
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to make an alias information	*/
/*	from a very long line			*/
/*	Line format:				*/
/*	E-mail:	aliases_1,alias_2,....		*/
/*						*/
/************************************************/
static ALITYP *mkaliases(char *entry,char *domain)

{
ALITYP *aliase;
char *ptr;

aliase=(ALITYP *)0;
(void) que_cleanstring(entry);
if ((ptr=strchr(entry,':'))!=(char *)0) {
  *ptr='\000';
  ptr++;
  aliase=(ALITYP *)calloc(1,sizeof(ALITYP));
  aliase->name=strdup(entry);
  (void) que_standardize(ptr);
  (void) getaliases(aliase,domain,ptr);
  }
return aliase;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to 'compute' the password	*/
/*	file full path.				*/
/*						*/
/************************************************/
static FILE *openpass(char *domain,int create)

{
FILE *fichier;
char maildir[400];
char passname[400];

fichier=(FILE *)0;
(void) rou_mkname(maildir,sizeof(maildir),d_spool,MAILDIR,domain,"");
(void) snprintf(passname,sizeof(passname),"%s/%s",maildir,PASSNAME);
if (((fichier=fopen(passname,"r"))==(FILE *)0)&&(create==true)) {
  if ((fichier=fopen(passname,"w"))!=(FILE *)0) {
    (void) fclose(fichier);
    fichier=fopen(passname,"r");
    }
  else {
    (void) alert(0,"gesque.c:openpass, Unable to create <%s> (config?)",
		    passname);
    }
  }
return fichier;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate a user passwd record*/
/*	return null if not found.		*/
/*	No need to free memory space returned	*/
/*	by fgetpwent.				*/
/*						*/
/************************************************/
static struct passwd *getpassrec(FILE *fichier,char *email)

{
struct passwd *goodpw;
struct passwd *pw;
char *ptr;
char username[200];


goodpw=(struct passwd *)0;
(void) memset(username,'\000',sizeof(username));
(void) strncpy(username,email,sizeof(username)-1);
if ((ptr=strchr(username,'@'))!=(char *)0)
  *ptr='\000';
while ((pw=fgetpwent(fichier))!=(struct passwd *)0) {
  if (strcmp(pw->pw_name,username)==0) {
    goodpw=pw;
    break;    
    }
  }
return goodpw;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to link an E-mail within the	*/
/*	queue directory to the end-user POP	*/
/*	directory.				*/
/*	called if dovecot delivery is not	*/
/*	available.				*/
/*						*/
/************************************************/
int storeemails(char **rcpts,char *emailpath,char *emailname)

{
int done;

done=false;
if (rcpts!=(char **)0) {
  int i;

  done=true;
  for (i=0;rcpts[i]!=(char *)0;i++) {
    char *username;
    char *dom;

    username=strdup(rcpts[i]);
    if ((dom=strchr(username,'@'))!=(char *)0) {
      char dir[300];
      char dest[300];

      *dom='\000';
      dom++;
      (void) rou_mkname(dest,sizeof(dest),d_spool,MAILDIR,dom,"");
      /*"new" is the directory dovecot in "Maildir Mode" is	*/
      /*expecting new E-mail to be stored.			*/
      (void) snprintf(dir,sizeof(dir),"%s/%s/new",dest,username);
      (void) rou_createdirs(dir);
      (void) snprintf(dest,sizeof(dest),"%s/%s",dir,emailname);
      if (link(emailpath,dest)<0) {
	(void) alert(0,"gesque.c:storeemails, Unable to link <%s> to <%s> "
		       "(error=<%s>)",emailpath,dest,strerror(errno));
	done=false;
        }
      }
    (void) free(username);
    }
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract PASTYP information	*/
/*	from a line buffer.			*/
/*						*/
/************************************************/
PASTYP *que_getusrinfo(char *buffer)

{
PASTYP *pi;

pi=(PASTYP *)calloc(1,sizeof(PASTYP));
pi->domain=(char *)calloc(strlen(buffer),sizeof(char));
pi->userid=(char *)calloc(strlen(buffer),sizeof(char));
pi->username=(char *)calloc(strlen(buffer),sizeof(char));
pi->passwd=(char *)calloc(strlen(buffer),sizeof(char));
if (sscanf(buffer,"%s %s %s '%[^'\n\r']",pi->domain,
					 pi->userid,
					 pi->passwd,
					 pi->username)>=3) {
  (void) que_standardize(pi->domain); 
  (void) que_standardize(pi->userid); 
  }
else {
  pi=que_freeusrinfo(pi);
  }
return pi;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add an E-mail entry within */
/*	the domain .passwd file.		*/
/*	Checking if the domain is already	*/
/*	existing and user is existing		*/
/*						*/
/************************************************/
int mngemails(MNGTYP action,PASTYP *pi)

{
#define OPME	"gesque.c:addemails,"

int done;
char *act;
char lockname[100];
char dir[300];

done=false;
act=(char *)0;
switch (action) {
  case m_add	:
    act=ADDUSR;
    break;
  case m_rem	:
    act=REMUSR;
    break;
  case m_del	:
    act=DELDOM;
    break;
  case m_unk	:
  default	:
    break;
  }
if (act==(char *)0)
  return done;	/*Hmmm trouble trouble				*/
(void) snprintf(lockname,sizeof(lockname),"%s-%s",pi->domain,PASSNAME);
(void) rou_mkname(dir,sizeof(dir),d_spool,MAILDIR,pi->domain,"");
if (lck_locking(lockname,true,5)==true) {
  FILE *fichier;

  if (rou_createdirs(dir)==true) {
    if ((fichier=openpass(pi->domain,true))!=(FILE *)0) {
      (void) fclose(fichier);
      }
    }
  if ((strlen(pi->userid)>0)&&(strlen(pi->passwd)>0)) {
    char *ptr;
    char list[250];

    while ((ptr=strchr(pi->username,':'))!=(char *)0)
       *ptr=' ';
    ptr=pi->passwd;
    (void) snprintf(list,sizeof(list),"%s/%s-%05d","/tmp",appname,getpid());
    if ((fichier=fopen(list,"w"))!=(FILE *)0) {
      char shell[200];
      char commande[300];

      (void) chmod(list,S_IRUSR|S_IWUSR|S_IRGRP);
      (void) fprintf(fichier,"%s:%s:%s\n",pi->userid,ptr,pi->username);
      (void) fclose(fichier);
      (void) setenv(BASE,base,1);
      (void) rou_mkname(shell,sizeof(shell),d_usrlib,UTILDIR,GESPAS,".sh");
      (void) snprintf(commande,sizeof(commande),"cat %s | %s %s %s",
						 list,shell,act,pi->domain);
      if (system(commande)<0)
        (void) alert(0,"%s Unable to execute commande <%s>",OPME,commande);
      (void) usleep(10000);
      (void) unlink(list);
      }
    else {
      (void) alert(0,"%s Unable to creat password file for <%s> (error=<%s>)",
		      OPME,pi->domain,strerror(errno));
      }
    }
  (void) lck_locking(lockname,false,1);
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a	*/
/*	PASTYP structure.			*/
/*						*/
/************************************************/
PASTYP *que_freeusrinfo(PASTYP *pi)

{
if (pi!=(PASTYP *)0) {
  pi->domain=rou_freestr(pi->domain);
  pi->userid=rou_freestr(pi->userid);
  pi->username=rou_freestr(pi->username);
  pi->passwd=rou_freestr(pi->passwd);
  (void) free(pi);
  pi=(PASTYP *)0;
  }
return pi;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return a allocate a new 	*/
/*	memory space, with the user crypted	*/
/*	password.				*/
/*						*/
/************************************************/
char *que_getpasswd(char *email)

{
char *crypted;
char *domain;
FILE *fichier;

crypted=(char *)0;
domain=eml_strdomain(email);
if ((fichier=openpass(domain,false))!=(FILE *)0) {
  struct passwd *pw;

  if ((pw=getpassrec(fichier,email))!=(struct passwd *)0) {
    crypted=strdup(pw->pw_passwd);
    }
  (void) fclose(fichier);
  }
return crypted;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to link an E-mail within the	*/
/*	queue directory to the end-user POP	*/
/*	directory.				*/
/*						*/
/************************************************/
int que_storeemails(char *emailorg,char **rcpts,char *emailpath,char *emailname)

{
#define	OPQS	"gesque.c:que_storeemails,"

/*dovecot delivery program			*/
#define	DOVDEL	"/usr/libexec/dovecot/deliver"

static	int dovemissing=false;

int done;
FILE *femail;
int proceed;
int phase;

done=false;
femail=(FILE *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*dove missing?	backup	*/
      if (dovemissing==true) {
	done=storeemails(rcpts,emailpath,emailname);
	phase=999;	/*backup proceeding done*/
	}
      break;
    case 1	:	/*openning email file	*/
      if ((femail=fopen(emailpath,"r"))==(FILE *)0) {
	(void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
			OPQS,emailpath,strerror(errno));
        (void) journalling(true,EVJRL,"Alert Unable to post mail <%s>",emailpath);
	phase=999;	/*trouble!		*/
 	}
      break;
    case 2	:	/*dove missing?	backup	*/
      if (rcpts!=(char **)0) {
	int i;

	done=true;
  	for (i=0;(rcpts[i]!=(char *)0)&&(done==true);i++) {
	  int dummycnt;
	  int ok;
	  FILE *canal;
	  char *pipename;
	  char buffer[500];
	
	  dummycnt=asprintf(&pipename,"%s -f %s -d %s",DOVDEL,emailorg,rcpts[i]);
	  if ((canal=popen(pipename,"w"))==(FILE *)0) {
	    done=false;
	    dovemissing=true;
            (void) journalling(true,APJRL,"Unable to popen <%s> (error=<%s>)",
					  pipename,strerror(errno));
	    break;	/*no need to proceed further	*/
	    }
	  (void) rewind(femail);
	  while (fgets(buffer,sizeof(buffer),femail)!=(char *)0) 
	    (void) fputs(buffer,canal);
	  ok=pclose(canal);
	  switch (ok) {
	    case -1	:
	      (void) alert(0,"%s pclose file <%s> (error=<%s>)",
			      OPQS,pipename,strerror(errno));
	      done=false;
	      break;
	    case  0	:
	      break;
	    default	:
	      ok/=256;
              (void) journalling(true,APJRL,"Unable to store file <%s> via <%s> to <%s> (pclose ret='%d')",
					     emailpath,pipename,rcpts[i],ok);
	      (void) alert(0,"%s pclose file <%s> (error='%d')",OPQS,pipename,ok);
	      done=false;
	      break;
	    }
	  (void) free(pipename);
	  }
	}
      break;
    case 3	:	/*closing input Email.	*/
      (void) fclose(femail);
      break;
    default	:	/*SAFE Guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure remove any ending \n or \r at	*/
/*	an E-mail string's end.			*/
/*						*/
/************************************************/
char *que_cleanstring(char *string)

{
if (string!=(char *)0) {
  register int taille;
  register char *ptr;

  taille=strlen(string)-1;
  ptr=string+taille;
  while ((*ptr=='\n') || (*ptr=='\r')) {
    *ptr='\000';
    ptr--;
    taille--;
    if (taille<0)
      break;
    }
  }
return string;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to remove duplicate space and	*/
/*	lowercase the string contents		*/
/*						*/
/************************************************/
char *que_standardize(char *str)

{
register char *ptr1;
register char *ptr2;

for (ptr1=str,ptr2=str;*ptr1!='\000';ptr1++) {
  if (isblank(*ptr1)!=0) {
    switch (*(ptr1+1)) {
      case ' '	  :
      case '\t'	  :
      case '\000' :
	break;
      default	  :
	if (ptr2!=str) {
          *ptr2=*ptr1;
          ptr2++;
	  }
	break;
      }
    }
  else {
    *ptr2=(char)tolower(*ptr1);
    ptr2++;
    }
  }
*ptr2='\000';
return str;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to create a list of E-mail	*/
/*	from a Q file contents.			*/
/*						*/
/************************************************/
MLTYP *que_getemails(char *queudir,char *emailid)

{
#define	OPGE	"gesque.c:getemails,"

typedef	enum	{
	e_relay,	/*Email is relayable	*/
	e_spam,		/*Email is a spam	*/
	e_from,		/*E-mail from		*/
	e_rcpt,		/*E_mail rcpt		*/
	e_subject,	/*email subject		*/
	e_quit,		/*Queue directive end	*/
	e_unknown	/*Unknown data		*/
	}ENUML;

static VOCTYP vocable[]={
		{e_relay,SRELAY,(const void *)0},
		{e_spam,SPAM,(const void *)0},
		{e_from,SMAILF,(const void *)0},
		{e_rcpt,SRCPTT,(const void *)0},
		{e_subject,SUBJECT,(const void *)0},
		{e_quit,SQUIT,(const void *)0},
		{e_unknown,(char *)0,(const void *)0}
		};

MLTYP *ml;
FILE *file;
char emailqueue[300];

ml=(MLTYP *)0;
(void) rou_mkname(emailqueue,sizeof(emailqueue),d_spool,
					        queudir,emailid,QUEUEEXT);
if ((file=fopen(emailqueue,"r"))!=(FILE *)0) {
  char line[512];

  ml=(MLTYP *)calloc(1,sizeof(MLTYP));
  while (fgets(line,sizeof(line),file)!=(char *)0) {
    VOCTYP *voc;

    (void) que_cleanstring(line); 
    if ((voc=rou_getvoca(vocable,line))!=(VOCTYP *)0) {
      switch(voc->code) {
        case e_relay	:
	  ml->relayed=atoi(line);
	  break;
        case e_spam	:
	  ml->spam=true;
	  ml->spamperti=strdup(line);
	  break;
        case e_from	:
	  if (ml->from==(char *)0) {
	    ml->from=strdup(line);
	    }
	  break;
        case e_rcpt	:
	   ml->rcpts=(char **)rou_addveclst((void **)ml->rcpts,
					    (void *)strdup(line));
	  break;
        case e_subject	:
	  if (ml->subject==(char *)0) {
	    ml->subject=strdup(line);
	    }
	  break;
	case e_quit	:
	  break;
	default		:
          (void) alert(0,"%s VOC->%d, (bug?) can't reach keyword "
			 "file <%s>, line <%s>",
			 OPGE,voc->code,emailqueue,line);
	  break;
	}
      }
    }
  (void) fclose(file);
  }
else
  (void) alert(0,"%s Unable to open file <%s> (error=<%s>)",
		  OPGE,emailqueue,strerror(errno));
if (ml->from==(char *)0)
  ml=que_dropemails(ml);
return ml;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by an	*/
/*	MLTYP structure.			*/
/*						*/
/************************************************/
MLTYP *que_dropemails(MLTYP *ml)

{
if (ml!=(MLTYP *)0) {
  ml->rcpts=rou_freestrlst(ml->rcpts);
  ml->from=rou_freestr(ml->from);
  ml->subject=rou_freestr(ml->subject);
  ml->spamperti=rou_freestr(ml->spamperti);
  (void) free(ml);
  ml=(MLTYP *)0;
  }
return ml;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if a domain is known	*/
/*	locally.				*/
/*						*/
/************************************************/
int que_chkdomain(char *domain)

{
int found;
struct stat bufstat;
char maildir[400];

found=false;
(void) rou_mkname(maildir,sizeof(maildir),d_spool,MAILDIR,domain,"");
if (stat(maildir,&bufstat)==0) {
  if (S_ISREG(bufstat.st_mode)==0)
    found=true;
  } 
return found;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if an E_mail is	*/
/*	locally existing. Used in case E-mail	*/
/*	are store locally (keeplocal) by clement*/
/*	Return a exiting status.		*/
/*						*/
/************************************************/
int que_chkemails(char *email)

{
int chkerr;
char *domain;
int proceed;
int phase;

chkerr=ERRPROC;
domain=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*scanning request		*/
      domain=eml_strdomain(email);
      break;
    case  1	:	/*check if we still have a user	*/
      if (strlen(domain)>0) {
	FILE *fichier;

        chkerr=NORELAY;
	if ((fichier=openpass(domain,false))!=(FILE *)0) {
          chkerr=REJECT;
	  if (getpassrec(fichier,email)!=(struct passwd *)0)
	    chkerr=CMDOK;
	  (void) fclose(fichier);
          }
	}
      break;
    default	:	/*safe guard			*/
      proceed=false;
      break;
    }
  phase++;
  }
return chkerr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add an E-mail entry within */
/*	the domain .passwd file.		*/
/*	Checking if the domain is already	*/
/*	existing and user is existing		*/
/*						*/
/************************************************/
int que_mngemails(MNGTYP action,PASTYP *pi)

{
#define OPMN	"gesque.c:que_addemails,"

int done;

done=false;
switch (fork()) {
  case -1	:
    (void) alert(0,"%s unable to fork (error=<%s>)",OPMN,strerror(errno));
    break;
  case  0	:
    (void) mngemails(action,pi);
    (void) exit(0);
    break;
  default	:
    done=true;
    (void) usleep(10000);
    break;
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to move the full domain	*/
/*	directory to a domain.back directory.	*/
/*	if domain.back directory exist, the back*/
/*	domain is rename and removed.		*/
/*						*/
/************************************************/
int que_deldom(char *domain)

{
#define OPDL	"gesque.c:que_deldom,"

int done;
int phase;
int proceed;
struct stat bufstat;
char lockname[100];
char dir[300];
char back[400];

done=false;
phase=0;
proceed=true;
(void) snprintf(lockname,sizeof(lockname),"%s-%s",domain,PASSNAME);
(void) rou_mkname(dir,sizeof(dir),d_spool,MAILDIR,domain,"");
if (stat(dir,&bufstat)<0) {
  (void) alert(0,"%s directory <%s> no existing",OPDL,dir);
  proceed=false;
  }
while (proceed==true) {
  switch (phase) {
    case  0	:	/*set lock		*/
      proceed=lck_locking(lockname,true,5);
      break;
    case  1	:	/*is directory existing	*/
      (void) snprintf(back,sizeof(back),"%s.back",dir);
      if (stat(back,&bufstat)==0) {
	char todel[400];

	/*lets delete all previous back directory	*/
        (void) snprintf(todel,sizeof(todel),"%s.%05d",dir,getpid());
	(void) rename(back,todel);
	switch (fork()) {
	  case -1	:
	    break;
	  case  0	:
	    (void) snprintf(back,sizeof(back),"rm -fr %s",todel);
	    if (system(back)<0)
              (void) alert(0,"%s unable to execute command <%s>",OPDL,back);
	    (void) exit(0);
	    break;
	  default	:
	    (void) usleep(10000);
	    break;
          }
	}
      break;
    case  2	:	/*moving All domain dir to back	*/
      if (rename(dir,back)<0) 
        (void) alert(0,"%s unable to rename directory <%s> to <%s> "
		       "(error=<%s>)",OPDL,dir,back,strerror(errno));
      else 
	done=true;
      break;
    default	:	/*SAFE Guard, phase end		*/
      (void) lck_locking(lockname,false,1);
      proceed=false;
      break;
    }
  phase++;
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add an E-mail aliases entry*/
/*	within the domain .passwd file.		*/
/*						*/
/************************************************/
int que_mngaliases(MNGTYP action,char *domain,char *name,char *aliases)

{
#define OPAL	"gesque.c:que_mngaliase,"

int done;
char lockname[100];
char dir[300];

done=false;
(void) snprintf(lockname,sizeof(lockname),"%s-%s",domain,ALIASNAME);
(void) rou_mkname(dir,sizeof(dir),d_spool,MAILDIR,domain,"");
if (lck_locking(lockname,true,5)==true) {
  ALITYP **lists;

  lists=que_getaliases(domain);
  if ((strlen(name)>0)&&(strlen(aliases)>0)) {
    char *newlist;
    ALITYP *newone;

    newlist=rou_memstrcat((char *)0,"%s: %s",name,aliases);
    if ((newone=mkaliases(newlist,domain))!=(ALITYP *)0) {
      if (lists!=(ALITYP **)0) {
	int i;

	for (i=0;lists[i]!=(ALITYP *)0;i++) {
	  if (strcmp(lists[i]->name,newone->name)==0) {
	    (void) freeonealiases(lists[i]);
	    if (action==m_rem) {
	      int j;

	      for (j=i;lists[j]!=(ALITYP *)0;j++)
	        lists[j]=lists[j+1];
	      newone=freeonealiases(newone);
	      }
	    else
	      lists[i]=newone;
	    newone=(ALITYP *)0;
	    break;
	    }
	  }	
        }
      if (newone!=(ALITYP *)0)
	lists=(ALITYP **)rou_addveclst((void **)lists,(void *)newone);
      }
    (void) dumpaliases(domain,lists);
    lists=que_freealiases(lists);
    newlist=rou_freestr(newlist);
    }
  lists=que_freealiases(lists);
  (void) lck_locking(lockname,false,1);
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to build a list of aliases for*/
/*	a specific domain.			*/
/*						*/
/************************************************/
ALITYP **que_getaliases(char *domain)

{
ALITYP **lists;
FILE *fichier;
char dir[300];
char filename[400];

lists=(ALITYP **)0;
(void) rou_mkname(dir,sizeof(dir),d_spool,MAILDIR,domain,"");
(void) snprintf(filename,sizeof(filename),"%s/%s",dir,ALIASNAME);
if ((fichier=fopen(filename,"r"))!=(FILE *)0) {
  ALITYP *cural;
  char line[1000];

  cural=(ALITYP *)0;
  while (rou_getstr(fichier,line,sizeof(line),'#')!=(char *)0) {
    switch (line[0]) {
      case '\t'	:
      case ' '	:
	(void) que_standardize(line);
	if (cural!=(ALITYP *)0) 
	  (void) getaliases(cural,domain,line);
        break;
      default	:
	cural=mkaliases(line,domain);
	lists=(ALITYP **)rou_addveclst((void **)lists,(void *)cural);	
	break;
      }
    }
  (void) fclose(fichier);
  }
return lists;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by an	*/
/*	Aliases list.				*/
/*						*/
/************************************************/
ALITYP **que_freealiases(ALITYP **lists)

{
if (lists!=(ALITYP **)0) {
  int i;

  for (i=0;lists[i]!=(ALITYP *)0;i++) 
    lists[i]=freeonealiases(lists[i]);
  (void) free(lists);
  lists=(ALITYP **)0;
  }
return lists;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract list of aliases	*/
/*	related ton one user.			*/
/*	If not one aliase is found, return	*/
/*	pointer is NULL.			*/
/*						*/
/************************************************/
char **que_getonealiases(char *email)

{
char **list;
char **done;

list=(char **)0;
done=(char **)0;
(void) scanaliases(email,&done,&list);
done=rou_freestrlst(done);
return list;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to recall an E_mail previously*/
/*	stored in quarantine area.		*/
/*						*/
/************************************************/
int que_mngquarantine(int journal,char *email,char *emid,int efface)

{
#define	OPRE	"gesque.c:que_mngquarantine,"
#define	TDIR	300

int statut;
FILE *filein;
FILE *fileout;
long reading;
long writing;
char *domain;
char *linkfrom;
char *linkto;
char *lisqin;
char *lisqlck;
char *lisqout;
char *fname;
char *mqdir;
char *username;
int phase;
int proceed;
char line[500];

statut=true;
filein=(FILE *)0;
fileout=(FILE *)0;
reading=0;
writing=0;
domain=(char *)0;
linkfrom=(char *)0;
linkto=(char *)0;
linkto=(char *)0;
lisqin=(char *)0;
lisqout=(char *)0;
lisqlck=(char *)0;
fname=(char *)calloc(TDIR,sizeof(char));
(void) rou_mkname(fname,TDIR,d_spool,TRIADIR,"","");
mqdir=(char *)calloc(TDIR,sizeof(char));
(void) rou_mkname(mqdir,TDIR,d_spool,QUEUDIR,"","");
username=strdup(email);
phase=1;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*error phase			*/
      statut=0;
      proceed=false;
      break;
    case  1	:	/*extracting domain		*/
      if ((domain=strchr(username,'@'))!=(char *)0) {
        *domain='\000';
	domain++;
	}
      else {
	(void) tracing(TRALOG,0,"%s <%s> wrong format",OPRE,username);
	phase=-1;
	}
      break;
    case  2	:	/*making filename		*/
      fname=rou_memstrcat(fname,"/%s/%s",domain,username);
      break;
    case  3	:	/*locking .lisq filename	*/
      lisqin=rou_memstrcat((char *)0,"%s/%s",fname,LISQ);
      lisqlck=rou_memstrcat((char *)0,"%s.lock",lisqin);
      lisqout=rou_memstrcat((char *)0,"%s.new",lisqin);
      if (link(lisqin,lisqlck)<0) {
	(void) alert(0,"%s Unable to link <%s> (status=<%s>)",
			OPRE,lisqlck,strerror(errno));
	phase=9999;	/*getting out fast		*/
        }
      break;
    case  4	:	/*opening .lisq file contents	*/
      if ((filein=fopen(lisqin,"r"))==(FILE *)0) {
	(void) alert(0,"%s Unable to open <%s> (status=<%s>)",
			OPRE,lisqin,strerror(errno));
	phase=9999;	/*getting out fast		*/
        }
      break;
    case  5	:	/*opening .lisq.new to do copy	*/
      if ((fileout=fopen(lisqout,"w"))==(FILE *)0) {	
	(void) fclose(filein);
	(void) alert(0,"%s Unable to open <%s> (status=<%s>)",
			OPRE,lisqout,strerror(errno));
	phase=9999;	/*getting out fast		*/
	}
      break;
    case  6	:	/*duplicating contents		*/
      while (fgets(line,sizeof(line),filein)!=(char *)0) {
	char curid[100];

	if (sscanf(line,"%[^|]",curid)==1) {
	  if (strcmp(curid,emid)!=0) {
	    (void) fprintf(fileout,"%s",line);
	    }
	  else {
	    if (journal==true) {
	      char *ptrstr;

              ptrstr="Sending";
	      if (efface==true)
                ptrstr="Removing";
              (void) journalling(true,EVJRL,"%s request from <%s> "
					    "\n%-9s about file %s",
				    	     ptrstr,email,"",curid);
	      }
	    }
	  }
        }
      (void) fclose(fileout);
      (void) fclose(filein);
      break;
    case  7	:	/*mv new file to be  .lisq file	*/
      (void) unlink(lisqin);
      if (link(lisqout,lisqin)<0) 
        (void) alert(0,"%s Unable to link file name <%s> to name <%s>",
		        OPRE,lisqout,lisqin);
      (void) unlink(lisqout);
      break;
    case  8	:	/*moving first QUEUEXT		*/
      linkfrom=rou_memstrcat((char *)0,"%s/%s%s",fname,emid,QUEUEEXT);
      linkto=rou_memstrcat((char *)0,"%s/%s%s",mqdir,emid,QUEUEEXT);
      if (efface==false) {	/*moving the file	*/
        if (link(linkfrom,linkto)<0)
	  (void) tracing(TRALOG,0,"%s Unable to hard_link <%s> "
				     "to <%s> (error=<%s>)",
				     OPRE,linkfrom,linkto,strerror(errno));
	}
      if (unlink(linkfrom)<0)
	(void) tracing(TRALOG,0,"%s Unable to unlink <%s> (error=<%s>)",
				 OPRE,linkfrom,strerror(errno));
      linkto=rou_freestr(linkto);
      linkfrom=rou_freestr(linkfrom);
      break;
    case  9	:	/*moving first MAILEXT		*/
      linkfrom=rou_memstrcat((char *)0,"%s/%s%s",fname,emid,EMAILEXT);
      linkto=rou_memstrcat((char *)0,"%s/%s%s",mqdir,emid,EMAILEXT);
      if (efface==false) {	/*moving the file	*/
        if (link(linkfrom,linkto)<0)
	  (void) tracing(TRALOG,0,"%s Unable to hard_link <%s> "
				     "to <%s> (error=<%s>)",
				     OPRE,linkfrom,linkto,strerror(errno));
        }
      if (unlink(linkfrom)<0)
	(void) tracing(TRALOG,0,"%s Unable to unlink <%s> (error=<%s>)",
				   OPRE,linkfrom,strerror(errno));
      linkto=rou_freestr(linkto);
      linkfrom=rou_freestr(linkfrom);
      break;
    default	:	/*everything done, removing lock	*/
      (void) unlink(lisqlck);
      lisqout=rou_freestr(lisqout);
      lisqlck=rou_freestr(lisqlck);
      lisqin=rou_freestr(lisqin);
      proceed=false;
      break;
    }
  phase++;
  }
fname=rou_freestr(fname);
username=rou_freestr(username);
return statut;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to copy a file from /base/tmp	*/
/*	directory to a specicfic clement	*/
/*	directory.				*/
/*	Return a status string.			*/
/*						*/
/************************************************/
const char *que_copy(char *command)

{
#define	OPCP	"gesque.c:que_copy,"

int status;
const char *retour;
uid_t gid;
uid_t uid;
char *orig;
char *dest;
FILE *forig;
FILE *fdest;
int phase;
int proceed;
char norig[400];
char ndest[400];
char nfin[400];
char line[500];

status=ERRPROC;
retour=eml_ascstat(status,"unknown error");
gid=getegid();	/*lets be prepare to be back as root if needed	*/
uid=geteuid(); 	/*and to change group ownership			*/
orig=(char *)0;
dest=(char *)0;
forig=(FILE *)0;
fdest=(FILE *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case 0	:	/*lets extract origin/destination	*/
      orig=calloc(strlen(command),sizeof(char));
      dest=calloc(strlen(command),sizeof(char));
      if (sscanf(command,"%s %s",orig,dest)!=2) {
	retour=eml_ascstat(BADPAR,"wrong copy parameters");
	phase=999;	/*get out fast				*/
	}
      break;
    case 1	:	/*lets check about destination		*/
      if (strcasecmp(dest,"etc")==0) {
        (void) rou_mkname(ndest,sizeof(ndest),d_etc,"",orig,".new");
        (void) rou_mkname(nfin,sizeof(nfin),d_etc,"",orig,"");
	break;
        }	
      if (strcasecmp(dest,"spool")==0) {
        (void) rou_mkname(ndest,sizeof(ndest),d_spool,MAILDIR,orig,".new");
        (void) rou_mkname(nfin,sizeof(nfin),d_spool,MAILDIR,orig,"");
	break;
	}
      retour=eml_ascstat(BADPAR,"unknown destination");
      phase=999;	/*get out fast				*/
      break;
    case 2	:	/*lets open destination file		*/
      if ((fdest=fopen(ndest,"w"))==(FILE *)0) {
	(void) alert(0,"%s Unable to open <%s> (error=<%s>)",
			OPCP,ndest,strerror(errno));
        retour=eml_ascstat(BADPAR,"Unable to open destination file");
        phase=999;	/*get out fast				*/
	}
      break;
    case 3	:	/*taking back root capability		*/
      if (setegid(getgid())<0) {
	(void) alert(0,"%s Unable to set the Egid to '%d'",OPCP,getgid());
	}
      if (seteuid(getuid())<0) {
	(void) alert(0,"%s Unable to set the Euid to '%d'",OPCP,getuid());
	}
      break;
    case 4	:	/*lets set protection and group		*/
      if (rou_mkname(norig,sizeof(norig),d_tmp,"",orig,"")!=(char *)0){
	int curphase;
	struct stat bufstat;

	curphase=phase;
	phase=999;
        retour=eml_ascstat(BADPAR,"Unable to open destination file");
	if (stat(norig,&bufstat)<0) {
	  (void) alert(0,"%s Unable to stat <%s> (error=<%s>)",
			  OPCP,norig,strerror(errno));
	  break;	/*get out fast				*/
	  }
  	if (S_ISREG(bufstat.st_mode)==0) {
	  (void) alert(0,"%s File <%s> is not regular (error=<%s>)",
			  OPCP,norig,strerror(errno));
	  break;	/*get out fast				*/
	  }
	if (chmod(ndest,bufstat.st_mode|S_IRUSR|S_IWUSR)<0) {
	  (void) alert(0,"%s Unable to chmod <%s> (error=<%s>)",
			  OPCP,ndest,strerror(errno));
	  }
	if (chown(ndest,-1,bufstat.st_gid)<0) {
	  (void) alert(0,"%s Unable to chgrp <%s> (error=<%s>)",
			  OPCP,ndest,strerror(errno));
	  break;	/*get out fast				*/
	  }
        phase=curphase;
	}
      else {
	(void) alert(0,"%s, Unable to assign 'forig' from command <%s>",
			"gesque.c:que_copy,",command);
	phase=999;	/*get out fast				*/
	}
      break;
    case 5	:	/*lets open origin file			*/
      if ((forig=fopen(norig,"r"))==(FILE *)0) {
        retour=eml_ascstat(BADPAR,"Unable to open origin file");
        phase=999;	/*get out fast				*/
	}
      break;
    case 6	:	/*let duplicate file			*/
      while (fgets(line,sizeof(line),forig)!=(char *)0) {
	(void) fprintf(fdest,"%s",line);
	}
      (void) fclose(fdest);
      (void) fclose(forig);
      break;
    case 7	:	/*let duplicate file			*/
      (void) unlink(nfin);
      if (link(ndest,nfin)<0) {
        retour=eml_ascstat(BADPAR,"Unable to link destination file");
	}
      else {
        (void) unlink(ndest);
        retour=eml_ascstat(CMDOK,"");
	}
      break;
    default	:	/*SAFE guard				*/
      (void) setegid(gid);
      (void) seteuid(uid);
      dest=rou_freestr(dest);
      orig=rou_freestr(orig);
      proceed=false;
      break;
    }
  phase++;
  }
return retour;
}
