/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	LVLUID:					*/
/*	checkid subroutine level		*/
/*						*/
/************************************************/
#include	<malloc.h>
#include	<string.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<security/pam_appl.h>
#include	"subrou.h"
#include	"unieml.h"
#include	"gestls.h"
#include	"lvluid.h"

#define	PASSWD		"passwd"

static char *locpass;	/*current checked passwd*/
static int pam_status;	/*PAM current status	*/
/*

*/
/************************************************/
/*						*/
/*	Routine called to get password as it	*/
/*	was type from keyboard.			*/
/*						*/
/************************************************/
static int pampass(int num_msg, const struct pam_message **msg,
                     struct pam_response **response, void *appdata_ptr)

{
struct pam_response *pr;
const struct pam_message *pm;

*response=(struct pam_response *)malloc(num_msg*sizeof(struct pam_response));
if (*response==(struct pam_response *)0)
  return(PAM_CONV_ERR);
(void) memset((void *)*response,0,num_msg*sizeof(struct pam_response));
for (pr=*response, pm = *msg; num_msg--; pr++, pm++) {
  switch (pm->msg_style) {
    case PAM_PROMPT_ECHO_ON:
    case PAM_PROMPT_ECHO_OFF:
      pr->resp=strdup(locpass);
      break;
    case PAM_TEXT_INFO:
    case PAM_ERROR_MSG:
      break;
    default:
      /* Something odd happened 			*/
      /* XXX - should free non-NULL response members	*/
      (void) free((void *)(*response));
      *response = NULL;
      return(PAM_CONV_ERR);
      break;
    }
  }
return(PAM_SUCCESS);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare PAMINFO and call	*/
/*	PAM to check userid/password match.	*/
/*						*/
/************************************************/
static int checkviapam(char *email,char *passwd)

{
int statut;
int oldeuid;
char *username;
pam_handle_t *pamh;
char *ptr;
struct pam_conv pam_conv;

statut=ERRPROC;
oldeuid=geteuid();
username=strdup(email);
(void) seteuid(getuid());
pam_conv.conv=pampass;
if ((pam_status=pam_start(PASSWD,(void *)0,&pam_conv,&pamh))!=PAM_SUCCESS) {
  (void) alert(0,"Unable to open PAM Library for application '%s' (fatal)\n",appname);
  }
else {
  statut=BADID;
  if ((ptr=strchr(username,'@'))!=(char *)0)
    *ptr='\000';
  (void) pam_set_item(pamh,PAM_USER,(const void *)username);
  locpass=passwd;
  switch (pam_status=pam_authenticate(pamh,PAM_SILENT)) {
    case PAM_SUCCESS	:
      statut=CMDOK;
      break;
    default		:
      /*trouble trouble		*/
      break;
    }
  (void) pam_end(pamh,pam_status);
  }
(void) free(username);
(void) seteuid(oldeuid);
return statut;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare check a password	*/
/*	using a pop channel instead of PAM	*/
/*	PAM.					*/
/*						*/
/************************************************/
static int checkviapop(char *pophost,char *username,char *passwd)

{
#define	OPSR	"checkid.c:checkviapop,"
#define	MXWAIT	30	/*waiting for 30 second MAX	*/

int status;
int port;
char hostname[100];

status=ERRPROC;
port=110;		/*POP-3 Port			*/
if (sscanf(pophost,",%95[^\n\r,],%d",hostname,&port)>=2) {
  TLSFILE *tls;

  if ((tls=tls_opentcp(hostname,port,MXWAIT))!=(TLSFILE *)0) {
    int phase;
    int proceed;
    char *buffer;

    phase=0;
    proceed=true;
    status=BADID;
    while (proceed==true) {
      if ((buffer=tls_waitforaction(MXWAIT,tls))==(char *)0) {
	break;			/*timeout			*/
	}
      switch (phase) {
	case 0		:	/*checking for signon		*/
	  if (*buffer=='+') {	/*and sending username		*/
	    (void) tls_printf(tls,"user %s\n",username);
	    }
	  else
            proceed=false;
	  break;
	case 1		:	/*checking for user accepted	*/
	  if (*buffer=='+') {	/*and sending password		*/
	    (void) tls_printf(tls,"pass %s\n",passwd);
	    break;
	    }
	  /*NO break if trouble					*/
	default		:	/*checking for password accepted*/
	  if (*buffer=='+')		
	    status=CMDOK;
				/*sending quit and exiting	*/
	  (void) tls_printf(tls,"quit\n");
	  (void) usleep(10000);
	  proceed=false;
	  break;
        }
      buffer=rou_freestr(buffer);
      phase++;
      }
    (void) tls_close(tls);
    }
  }
else {
  (void) alert(0,"%s Unable to scan <%s>",OPSR,pophost);
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to do usrid checking according*/
/*	checking mode.				*/
/*	2 checking mode are defined:		*/
/*		- PAM, using server PAM function*/
/*		- POP, relying on POP-3 Protocol*/
/*						*/
/************************************************/
int uid_chkid(char *chkmod,char *username,char *password)

{
typedef	enum	{
	c_pam,		/*use PAM function	*/
	c_pop,		/*use POP_3 protocol	*/
	c_unknown
	}ENUMOD;

static VOCTYP vocable[]={
		{c_pam,"PAM",(const void *)0},
		{c_pop,"POP",(const void *)0},
		{c_unknown,(char *)0,(const void *)0}
		};

int status;
VOCTYP *voc;

status=CMDBAD;
if ((voc=rou_getvoca(vocable,chkmod))!=(VOCTYP *)0) {
  switch (voc->code) {
    case c_pam		:
      status=checkviapam(username,password);
      break;
    case c_pop		:
      status=checkviapop(chkmod,username,password);
      break;
    case c_unknown	:
      (void) alert(0,"lvluid.c:uid_chkid, Unknown mode <%s>",chkmod);
      break;
    }
  }
return status;
}
