/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	MODCLM:					*/
/*	Module level routines to handle		*/
/*	Clamav data-base virus access.		*/
/*						*/
/************************************************/
#include	<errno.h>
#include	<string.h>
#include	"subrou.h"
#include	"modclm.h"

typedef	enum	{	/*message type		*/
	vir_check,	/*check virus		*/
	vir_virlib,	/*vir library info	*/
	vir_unknown	/*unknown message	*/
	}VENUM;
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan the virus check	*/
/*	request					*/
/*						*/
/************************************************/
VENUM getvirrqst(char *message)

{
static	struct	{
	VENUM code;	/*message CODE		*/
	char *key;
	}msgcode[]={
		{vir_check,CKVIRI},
		{vir_virlib,VIRLIB},
		{vir_unknown,(char *)0}
		};

register VENUM code;
register int i;

code=vir_unknown;
for (i=0;msgcode[i].key!=(char *)0;i++) {
  if (strstr(message,msgcode[i].key)==message) {
    register char *ptr;

    ptr=message+strlen(msgcode[i].key);
    while (*(ptr)==' ')
      ptr++;
    (void) memmove(message,ptr,strlen(ptr)+1);
    code=msgcode[i].code;
    break;
    }
  }
return code;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for virus check 	*/
/*	request and reply about virus name if	*/
/*	found.					*/
/*						*/
/************************************************/
static void clamcycle(CHLTYP *mng)

{
#define	OPCL	"modclm.c:clamcycle,"

register int dummycnt;
CLAMTYP *clam;
unsigned long request;
unsigned long viruses;

dummycnt=0;
clam=(CLAMTYP *)0;
request=0;
viruses=0;
if ((clam=cla_openscanner((char *)0,0))==(CLAMTYP *)0)
  quickout=true;
while (quickout==false) {
  char *caller;
  char *rqst;
  u_long msec;
  TIMESPEC timeout;

  (void) rou_nozombie();
  (void) cla_reload(clam);
  msec=(u_long)0;
  timeout.tv_sec=1;
  timeout.tv_nsec=0;
  if (ipc_waitpost(mng->boxe,&timeout)==false) {
    continue;
    }
  if ((rqst=ipc_getpost(mng->boxe,&caller,&msec))!=(char *)0) {
    const char *virname; 
    CLMCHKTYP clmstat;
    char *answ;
    VENUM code;

    virname="";
    clmstat=cla_nocheck;
    answ=(char *)0;
    switch (code=getvirrqst(rqst)) {
      case vir_check	:
	request++;
	switch (clmstat=cla_checkvirus(&clam,rqst,&virname)) {
	  case cla_clean	:
	  case cla_nocheck	:
	    virname="";
	    break;
	  case cla_virus	:
	    viruses++;
	    break;
	  }
	dummycnt=asprintf(&answ,"%d %s",(int)clmstat,virname);
        (void) settitle("%s, Virus scanner (%05ld/%05ld)",appname,viruses,request);
	break;
      case vir_virlib	:
	dummycnt=asprintf(&answ,"%d %s",CMDOK,cla_getversdate(clam));
	break;
      default		:
        (void) alert(0,"%s request <%s> unknown",OPCL,rqst);
        answ=strdup(eml_ascstat(ERRPROC));
        break;
      }
    if (answ!=(char *)0) {
      if (ipc_postreply(mng->boxe,caller,answ)==0) {
        (void) alert(0,"%s Unable to post reply",OPCL);
	}
      answ=rou_freestr(answ);
      }
    caller=rou_freestr(caller);
    rqst=rou_freestr(rqst);
    }
  }
clam=cla_closescanner(clam);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fork a process and start	*/
/*	wait for request to be check about	*/
/*	virus contents				*/
/*						*/
/************************************************/
pid_t clm_manager(CHLTYP *mng)

{
pid_t child;
IPTYP **relais;

relais=(IPTYP **)0;
switch((child=rou_fork(false))) {
  case  -1      :
    (void) alert(0,"modclm.c:clm_manager, Unable to start local process "
		   "(error=<%s>)",strerror(errno));
    break;
  case   0      :
    (void) journalling(true,APJRL,"%s (process='%d')",
				  "Virus scanner",getpid());
    (void) settitle("%s, Virus scanner (%05d/%05d)",appname,0,0);
    memleak=DMTRACE;	/*memory leak tracking (xtrem debugging)*/
    (void) dbg_setnoleak(1);
    (void) clamcycle(mng);
    (void) dbg_dumpmem(appname,"Virus_scanner");
    (void) sleep(1);
    (void) exit(0);
    break;
  default       :
    break;
  }
return child;
}
