/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	MODTRA:					*/
/*	Module level routines to handle		*/
/*	Protocol exchange with remote site.	*/
/*						*/
/************************************************/
#include	<malloc.h>
#include	<syslog.h>
#include	<string.h>
#include	<unistd.h>
#include	<errno.h>
#include	<ctype.h>
#include	<netdb.h>
#include	<sys/file.h>
#include	"modtra.h"
#include	"lvlcla.h"
#include	"lvlspm.h"
#include	"lvlchl.h"
#include	"gestls.h"
#include	"gesque.h"
#include	"unieml.h"
#include	"uniper.h"
#include	"unidns.h"
#include	"subrou.h"

typedef struct	{
	char *domain;	/*domain named	*/
	RCPTYP **rcpts;	/*list of rcpts	*/
	}QLSTYP;

static	pid_t chkrpid=(pid_t)0;
static	pid_t viripid=(pid_t)0;
/*

*/
/************************************************/
/*						*/
/*	Procedure to update the local .lisq file*/
/*	within the user quarantine file.	*/
/*						*/
/************************************************/
static void updlisq(char *dir,CTTYP *cta,char *seq)


{
/*
LOG Format:
ID|expediteur|date|taille|sujet
*/
#define LOGFRM "%s%s|%s|%s|%ld|%s\n"
int taille;
char *filename;
FILE *file;

taille=strlen(dir)+150;
filename=(char *)calloc(taille,sizeof(char));
rou_mkname(filename,taille,d_spool,dir,"/",LISQ);
if ((file=fopen(filename,"a+"))==(FILE *)0)
  (void) core_dump("%s Unable open logfile <%s> (error=<%s>)",
		   "modtra.c:updlisq,",filename,strerror(errno));
(void) flock(fileno(file),LOCK_EX);
(void) fprintf(file,LOGFRM,cta->session,seq,
			   cta->evl->realfrom,
			   rou_getstrdatetime(cta->lastxchg),
			   cta->mtaille,
			   cta->evl->subject);
(void) fclose(file);	/*Dropping lock too	*/
(void) free(filename);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return a QLSTYP pointeur	*/
/*	according domain found.			*/
/*						*/
/************************************************/
static QLSTYP *locquar(QLSTYP **qlst,char *domain)

{
QLSTYP *quar;

quar=(QLSTYP *)0;
if (qlst!=(QLSTYP **)0) {
  int i;

  for (i=0;qlst[i]!=(QLSTYP *)0;i++) {
    if (strcmp(qlst[i]->domain,domain)==0) {
      quar=qlst[i];
      break;
      }
    }
  }
return quar;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to collect all different	*/
/*	quarantined RCPT according domain name	*/
/*						*/
/************************************************/
static QLSTYP **getqualist(CTTYP *cta)

{
QLSTYP **qlst;
int i;

qlst=(QLSTYP **)0;
for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
  RCPTYP *rcpt;
  QLSTYP *nqua;
  char *domain;

  rcpt=cta->rcpts[i];
  if (rcpt->status!=m_quarantine)
    continue;
  if ((domain=eml_strdomain(rcpt->rcpt))==(char *)0)
    domain=rcpt->rcpt;
  if ((nqua=locquar(qlst,domain))==(QLSTYP *)0) {
    nqua=(QLSTYP *)calloc(1,sizeof(QLSTYP));
    nqua->domain=strdup(domain);
    qlst=(QLSTYP **)rou_addveclst((void **)qlst,(void *)nqua);
    }
  nqua->rcpts=(RCPTYP **)rou_addveclst((void **)nqua->rcpts,(void *)rcpt);
  }
return qlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free all memory used	*/
/*	by a QLSTYP list			*/
/*						*/
/************************************************/
static QLSTYP **freequalist(QLSTYP **qlst)

{
if (qlst!=(QLSTYP **)0) {
  int i;

  for (i=0;qlst[i]!=(QLSTYP *)0;i++) {
    qlst[i]->domain=rou_freestr(qlst[i]->domain);
    (void) free(qlst[i]->rcpts);
    (void) free(qlst[i]);
    }
  (void) free(qlst);
  qlst=(QLSTYP **)0;
  }
return qlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to request the virus lib	*/
/*	information.				*/
/*						*/
/************************************************/
static char *getclamvers(TRATYP *track)

{
char *clamvers;
char *rsp;

clamvers=(char *)0;
if ((rsp=eml_xchgmsg(track->boxe,viripid,"%s",VIRLIB))!=(char *)0) {
  register char *ptr;
  int status;

  ptr=(char *)0;
  (void) sscanf(rsp,"%d",&status);
  switch (status) {
    case CMDOK		:
      if ((ptr=strchr(rsp,' '))!=(char *)0) 
	ptr++;
      else
        ptr="No vir-lib date";
      clamvers=strdup(ptr);
      break;
    default		:
      break;
    }
  (void) free(rsp);
  }
return clamvers;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to request a virus checking	*/
/*	to virus scanner.			*/
/*						*/
/************************************************/
static CLMCHKTYP isaknownvirus(TRATYP *t,char *dataname,char **virname)

{
#define	VFM	"%s %s"

CLMCHKTYP status;
char *rsp;

status=cla_nocheck;
if ((rsp=eml_xchgmsg(t->boxe,viripid,VFM,CKVIRI,dataname))!=(char *)0) {
  register char *ptr;

  ptr=(char *)0;
  status=(CLMCHKTYP)atoi(rsp);
  switch (status) {
    case cla_virus	: 
      if ((ptr=strchr(rsp,' '))!=(char *)0) 
	ptr++;
      else
        ptr="Unspecified!";
      *virname=strdup(ptr);
      break;
    default		:
      break;
    }
  (void) free(rsp);
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if received E-mail	*/
/*	is plagued with a virus.		*/
/*						*/
/************************************************/
static int checkvirus(TRATYP *track,CTTYP *cta,char **answ)

{
#define	VIRUS	"VIRUS=<%s>"

int itsok;
char *virname;

itsok=true;
virname=(char *)0;
switch (isaknownvirus(track,cta->dataname,&virname)) {
  case cla_clean	:
    break;
  case cla_virus	: 
    (void) free(*answ);
    *answ=strdup(eml_ascstat(BADVIR,virname));
    cta->evl->virname=strdup(virname);
    (void) journalling(false,EVJRL,VIRUS,virname);
    itsok=false;
    break;
  case cla_nocheck	:
    (void) journalling(false,EVJRL,VIRUS,"Not checked");
    break;
  }
virname=rou_freestr(virname);
return itsok;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if process need to	*/
/*	reload its application.			*/
/*						*/
/************************************************/
static int trareload(BOXTYP *boxe)

{
#define	OPRE	"modtra.c:trareload,"

register int done;
register int todo;

done=true;
todo=sigreload;
if ((chkrpid==(pid_t)0)|| (rou_checkprocess(chkrpid)==false)) {
  chkrpid=(pid_t)0;
  todo=true;
  }
if ((viripid==(pid_t)0)|| (rou_checkprocess(viripid)==false)) {
  viripid=(pid_t)0;
  todo=true;
  }
if (todo==true) {
  register int try;

  sigreload=false;
  done=false;
  for (try=0;try<3;try++) {
    if (chkrpid==(pid_t)0)
      chkrpid=chl_getmngpid(boxe,stmng[(int)chl_checker]);
    if (viripid==(pid_t)0)
      viripid=chl_getmngpid(boxe,stmng[(int)chl_clamav]);
    if ((chkrpid!=(pid_t)0)&&(viripid!=(pid_t)0)) {
      done=true;
      break;
      }
    if (chkrpid==(pid_t)0)
      (void) alert(0,"%s Unable to get <%s> pid (trouble?)",
		      OPRE,stmng[(int)chl_checker]);
    if (viripid==(pid_t)0)
      (void) alert(0,"%s Unable to get <%s> pid (trouble?)",
		      OPRE,stmng[(int)chl_clamav]);
    (void) sleep(1);
    }
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to establish what kind of	*/
/*	quarantaine policy is related a specific*/
/*	recipient.				*/
/*						*/
/************************************************/
static DFSTYP *getqualocdom(TRATYP *track,char *domain)

{
DFSTYP *dfs;

dfs=track->dfs;
if (track->skrlst!=(SKRTYP **)0) {
  DOMTYP *doms;

  if ((doms=eml_findskr(track->skrlst,domain,false))!=(DOMTYP *)0){
    if (doms->sndr1!=(SKRTYP *)0) {
      DFSTYP *dfsloc;
      int i;

      dfsloc=(DFSTYP *)0;
      for (i=0;i<2;i++) {
	switch (i) {
	  case 0	:
            dfsloc=eml_locatedfs(doms->sndr1->dfs,domain);
	    break;
	  case 1	:
            dfsloc=eml_locatedfs(doms->sndr1->dfs,DFLTDOM);
	    break;
	  }
	if (dfsloc!=(DFSTYP *)0) {
	  dfs=dfsloc;
	  break;
	  }
	}
      }
    (void) free(doms);
    }
  }
return dfs;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to dispatch E-mail in the	*/
/*	proper quarantine area			*/
/*						*/
/************************************************/
static void sortquare(TRATYP *track,CTTYP *cta,QLSTYP **qlst)

{
#define	OPQU	"modtra.c:sortquare,"

int nseq;
int i;

nseq=1;
for (i=0;qlst[i]!=(QLSTYP *)0;i++) {

  if (qlst[i]->rcpts!=(RCPTYP **)0) {
    int j;

    for (j=0;qlst[i]->rcpts[j]!=(RCPTYP *)0;j++,nseq++) {
      char *username;
      char *ptr;
      char *qdir;
      char *domain;
      FILE *qfile;
      DFSTYP *dfs;
      char ext[30];
      char seq[50];
      char filename[300];

      (void) snprintf(seq,sizeof(seq),".%02d.%05d",cta->sequence,nseq);
      (void) snprintf(ext,sizeof(ext),"%s%s",seq,QUEUEEXT);
      username=strdup(qlst[i]->rcpts[j]->rcpt);
      domain=eml_strdomain(username);
      dfs=getqualocdom(track,domain);
      if ((ptr=strchr(username,'@'))!=(char *)0)
        *ptr='\000';
      if (dfs->qualoc==true)
        qdir=rou_memstrcat((char *)0,"%s%s/%s/",
				     TRIADIR,qlst[i]->domain,username);
      else
        qdir=strdup(QUEUDIR);
      (void) rou_mkname(filename,sizeof(filename)-20,d_spool,qdir,"","");
      filename[strlen(filename)-1]='\000';
      (void) rou_createdirs(filename);
      (void) rou_mkname(filename,sizeof(filename),d_spool,qdir,
						  cta->session,ext);
      if ((qfile=fopen(filename,"w"))==(FILE *)0) 
        (void) core_dump("%s Unable to open <%s> (error=<%s>)",
			 OPQU,filename,strerror(errno));
      (void) fprintf(qfile,"%s %s\n",SPAM,qlst[i]->rcpts[j]->why);
      (void) fprintf(qfile,"%s %s\n",SMAILF,cta->frm->from);
      (void) fprintf(qfile,"%s %s\n",SUBJECT,cta->evl->subject);
      (void) fprintf(qfile,"%s %s\n",SRCPTT,qlst[i]->rcpts[j]->rcpt);
      (void) fprintf(qfile,SQUIT);
      (void) fclose(qfile);
      /*duplicating E-mail data via hard link			*/
      (void) snprintf(ext,sizeof(ext),"%s%s",seq,EMAILEXT);
      (void) rou_mkname(filename,sizeof(filename),d_spool,qdir,
						  cta->session,ext);
      if (link(cta->dataname,filename)<0) 
        (void) core_dump("%s Unable to link E-mail data to <%s> (error=<%s>)",
			  OPQU,filename,strerror(errno));
      (void) updlisq(qdir,cta,seq);
      qdir=rou_freestr(qdir);
      username=rou_freestr(username);
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare the actual	E-mail	*/
/*	sending to sendmail for direct E-mail.	*/
/*						*/
/************************************************/
static void sortdirect(CTTYP *contact)

{
#define	OPDI	"modtra.c:sortdirect,"

FILE *qfile;
int i;
char ext[10];
char filename[300];

(void) snprintf(ext,sizeof(ext),".%02d%s",contact->sequence,QUEUEEXT);
(void) rou_mkname(filename,sizeof(filename),d_spool,QUEUDIR,
					    contact->session,ext);
if ((qfile=fopen(filename,"w"))==(FILE *)0)
  (void) core_dump("%s Unable to open <%s> (error=<%s>)",
		    OPDI,filename,strerror(errno));
if (contact->relayable==true)
  (void) fprintf(qfile,"%s 1\n",SRELAY);
(void) fprintf(qfile,"%s %s\n",SMAILF,contact->frm->from);
(void) fprintf(qfile,"%s %s\n",SUBJECT,contact->evl->subject);
for (i=0;contact->rcpts[i]!=(RCPTYP *)0;i++) {
  switch (contact->rcpts[i]->status) {
    case m_relayed	:
    case m_direct	:
      (void) fprintf(qfile,"%s %s\n",SRCPTT,contact->rcpts[i]->rcpt);
      break;
    case m_rejected	:	/*nothing to do	*/
    case m_delayed	:	/*nothing to do	*/
    case m_quarantine	:	/*nothing to do	*/
      break;
    default		:
      (void) alert(0,"%s rcpt status unkcheck '%d'",contact->rcpts[i]->status);
      break;
    }
  }
(void) fprintf(qfile,SQUIT);
(void) fclose(qfile);
/*duplicating E-mail data via hard link			*/
(void) snprintf(ext,sizeof(ext),".%02d%s",contact->sequence,EMAILEXT);
(void) rou_mkname(filename,sizeof(filename),d_spool,QUEUDIR,
						    contact->session,ext);
if (link(contact->dataname,filename)<0) 
  (void) core_dump("%s Unable to link E-mail data to <%s> (error=<%s>)",
		      OPDI,filename,strerror(errno));
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for advices, as 	*/
/*	can come from differente checker, 	*/
/*	procedure wait until all advices are	*/
/*	received or a timeout is set.		*/
/*						*/
/************************************************/
static char *getadvrply(BOXTYP *boxe,char *msgref)

{
#define	OPGE	"motra.c:getadvrply,"

#define	MXWRPLY	120	/*max sec to get advice	*/
#define MXCOL	4000

char *rsp;
int maxmsg;
int gotall;
time_t start;
char collected[MXCOL];

rsp=(char *)0;
maxmsg=0;
gotall=false;
start=rou_systime();
(void) memset(collected,'\000',sizeof(collected));
while ((start+MXWRPLY)>rou_systime()) {
  char *got;
 
  if ((gotall==true)||(quickout==true))
    break;
  (void) usleep(1000000); 
  if ((got=ipc_getreply(boxe,msgref))!=(char *)0) {
    char **split;

    if ((split=eml_splitmsg(got))!=(char **)0) {
      int num;

      if ((num=rou_vecsize((void **)split))>0) {
	int status;
	int nummsg;

	num--;
	nummsg=0;
	if (sscanf(split[num],"%d %d/%d",&status,&nummsg,&maxmsg)==3) {
          int i;

	  if (maxmsg>=MXCOL)
	    core_dump("%s far too many (>%d) reply!",OPGE,MXCOL);
	  if (nummsg<1)
	    core_dump("%s error in protocol reply!",OPGE);
	  collected[nummsg-1]='A';
	  gotall=(strlen(collected)==maxmsg);
	  if (status==CMDOK) {
	    for (i=0;i<num;i++)
	      rsp=rou_memstrcat(rsp,"%s\n",split[i]);
	    }
	  }
	}
      split=rou_freestrlst(split);
      }
    got=rou_freestr(got);
    }
  }
if (rsp!=(char *)0)
  rsp=rou_memstrcat(rsp,"%d %d/%d This were my advices",
			 CMDOK,strlen(collected),maxmsg);
else {
  errno=ETIME;
  }
return rsp;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send an advice request and	*/
/*	wait for replies.			*/
/*						*/
/************************************************/
static char *waitforadv(BOXTYP *boxe,char *msg)

{
#define	OPWA	"modtra.c:waitforadv,"
char *rsp;
char *msgref;

rsp=(char *)0;
if ((msgref=ipc_postmsg(boxe,chkrpid,msg))!=(char *)0) {
  rsp=getadvrply(boxe,msgref);
  msgref=rou_freestr(msgref);
  }
else {
  (void) alert(0,"%s Unable to post message <%s> (error=<%s>)",
		  OPWA,msg,strerror(errno));
  }
if (rsp==(char *)0) {
  (void) tracing(TRALOG,0,"%s, No reply to advice request",OPWA);
  journalling(false,APJRL,"No reply to advice request to <%s>",msg);
  }
return rsp;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to adjuste each user status	*/
/*	deciding if E-mail need to go in	*/
/*	quarantine or to be transmited direct	*/
/*	this is done by consulting remote 	*/
/*	mentor.					*/
/*	If the Email was found too big for one	*/
/*	Recipient the returned status is true	*/
/*						*/
/************************************************/
static int checkuserstatus(CTTYP *contact)

{
if (contact->rcpts!=(RCPTYP **)0) {
  int i;

  for (i=0;contact->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;

    rc=contact->rcpts[i];
    if (rc->status==m_indata) {
      rc->status=m_direct;
      rc->why=strdup("accepted"); 
      }
    if (contact->toobig==true) {
      rc->why=rou_freestr(rc->why); 
      rc->status=m_discarded;
      rc->why=strdup("Email found too big for one recipient"); 
      }
    }
  }
return contact->toobig;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to do the actual E-mail	*/
/*	sorting according RCPT status.		*/
/*						*/
/************************************************/
static void sortoutemail(TRATYP *track,CTTYP *contact)

{
int i;

for (i=0;i<2;i++) {
  int count;

  count=0;
  switch(i)  {
    case 0  :	/*taking care of direct E-mail	*/
      count=eml_countadvice(contact,m_direct);
      count+=eml_countadvice(contact,m_relayed);
      if (count>0)
	(void) sortdirect(contact);
      break;
    case 1  :	/*taking care of quarantine	*/
      count=eml_countadvice(contact,m_quarantine);
      if (count>0) {
	QLSTYP **quare;

	if ((quare=getqualist(contact))!=(QLSTYP **)0) {
	  (void) sortquare(track,contact,quare);
	  quare=freequalist(quare);
	  }
	}
      break;
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle RCPT checking	*/
/*	via a local (or remote) SMTP server	*/
/*						*/
/************************************************/
static int rmtchecking(TRATYP *t,char *sesid,FRMTYP *frm,
		       char *rcpt,int relayable,char **rsp)

{
#define	CKFRM	"%s %s %s %lu %s,%d %s %d"
int checkstat;
const char *ptrstat;
char errstat[500];

checkstat=ERRPROC;
(void) strcpy(errstat,"");
if ((*rsp=eml_xchgmsg(t->boxe,chkrpid,CKFRM,CKRCPT,frm->ipremote,
		      frm->from,frm->mtaille,
		      frm->listenip,frm->listenport,
		      rcpt,relayable))!=(char *)0) {
  int pos;
  char *ptr;

  pos=0;
  (void) que_cleanstring(*rsp);
  while ((ptr=strchr(*rsp,'\n'))!=(char *)0)
     *ptr=' ';	/*to remove any LF within the reply	*/
  (void) sscanf(*rsp,"%d%n",&checkstat,&pos);
  ptr=(*rsp)+pos;
  while (*ptr==' ')
    ptr++;
  (void) memmove(*rsp,ptr,strlen(ptr)+1);
  }
ptrstat=eml_ascstat(checkstat,*rsp);
switch (checkstat) {
  case BYPASS	:	/*RELAYOK become CMDOK		*/
    (void) snprintf(errstat,sizeof(errstat),"%d %s",CMDOK,*rsp);
    break;
  case RELAYOK	:	/*RELAYOK become CMDOK		*/
  case CMDOK	:
    (void) snprintf(errstat,sizeof(errstat),"%d Address Accepted",CMDOK);
    break;
  case NDIRTY	:
  case NOSPAMS	:
    (void) strncpy(errstat,ptrstat,sizeof(errstat));
    break;
  case BADSPF	:
  case WRNGSPF	:
  case NEWSITE	:
  case CHKUSR	:
  case NOSNDR	:
  case ERRPROC	:
    (void) strncpy(errstat,ptrstat,sizeof(errstat));
    break;
  case REJECT	:
  default	:
    if (strlen(*rsp)>0)
      (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,*rsp);
    else
      (void) strncpy(errstat,ptrstat,sizeof(errstat));
    break;
  }
*rsp=rou_freestr(*rsp);
*rsp=strdup(errstat);
return checkstat;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check the advice returned	*/
/*	by the remote mentor.			*/
/*						*/
/************************************************/
static int checkadvice(CTTYP *cat,char *rsp)

{
int status;
char **data;

status=ERRPROC;
if ((data=eml_splitmsg(rsp))!=(char **)0) {
  int num;

  if ((num=rou_vecsize((void **)data))>0) {
    int max;
    int i;

    num--;
    (void) sscanf(data[num],"%d",&status);
    max=rou_vecsize((void **)cat->rcpts);
    for (i=0;i<num;i++) {
      int numrcpt;
      int locstat;
      RCPTYP *rcploc;
      char info[100];
    
      (void) strcpy(info,""); 
      (void) sscanf(data[i],"%d-%d %95s",&locstat,&numrcpt,info);
      rcploc=cat->rcpts[numrcpt];
      if (numrcpt<max) {
	switch (locstat) {
	  case SENADV	:
	    rcploc->status=m_direct;
	    break;
	  case QUAADV	:
	    rcploc->status=m_quarantine;
	    if (strlen(info)>0) {
	      rcploc->why=rou_freestr(rcploc->why);
	      rcploc->why=strdup(info);
	      }
	    break;
	  case TOOBIG	:
	    cat->toobig=true; 
	    break;
	  case DISADV	:	/*nothing to do	*/
	    break;
	  case RLYADV	:	/*nothing to do	*/
	    break;
	  default	:
	    (void) alert(0,"modtra.c:checkadvice, Unknown status <%s> rcpt->status='%d'",
			    data[i],rcploc->status);
	    break;
	  }
	}
      }
    }
  data=rou_freestrlst(data);
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to log meaningful E-mail	*/
/*	header informatin to be reported in	*/
/*	local logs.				*/
/*						*/
/************************************************/
static void reporthdrs(CTTYP *cta)

{
if (cta->evl->realfrom!=(char *)0)
  (void) journalling(false,EVJRL,"E-From: %s",cta->evl->realfrom);
if (cta->evl->subject!=(char *)0)
  (void) journalling(false,EVJRL,"Subject: %s",cta->evl->subject);
if (cta->evl->msgid!=(char *)0)
  (void) journalling(false,EVJRL,"Message-Id: %s",cta->evl->msgid);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to 'force' an advice in case	*/
/*	no answer was provided by remote mentor	*/
/*						*/
/************************************************/
static void forceadvice(CTTYP *cta)

{
if (cta->rcpts!=(RCPTYP **)0) {
  int i;

  (void) journalling(false,EVJRL,"Advice localy set");
  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;

    rc=cta->rcpts[i];
    switch (rc->status) {
      case m_indata	: /*Lets say Email good	*/
	rc->status=m_direct;
        break;
      case m_quarantine	:
      case m_direct	:
      case m_relayed	:
      case m_rejected	:
      case m_delayed	:
      case m_discarded	:
      case m_undefined	:
        break;
      }
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to send header information	*/
/*	to remote Mentor, this will help	*/
/*	Mentor to advise clement about E-mail	*/
/*						*/
/************************************************/
static char *getadvice(TRATYP *track,CTTYP *cta)

{
#define	OPHD	"modtra.c:sendhdrs,"

char *rsp;
char *msg;
int phase;
int proceed;

rsp=(char *)0;
msg=(char *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase)  {
    case  0	:	/*Message Channel		*/
      msg=rou_memstrcat(msg,"%s 1/1 ",ADVICE);
      break;
    case  1	:	/*Message Type & origine	*/
      msg=rou_memstrcat(msg,"%s %s\n",MENID,cta->session);
      msg=rou_memstrcat(msg,"%s %s\n",CLIP,cta->ipremote);
      msg=rou_memstrcat(msg,"%s %s\n",MFROM,cta->frm->from);
      break;
    case  2	:
      if (cta->evl->subject!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",SUBJ,cta->evl->subject);
      break;
    case  3	:
      if (cta->evl->msgid!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",MSGID,cta->evl->msgid);
      break;
    case  4	:
      if (cta->evl->inrplyto!=(char **)0) {
	int i;
	
	for (i=0;cta->evl->inrplyto[i]!=(char *)0;i++) 
          msg=rou_memstrcat(msg,"%s %s\n",RPLID,cta->evl->inrplyto[i]);
	}
      break;
    case  5	:
      msg=rou_memstrcat(msg,"%s %ld %ld %d %d\n",MSGINF,cta->evl->cachet,
			     cta->mtaille,cta->evl->nbrerror,cta->relayable);
      break;
    case  6	:
      if (cta->evl->realfrom!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",HFROM,cta->evl->realfrom);
      break;
    case  7	:
      if (cta->evl->spamlvl!=(char *)0)
        msg=rou_memstrcat(msg,"%s %s\n",SPAM,cta->evl->spamlvl);
      if (cta->evl->virname!=(char *)0) 
        msg=rou_memstrcat(msg,"%s %s\n",VIRINF,cta->evl->virname);
      break;
    case  8	:
      if (cta->evl->distlist!=(char **)0) {
	int i;
	
	for (i=0;cta->evl->distlist[i]!=(char *)0;i++)
          msg=rou_memstrcat(msg,"%s %s\n",HTO,cta->evl->distlist[i]);
	}
      break;
    case  9	:
      if (cta->rcpts!=(RCPTYP **)0) {
	int i;

	for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
	  int status;
	  RCPTYP *rc;
	  char *why;

	  rc=cta->rcpts[i];
	  why="";
	  status=GIVADV;
	  switch (rc->status) {
	    case m_indata	:
	    case m_quarantine	:
	    case m_direct	:
	      break;
	    case m_relayed	:
	      status=RLYADV;
	      why=rc->why;
	      break;
	    case m_delayed	:
	    case m_rejected	:
	      status=DISADV;
	      why=rc->why;
	      break;
	    case m_discarded	:	/*should never occure	*/
	    case m_undefined	:
	      break;
	    }
          msg=rou_memstrcat(msg,"%s %02d <%s> %03d %s\n",
				 ADVRCPT,i,rc->rcpt,status,why);
	  }
        }
      break;
    default	:	/* SAFE guard processus done		*/
      proceed=false;
      break;
    }
  phase++;
  }
if ((rsp=waitforadv(track->boxe,msg))==(char *)0) {
  (void) alert(0,"%s Unable to get message <%s> reply , (error=<%s>)",
      		    OPHD,msg,strerror(errno));
  }
(void) free(msg);
return rsp;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to transmit a get reply	*/
/*	about a user ID. used to allow access	*/
/*	Clement WEB interface.			*/
/*						*/
/************************************************/
static int checknewid(TRATYP *t,CTTYP *c)

{
#define	OPCK	"modtra.c:checknewid,"

#define	CKIDFRM	"%s %s"

int checkstat;
char *rsp;
int port;
char username[100];
char password[100];
char chkidsrv[100];
char errstat[400];

checkstat=ERRPROC;
rsp=(char *)0;
port=0;
(void) strcpy(username,"");
(void) strcpy(password,"");
(void) strcpy(chkidsrv,"");
if (sscanf(t->chkidsrv,"%95[^,\n\r],%d",chkidsrv,&port)!=2) {
  (void) alert(0,"%s Unable to scan chkidsrv <%s> (format incorrect)",
		  OPCK,t->chkidsrv);
  }
else {
  char *info;
  char *ptr;

  info=c->info;
  c->info=(char *)0;
  if (info!=(char *)0) {
    if ((ptr=strchr(info,':'))!=(char *)0) {
      ptr++;
      (void) memmove(info,ptr,strlen(ptr)+1);
      }
    (void) sscanf(info,"%95s %95s",username,password);
    if ((rsp=eml_xchgmsg(t->boxe,chkrpid,CKIDFRM,CHKID,info))!=(char *)0) {
      int pos;
  
      (void) que_cleanstring(rsp);
      pos=0;
      if (sscanf(rsp,"%d %n",&checkstat,&pos)==1) {
 	register char *prsp;	
 
	prsp=rsp+pos;
        (void) memmove(rsp,prsp,strlen(prsp)+1);
	}
      }
    info=rou_freestr(info);
    }
  }
switch (checkstat) {
  case ERRPROC	:
    (void) snprintf(errstat,sizeof(errstat),"%s",eml_ascstat(checkstat));
    (void) alert(0,"CHECKID wrong command");
    break;
  case IDOK	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"MCHKID: %s (%s)",errstat,username);
    break;
  case CMDOK	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"MCHKID: %s (%s)",errstat,username);
    break;
  case BADID	:
  default	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"MCHKID: %s (reject->%s, %s)",
			           errstat,username,password);
    break;
  }
rsp=rou_freestr(rsp);
(void) eml_printfcrlf(c->tls,"%s",errstat);
return ((checkstat==CMDOK)||(checkstat==IDOK));
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to transmit "take note this	*/
/*	email is good" to remote mentor.	*/
/*						*/
/************************************************/
static int saygoodemail(TRATYP *t,CTTYP *c)

{
int checkstat;
char *rsp;
char errstat[400];

checkstat=ERRPROC;
if ((rsp=eml_xchgmsg(t->boxe,chkrpid,"%s %s",GMAIL,c->info))!=(char *)0){
  int pos;
  
  (void) que_cleanstring(rsp);
  pos=0;
  if (sscanf(rsp,"%d %n",&checkstat,&pos)==1) {
    register char *prsp;

    prsp=rsp+pos;
    (void) memmove(rsp,prsp,strlen(prsp)+1);
    }
  }
switch (checkstat) {
  case ERRPROC	:
    (void) snprintf(errstat,sizeof(errstat),"%s",eml_ascstat(checkstat));
    (void) alert(0,"%s wrong command",GMAIL);
    break;
  case CMDOK	:
  case FAILED	:
  default	:
    (void) snprintf(errstat,sizeof(errstat),"%d %s",checkstat,rsp);
    (void) journalling(true,EVJRL,"%s: %s (%s)",GMAIL,errstat,rsp);
    break;
  }
rsp=rou_freestr(rsp);
(void) eml_printfcrlf(c->tls,"%s",errstat);
return true;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle RCPT check, first	*/
/*	check the RCPT localy and then do	*/
/*	a deep checking by asking to mentor.	*/
/*						*/
/************************************************/
static int checkrcpt(TRATYP *t,CTTYP *c)

{
#define	MXRCPT	50	/*maximum number of RCPT*/
#define	NOMX	"No MX or IP linkable to originator domain"

RCPTYP *rmt;
char *dom;
int letsroll;
long nr;
int status;
int proceed;
int phase;

rmt=(RCPTYP *)calloc(1,sizeof(RCPTYP));
rmt->status=m_indata;
rmt->rcpt=c->info;
dom=(char *)0;
c->info=(char *)0;
letsroll=true;
nr=1;
status=REJECT;
proceed=true;
phase=1;
while (proceed==true) {
  switch (phase) {
    case   0:		/*trouble, trouble			*/
      proceed=false;
      break;
    case   1	: 	/*is user already in rcpt		*/
      if (c->rcpts!=(RCPTYP **)0) {
        int i;

	for (i=0;c->rcpts[i]!=(RCPTYP *)0;i++) {
	  if (strcmp(c->rcpts[i]->rcpt,rmt->rcpt)==0) {
	    rmt->status=m_discarded;
	    rmt->smtpstatus=c->rcpts[i]->smtpstatus;
	    rmt->why=strdup(c->rcpts[i]->why);
	    break;
	    }
	  nr++;
	  }
        }
      break;
    case   2	: 	/*checking total number of RCPT		*/
      if ((c->relayable==false)&&(nr>MXRCPT)) {
	rmt->why=strdup(eml_ascstat(BADPAR,"Too many recipient specified"));
	rmt->status=m_rejected;
	}
      break;
    case   3	: 	/*no need to test if already rejected	*/
      switch (rmt->status) {
	case m_delayed		:
	case m_rejected		:
	case m_discarded	:
	  phase++;
	  break;
	default			:
	  break;
	}
      break;
    case   4	: {	/*checking via remote connection	*/
      char *answ;

      answ=(char *)0;
      switch ((status=rmtchecking(t,c->session,c->frm,rmt->rcpt,
				  c->authok,&answ))) {
	case BYPASS	:	/*everything OK			*/
	  break;
	case CMDOK	:	/*everything OK			*/
	  break;
	case RELAYOK	:	/*remote IP is relayable	*/
	  if (c->relayable==false)
            (void) journalling(false,EVJRL,"Found remote IP relayable");
	  c->relayable=true;
	  rmt->status=m_relayed;
	  status=CMDOK;		/*Lets say everything OK	*/
	  break;
	case NOMENT	:	/*mentor do care about it	*/
	  if (c->authok==true) {/*we trust remote originator	*/
	    if (c->relayable==false)
              (void) journalling(false,EVJRL,"Found remote IP Authenticated");
	    c->relayable=true;
	    answ=rou_freestr(answ);
	    answ=rou_memstrcat(answ,"%d Address relayable",CMDOK);
	    rmt->status=m_relayed;	/*mail will be relayed	*/
	    status=CMDOK;
	    }
	  else
	    rmt->status=m_rejected;
	  break;
	case NOSPAMS	:
	  rmt->status=m_rejected;
	  break;
	case ERRPROC	:	/*we are really in deep trouble	*/
	  rmt->status=m_delayed;
	  letsroll=false;
	  quickout=true;	/*lets get out fast		*/
	  break;
	case CHKUSR	:	/*Recipient need to be checked	*/
	case TOOMANY	:	/*To Many E_mail in queue	*/
	case TOOOFT	:	/*server connecting too often	*/
	case NEWSITE	:	/*server first encountance	*/
	case ALREAD	:	/*server already connected	*/
	case NOSNDR	:	/*Sender is not connected	*/
	case NOCHKR	:	/*Checker not available		*/
	case NDIRTY	:	/*Sender seems to be dirty	*/
	  rmt->status=m_delayed;
	  break;
	default		:	/*too bad for other status	*/
	  rmt->status=m_rejected;
	  break;
	}
      rmt->smtpstatus=status;
      rmt->why=answ;
      switch (rmt->smtpstatus) {
	case NEWSITE	:	/*no need to go further	*/
	case NOSPAMS	:	/*site is new or spammer*/
	  letsroll=false;	/*lets forget remote	*/
	  break;
	default		:	/*lets continue	session	*/
	  break;		/*nothing to do		*/
	}
      }
      break;
    case    5	:	/*checking MX for domain part 	*/
      dom=eml_strdomain(c->frm->from);
      if ((dom!=(char *)0)&&(dom!=c->frm->from)) {
	MXTYP **mxlst;
	u_int32_t **addrlist;
        struct hostent *hbname;

	mxlst=(MXTYP **)0;
	addrlist=(u_int32_t **)0;
	if ((mxlst=dns_getmx(dom))!=(MXTYP **)0) {
          mxlst=dns_cleanmx(mxlst);
	  break;	/*there is an MX		*/
	  }
        if ((addrlist=dns_getaddr(dom))!=(u_int32_t **)0) {
	  addrlist=dns_cleanaddr(addrlist);
	  break;	/*There an IP address		*/
	  }
        if ((addrlist=dns_gethostip(dom))!=(u_int32_t **)0) {
	  addrlist=dns_cleanaddr(addrlist);
	  break;	/*There an [IP] address		*/
	  }
        if ((hbname=gethostbyname(dom))!=(struct hostent *)0) {
	  break;	/*at least the name is defined	*/
	  }
	rmt->status=m_rejected;
	rmt->why=strdup(eml_ascstat(BADPAR,NOMX));
	letsroll=false;
        }
      break;
    default	: {	/*false safe, reporting status	*/
      int pos;
    			/*checking # already rejected	*/
      if (c->rcpts!=(RCPTYP **)0) {
	int verybad;
        int bad;
	int i;

	verybad=false;
	bad=0;	
	for (i=0;(c->rcpts[i]!=(RCPTYP *)0)&&(verybad==false);i++) {
	  switch (c->rcpts[i]->smtpstatus) {
	    case RELAYOK:	/*not to count bad	*/
	    case CMDOK	:	/*not to count bad	*/
	      break;
	    case NOSPAMS:
	    case BADSPF	:
	      if ((i==bad)&&(i>=3))
	        verybad=true;
	      break;
	    default	:
	      bad++;
	      if (bad>10)
		verybad=true;
	      break;
	    }
	  }
        if (verybad==true) {
	  rmt->why=strdup(eml_ascstat(MANYRJT));
	  letsroll=false;/*lets forget about remote	*/
	  }
        }
      pos=0;
      if (sscanf(rmt->why,"%d%n",&rmt->smtpstatus,&pos)==1) {
	register char *prsp;
	
	prsp=rmt->why+pos;
        (void) memmove(rmt->why,prsp,strlen(prsp)+1);
	}
      switch(rmt->status) {
	case m_rejected		:
	  (void) sleep(2);	/*error penalty for remote server */
	  break;
	case m_delayed		:
	  (void) sleep(1);	/*error penalty for remote server */
	  break;
	default			:
	  break;
	}
      (void) eml_printfcrlf(c->tls,"%d%s",rmt->smtpstatus,rmt->why);
      (void) journalling(false,EVJRL,"MRCPT: %d <%s>\n%-9s%s",
				     rmt->smtpstatus,rmt->rcpt,"",rmt->why);
      if ((rmt->status==m_rejected)||(rmt->status==m_delayed)) {
	char *origine;
	char *cnd;
        char *ptr;

        origine=c->peername;
        if (strcmp(origine,NORVERS)==0)
          origine=c->ipremote;
	cnd="Rejected";
	if (rmt->status==m_delayed)
	  cnd="Delayed";
	while ((ptr=strstr(rmt->why,">, "))!=(char *)0) {
	  ptr++;	/*stripping the ATTENTION!!! part	*/
	  *ptr='\000';
          }
        (void) journalling(true,WJRL,"%s %d %s %s/%s <%s> STATUS: %s",
			             cnd,
			             0,"Bytes",
			             c->frm->from,
				     origine,
			             rmt->rcpt,
			             rmt->why);
	}
      switch (rmt->status) {
	case m_discarded:
	  rmt=eml_freercpt(rmt);
	  break;
	default		:
          c->rcpts=(RCPTYP **)rou_addveclst((void **)c->rcpts,(void *)rmt);
	  break;
	}
      if (letsroll==false)
        (void) journalling(false,EVJRL,"STOP: disconnecting session");
      proceed=false;
      }
      break;
    }
  phase++;
  }
return letsroll;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to write E-mail report within	*/
/*	WEB area, such it can be search and 	*/
/*	displayed Cvia the customer browser.	*/
/*						*/
/************************************************/
static void webreport(CTTYP *cta)

{
#define	OPWE	"modtra.c:webreport"

if (cta->rcpts!=(RCPTYP **)0) {
  char *subject;
  char *unit;
  char *mailfrom;
  char *origine;
  u_long ltaille;
  int i;
  
  ltaille=cta->mtaille;
  unit="Bts";
  if (ltaille>1024) {
    unit="KBts";
    ltaille/=1024;
    }
  if (ltaille>1024) {
    unit="MBts";
    ltaille/=1024;
    }
  ltaille++;
  origine=cta->peername;
  if (strcmp(origine,NORVERS)==0)
     origine=cta->ipremote;
  mailfrom=cta->evl->realfrom;
  if (mailfrom==(char *)0)
     mailfrom=cta->frm->from;
  if (cta->evl->subject!=(char *)0) 
    subject=strdup(cta->evl->subject);
  else
    subject=strdup("");
  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;
    char *code;

    rc=cta->rcpts[i];
    code="ERRPROC";
    switch (rc->status) {
      case m_direct	:
        code="Direct";
        break;
      case m_relayed	:
        code="Relayed";
        break;
      case m_discarded	:
      case m_rejected	:
        code="Rejected";
        break;
      case m_delayed	:
        code="Delayed";
        break;
      case m_quarantine	:
        code="Quarantine";
        break;
      default		:
        core_dump("%s Unknown status '%d' for user <%s>",
		   OPWE,rc->status,rc->rcpt);
        break;
      }
    (void) journalling(true,WJRL,"%s %d %s %s/%s <%s> %s",
			          code,
			          ltaille,unit,
			          mailfrom,origine,
			          rc->rcpt,
			          subject);
    }
  subject=rou_freestr(subject);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to write E-mail report within	*/
/*	E-mail journal and transmit the info	*/
/*	to remote checker.			*/
/*						*/
/************************************************/
static void logreport(CTTYP *cta)

{
#define	OPSE	"modtra.c:logreport"
#define	RCFM	"RCPT: %10s%s%-15s <%s>"

if (cta->rcpts!=(RCPTYP **)0) {
  int i;

  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
    RCPTYP *rc;
    char *code;
    char *sep;
    char info[100];

    rc=cta->rcpts[i];
    code="ERRPROC";
    sep=" ";
    (void) strcpy(info," ");
    switch (rc->status) {
      case m_direct	:
        code="Direct";
        break;
      case m_relayed	:
        code="Relayed";
        break;
      case m_discarded	:
        code="Discarded";
        (void) strncpy(info,"Mail found too big for one of the recipient",sizeof(info)-1);
        break;
      case m_rejected	:
        code="Rejected";
        break;
      case m_delayed	:
        code="Delayed";
        break;
      case m_quarantine	:
        code="Quarantine";
	if (strlen(rc->why)>0) {
	  int lang;
	  int cal;
	  int perti;
	     
	  if (sscanf(rc->why,"%d/%d/%d",&lang,&cal,&perti)==3) {
	    (void) snprintf(info,sizeof(info),"%s/%s",
			         per_getstrper((LNGTYP)lang,(PERTYP)cal),
				 per_getstrper((LNGTYP)lang,(PERTYP)perti));
	    }
	  else {
	    (void) memset(info,'\000',sizeof(info));
            (void) strncpy(info,rc->why,sizeof(info)-1);
	    }
	  sep=":";
	  }
        break;
      default		:
        core_dump("%s Unknown status '%d' for user <%s>",
		   OPSE,rc->status,rc->rcpt);
        break;
      }
    (void) journalling(false,EVJRL,RCFM,code,sep,info,rc->rcpt);
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to dispatch data according	*/
/*	information collected via Mentor.	*/
/*						*/
/************************************************/
static void dispatchdata(TRATYP *track,CTTYP *cta)

{
#define GOFRM	"%d Message accepted for delivery"

register int dummycnt;
int phase;
int proceed;
char *answer;

phase=1;
proceed=true;
dummycnt=asprintf(&answer,GOFRM,CMDOK);
while (proceed==true) {
  switch (phase) {
    case   0	:	/*we have a big trouble			*/
      if (cta->rcpts!=(RCPTYP **)0) {
	int i;

	for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
	  RCPTYP *rcp;

	  rcp=cta->rcpts[i];
	  if (rcp->status==m_indata) {
	    rcp->status=m_rejected;
	    rcp->smtpstatus=MALABRT;
	    rcp->why=strdup(cta->info);
	    }
	  }
	}
      phase=999;
      break;
    case   1	:	/*extracting the header			*/
      if ((cta->evl=hdr_parseheader(cta->session,cta->headers))==(EVLTYP *)0) {
	cta->bigtrouble=true;	/*to investigate header trouble	*/
	cta->info=rou_freestr(cta->info);
	cta->info=strdup("Incorrect E-mail Header");
	(void) snprintf(answer,sizeof(answer),eml_ascstat(MALABRT,cta->info));
	phase=-1;	/*big trouble				*/
	}
      else 
	(void) reporthdrs(cta);
      break;
    case   2	:	/*checking if E-mail is a virus		*/
      if (checkvirus(track,cta,&answer)==false) {
	if (cta->rcpts!=(RCPTYP **)0) {
	  int i;

	  for (i=0;cta->rcpts[i]!=(RCPTYP *)0;i++) {
	    switch (cta->rcpts[i]->status) {
	      case m_indata	:
	      case m_relayed	:
	        cta->rcpts[i]->status=m_rejected;
		break;
	      default		:
		break;
              }
	    } 
	  }
        }
      break;
    case   3	: 	/*checking spam level 			*/
      cta->evl->spamlvl=spm_checklevel(cta->dataname);
      if (cta->evl->spamlvl!=(char *)0)
        (void) journalling(false,EVJRL,"Spam-lvl: %s",cta->evl->spamlvl);
      break;
    case   4	: 	/*Displaying the MSG Info 		*/
      (void) journalling(false,EVJRL,"%s: size=%ld,n_error=%d",
			       MSGINF,cta->mtaille,cta->evl->nbrerror);
      break;
    case   5	: 	/*checking bulk level 			*/
      break;
    case   6	: {	/*Preparing and sending EVL report	*/
      char *rsp;

      if ((rsp=getadvice(track,cta))!=(char *)0) {
        int restat;

        if ((restat=checkadvice(cta,rsp))!=CMDOK) {
	  cta->info=rou_freestr(cta->info);
	  cta->info=strdup(rsp);
	  (void) snprintf(answer,sizeof(answer),"%d %s",restat,rsp);
	  phase=-1;	/*trouble trouble			*/
	  }
        rsp=rou_freestr(rsp);
	}
      else
	(void) forceadvice(cta);
      }
      break;
    case   7	:	/*final touch to user status		*/
      if (checkuserstatus(cta)==true) {
	/*Email is too big					*/
	answer=rou_freestr(answer);
	answer=strdup(eml_ascstat(ISBIG));
	}
      break;
    case   8	:	/*making actual E-mail dispatching	*/
      cta->stop=rou_systime();
      (void) sortoutemail(track,cta);
      break;
    default	:	/*lets report finding			*/
      (void) logreport(cta);
      if (cta->evl!=(EVLTYP *)0)
        (void) webreport(cta);
      (void) eml_printfcrlf(cta->tls,answer);
      proceed=false;
      break;
    }
  phase++;
  }
(void) free(answer);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to handle top level protocol	*/
/*	exchange with a remote client.		*/
/*						*/
/************************************************/
static void doproto(TRATYP *track,TLSFILE *tls,LSTTYP *channel)

{
#define	OPDO	"modtra.c:doproto,"

CTTYP *contact;
const char *proname;
int proceed;

contact=eml_gotcontact(tls);
proname="ESMTP";
(void) eml_setsession(tls,contact);
contact->listenport=channel->port;
contact->proto=channel->proto;
switch (contact->proto) {
  case (pro_smtp)	:
    break;
  case (pro_chkid)	:
    proname="CHKID";
    break;
  default		:
    (void) alert(0,"%s Unknown protocol '%d'",OPDO,contact->proto);
    break;
  }
contact->attime=strdup(rou_getstrtime(rou_systime()));
*(strrchr(contact->attime,':'))='\000';	/*removing seconds display	*/
(void) eml_settitle(contact);
(void) eml_printfcrlf(tls,"%d %s %s %s-%s; %s",
                          SIGNON,contact->locname,proname,appname,
		          rou_getvers(),rou_ascsystime(rou_systime()));
proceed=true;
while (proceed==true) {
  if ((contact=eml_domails(contact))==(CTTYP *)0) 
    break;
  switch (contact->code) {
    case s_rcpt		:
      proceed=checkrcpt(track,contact);
      break;
    case s_auth	:
      if ((proceed=checknewid(track,contact))==true) {
        (void) journalling(false,EVJRL,"Remote user authentication OK");
	contact->authok=true;
	}
      break;
    case s_chkid	:
      proceed=checknewid(track,contact);
      break;
    case s_id		:
      contact->dfltcodeok=false;
      if (strcmp(track->dfltcode,contact->info)==0)
        contact->dfltcodeok=true;
      break;
    case s_goodmail	:
      proceed=saygoodemail(track,contact);
      break;
    case s_data		:
      contact->clamvers=rou_freestr(contact->clamvers);
      contact->clamvers=getclamvers(track);
      if (eml_collectdata(contact)==false) {
	contact->code=s_timeout;
	proceed=false;
	break;
	}
      (void) dispatchdata(track,contact);
      (void) eml_resetcontact(contact);
      break;
    case s_reset	:
      (void) eml_resetcontact(contact);
      (void) eml_setsession(tls,contact);
      (void) sleep(1);
      (void) eml_printfcrlf(contact->tls,"%d Reset state",CMDOK);
      break;
    case s_quit		:
      (void) eml_printfcrlf(contact->tls,eml_ascstat(QUITOK,contact->locname));
			/*no BREAK		*/
    case s_timeout	:
      proceed=false;
      break;
    case s_unknown	:
    default		:
      (void) alert(0,"%s code '%d' unexpected (bug?)",OPDO,contact->code);
      break;
    }
  if (quickout==true)
    proceed=false;
  }
contact=eml_itsover(contact);
(void) ipc_purgemsgs(track->boxe,getpid());
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for a connection and	*/
/*	dispatch contact according channel 	*/
/*	protocol.				*/
/*						*/
/************************************************/
static int tracking(TRATYP *track,LSTTYP *channel)

{
#define	OPTR	"modtra.c:tracking,"

int gotone;
TLSFILE *tls;

gotone=false;
if ((tls=tls_accept(channel))!=(TLSFILE *)0) {
  switch (channel->proto) {
    case pro_smtp	:
    case pro_chkid	:
      (void) doproto(track,tls,channel);
      break;
    case pro_unknown	:
      (void) alert(0,"%s undefined protocol",OPTR);
      break;
    }
  gotone=true;
  (void) tls_close(tls);
  }
return gotone;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to initiate a process and wait*/
/*	for incoming connection			*/
/*						*/
/************************************************/
pid_t tra_tracking(TRATYP *track,LSTTYP *channel,int num)

{
#define	OPTC	"modtra.c:tra_tracking,"
#define ACTCYC	"%s now %s [%s:%d] (%02d/%02d/%04d) (pid='%05d')"

pid_t child;

switch((child=rou_fork(false))) { 
  case  -1	:
    (void) alert(0,"%s Unable to start action process "
                   "(error=<%s>)",OPTC,strerror(errno));
    break;
  case   0	:	
      {
      u_int cycle;
  
      cycle=0;
      sigreload=true;
      (void) setjournal(WJRL,d_web,appname);
      memleak=DMTRACE;
      (void) dbg_setnoleak(1);
      (void) journalling(true,APJRL,ACTCYC,"Modtra","Starting",
				    channel->ipnum,
		         	    channel->port,num+1,
				    channel->iteration,cycle,getpid());
      while (quickout==false) {
        (void) settitle("%s waiting on [%s:%d] (%02d/%02d/%04d)",
		         appname,channel->ipnum,
		         channel->port,num+1,channel->iteration,cycle);
        if (trareload(track->boxe)==false) {
          (void) alert(0,"%s giving up about reloading (big trouble?)",
		         OPTC);
          break;
	  }
        if (tracking(track,channel)==true) {
	  cycle++;
	  if (debug>0) {
            (void) journalling(true,APJRL,ACTCYC,"Modtra","cycling",
				          channel->ipnum,
		         	          channel->port,num+1,
				          channel->iteration,cycle,getpid());
	    }
	  }
        if (quickout==true)
	  break;
	else {
	  TIMESPEC timeout;

          timeout.tv_sec=0;
          timeout.tv_nsec=50000;
          while (ipc_waitpost(track->boxe,&timeout)==true) {
	    char *caller;
	    char *rqst;
	    u_long msec;

	    if ((rqst=ipc_getpost(track->boxe,&caller,&msec))!=(char *)0) {
	      (void) alert(0,"action, Droping caller->msg=<%s->%s>",
			      caller,rqst);
	      caller=rou_freestr(caller);
  	      rqst=rou_freestr(rqst);
	      }
	    }
	  /*to make sure to have a new uniqID next time			*/
	  (void) nanosleep(&timeout,&timeout);
	  }
        }
      (void) journalling(true,APJRL,ACTCYC,"Modtra","exiting",
				    channel->ipnum,
		         	    channel->port,num+1,
				    channel->iteration,cycle,getpid());
      /*Call to check any memory leak if memleak set to 'true',	*/
      /*request done in debug mode only.			*/
      (void) dbg_dumpmem(appname,"modtra");
      memleak=DMFALSE;
      (void) closejournals();
      }
    (void) exit(0);
    break;
  default	:
    break;
  }
return child;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to allocate memory for a	*/
/*	tracking structure.			*/
/*						*/
/************************************************/
TRATYP *tra_maketrack(BOXTYP *boxe)

{
TRATYP *track;

track=(TRATYP *)calloc(1,sizeof(TRATYP));
track->boxe=boxe;
return track;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a	*/
/*	TRTYP structure				*/
/*						*/
/************************************************/
TRATYP *tra_freetrack(TRATYP *track)

{
if (track!=(TRATYP *)0) {
  track->dfs=eml_freeonedfs(track->dfs);
  track->dfltcode=rou_freestr(track->dfltcode);
  track->skrlst=cnf_freeskrlst(track->skrlst);
  track->chkidsrv=rou_freestr(track->chkidsrv);
  track->chkidmod=rou_freestr(track->chkidmod);
  (void) free(track);
  track=(TRATYP *)0;
  }
return track;
}
