/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	SUBROU:					*/
/*	Low level module, take care of all the	*/
/*	low level routine used by the safemail	*/
/*	application.				*/
/*						*/
/************************************************/
#include	<errno.h>
#include	<string.h>
#include	<syslog.h>
#include	<stdarg.h>
#include	<stdlib.h>
#include	<ctype.h>
#include	<unistd.h>
#include	<signal.h>
#include	<sys/time.h>
#include	<sys/file.h>
#include	<sys/stat.h>
#include	<sys/resource.h>
#include	<sys/prctl.h>
#include	<sys/wait.h>
#include	"subrou.h"

/*current debug level				*/
	int debug=0;
/*flag application running background/foreground*/
	int foreground=false;
/*debug log in verbose mode			*/
	int verbose=false;
/*base directory (where to store internal data)	*/
	char	*base=""; /*no prefix		*/

/*application name				*/
	char	*appname=CLEMENT;

/*data channel ready    			*/
	int	gotsigio=false;	
/*application reload request			*/
	int	sigreload=false;      
/*flag to request a quick exit from application	*/
	int	quickout=false;

#define	REVNUM	"$Revision: 2.1.0.401 $"

/*
^L
*/
/************************************************/
/*	Module static data			*/
/************************************************/
typedef	struct	{/*title space definition (proc)*/
	int max;	/*title max size	*/
	char *title;	/*title storage area	*/
	}TITLTYP;

typedef	struct	{/*journalling information	*/
	time_t stamp;	/*press stamp		*/
	char *news;	/*press information	*/
	}PRESSTYP;

typedef struct	{
	PRESSTYP **pressbook;	/*news to log	*/
	DIRENUM dir;		/*storing area	*/
	char *edition;		/*jrl name	*/
	}JRLTYP;

/*storage area for /proc title display		*/
static	TITLTYP *title=(TITLTYP *)0;

/*area to store journal information		*/
static	JRLTYP	jrl[MXJRL];
/*

*/
/************************************************/
/*						*/
/*	Procedure to journalling directory	*/
/*						*/
/************************************************/
static const char *getjlog(DIRENUM dir)

{
/*where to store journals			*/
#define	JLOG	"logs/"	
#define	WLOG	"wlogs/"	

const char *jlog;

jlog=(const char *)0;
switch (dir) {
  case d_web	:
    jlog=WLOG;
    break; 
  default	:
    jlog=JLOG;
    break; 
  }
return jlog;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to application+version name	*/
/*						*/
/************************************************/
static char *getapvers()

{
static char *apvers=(char *)0;

if (apvers==(char *)0) {
  static char apinfo[50];

  char *ptr;
  char version[30];

  (void) strcpy(version,rou_getvers());
  if ((ptr=strchr(version,'-'))!=(char *)0)
    *ptr='\000';
  (void) snprintf(apinfo,sizeof(apinfo),"%s-%s/",appname,version);
  apvers=apinfo;
  }
return apvers;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to catch signal and do what is*/
/*	needed.					*/
/*						*/
/************************************************/
static void sigalrm(int sig)

{
struct sigaction sa;
sigset_t blockmask;
sigset_t oldset;

(void) sigfillset(&blockmask);
(void) sigprocmask(SIG_SETMASK,&blockmask,&oldset);
(void) tracing(TRALOG,10,"sigalrm: signal <%s> received by pid='%d'",
	       		  sys_siglist[sig],getpid());
switch (sig)
  {
  case SIGIO 	 :	/*data received by socket	*/
    sa.sa_handler=SIG_IGN;
    gotsigio=true;
    break;
  case SIGBUS	 :
  case SIGSEGV   :
    (void) core_dump("Program memory fault");
    break;
  case SIGALRM   :
    break;
  case SIGHUP    :
    sigreload=true;
    break;
  case SIGPROF   :
  case SIGINT    :
  case SIGQUIT   :
    quickout=true;
    break;
  case SIGTERM   :
    quickout=true;
    break;
  case SIGUSR1   :
    debug++;
    (void) tracing(TRALOG,0,"Increasing debug Level, now '%d'",debug);
    break;
  case SIGUSR2   :
    debug--;
    if (debug<0)
      debug=0;
    (void) tracing(TRALOG,0,"Decreasing debug Level, now '%d'",debug);
    break;
  default        :
    (void) tracing(TRALOG,0,"Unexpected signal <%s> received",
			     sys_siglist[sig]);
    break;
  }
(void) sigprocmask(SIG_SETMASK,&oldset,(sigset_t *)0);
sa.sa_handler=sigalrm;
(void) sigemptyset(&sa.sa_mask);
sa.sa_flags=0;
(void) sigaction(sig,&sa,(struct sigaction *)0);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to trap the SIGIO signal	*/
/*						*/
/************************************************/
static void trappesigio(int trappe)

{
struct sigaction sa;

(void) sigemptyset(&sa.sa_mask);
sa.sa_flags=SA_RESTART;
if (trappe==true)
  sa.sa_handler=sigalrm;
else
  sa.sa_handler=SIG_IGN;
(void) sigaction(SIGIO,&sa,(struct sigaction *)0);
}
/*

*/
/************************************************/
/*						*/
/*	procedure to flush a pressbook contents	*/
/*						*/
/************************************************/
static void flushjournal(int numjrl)

{
u_long curday;
struct stat bufstat;
FILE *journal;
JRLTYP *jr;
char *jlog;
char ext[50];
char filename[300];

jr=jrl+numjrl;
if (jr->pressbook==(PRESSTYP **)0)
  return;		/*no time waste		*/
curday=rou_normdate(rou_systime());
(void) snprintf(ext,sizeof(ext),"-%ld.jrl",curday);
jlog=(char *)getjlog(jr->dir);
(void) rou_mkname(filename,sizeof(filename),jr->dir,jlog,jr->edition,ext);
if (stat(filename,&bufstat)<0) {
  int handle;

  if ((handle=creat(filename,0644))<0) {
    jrl[0].edition=(char *)0;
    (void) core_dump("Unable to create journal <%s> (error=<%s>)",
		      filename,strerror(errno));
    }
  (void) close(handle);
  }
if ((journal=fopen(filename,"a+"))==(FILE *)0) {
  char curdir[512];

  if (getcwd(curdir,sizeof(curdir))==(char *)0)
    (void) snprintf(curdir,sizeof(curdir),"Unknown directory! (bug?)");
  (void) alert(0,"Unable to open journal '%s'\n(strerror='%s',curdir='%s')",
	            filename,strerror(errno),curdir);
  } 
else {
  if (jr->pressbook!=(PRESSTYP **)0) {
    time_t start;
    int i;

    start=(time_t)0;
    (void) flock(fileno(journal),LOCK_EX);
    for (i=0;jr->pressbook[i]!=(PRESSTYP *)0;i++) {
      char logtime[40];

      if (start==0) {
	struct tm *ttime;

	start=jr->pressbook[i]->stamp;
        ttime=localtime(&start);
        (void) snprintf(logtime,sizeof(logtime),"%02d:%02d:%02d",
			  		        ttime->tm_hour,ttime->tm_min,
					        ttime->tm_sec);
        }
      else {
	int delay;
        int j;

	delay=jr->pressbook[i]->stamp-start;
	(void) strcpy(logtime,"");
	for (j=0;j<3;j++) {
	  char strloc[30];

          strloc[0]='\000';
	  switch (j) {
	    case  0  :
	      if (delay>=3600)
		(void) sprintf(strloc,"+%d:%02d:%02d",
					delay/3600,(delay/60)%60,delay%60);
	      break;
	    case  1  :
	      if (delay>=60)
		(void) sprintf(strloc,"+%02d:%02d",(delay/60)%60,delay%60);
	      break;
	    case  2  :
	      (void) sprintf(strloc,"+%02d",delay%60);
	      break;
            }
	  if (strloc[0]!='\000') {
	   (void) sprintf(logtime,"%8s",strloc);
	   break;
	   }
          }
        }
      (void) fprintf(journal,"%-10s%s\n",logtime,jr->pressbook[i]->news);
      (void) free(jr->pressbook[i]->news);
      (void) free(jr->pressbook[i]);
      }
    (void) fflush(journal);
    (void) flock(fileno(journal),LOCK_UN);
    (void) free(jr->pressbook);
    jr->pressbook=(PRESSTYP **)0;
    }
  (void) fclose(journal);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to prepare a string according	*/
/*	a format sequence and variadic parameter*/
/*	pointer return a dynamically allocated	*/
/*	apce, need to be release after usage.	*/
/*						*/
/************************************************/
char *rou_dynavsprintf(const char *fmt,va_list args)

{
int proceed;
int taille;
char *local;

proceed=true;
taille=64;
local=(char *)calloc(taille,sizeof(char));
while (proceed==true) {
  va_list ap;
  int parnum;

  proceed=false;
  va_copy(ap,args);
  if ((parnum=vsnprintf(local,taille,fmt,ap))>=taille) {
    taille=parnum+1;
    local=(char *)realloc((void *)local,(taille+1)*sizeof(char));
    proceed=true;
    }
  va_end(ap);
  }
return local;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure set the signal trap		*/
/*						*/
/************************************************/
void rou_setalrm()

{
struct sigaction sa;

(void) sigemptyset(&sa.sa_mask);
sa.sa_flags=0;
sa.sa_handler=sigalrm;
(void) sigaction(SIGSEGV,&sa,(struct sigaction *)0);
(void) sigaction(SIGBUS,&sa,(struct sigaction *)0);
(void) sigaction(SIGINT,&sa,(struct sigaction *)0);
(void) sigaction(SIGQUIT,&sa,(struct sigaction *)0);
(void) sigaction(SIGHUP,&sa,(struct sigaction *)0);
(void) sigaction(SIGTERM,&sa,(struct sigaction *)0);
(void) sigaction(SIGALRM,&sa,(struct sigaction *)0);
(void) sigaction(SIGUSR1,&sa,(struct sigaction *)0);
(void) sigaction(SIGUSR2,&sa,(struct sigaction *)0);
(void) trappesigio(false);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract and return current	*/
/*	version number.				*/
/*						*/
/************************************************/
char *rou_getvers()

{
static char *vernum=(char *)0;

if (vernum==(char *)0) {
  static char numero[30];
  int num1,num2,num3,num4;
  char revision[30];

  (void) sscanf(REVNUM,"%s %d.%d.%d.%d",revision,
				        &num1,&num2,&num3,&num4);
  (void) sprintf(numero,"%d.%d-%d",num1,num2,num4);
  vernum=numero;
  }
return vernum;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to extract and return current	*/
/*	version number as an INTEGER.		*/
/*						*/
/************************************************/
int rou_getnumvers()

{
int numvers;
int num1,num2,num3,num4;
char revision[30];

numvers=0;
if (sscanf(REVNUM,"%s %d.%d.%d.%d",revision,&num1,&num2,&num3,&num4)==5) {
  numvers=num1;
  numvers=(numvers*10)+num2;
  numvers=(numvers*1000)+num4;
  }
return numvers;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to 'compute' a new base 	*/
/*	according the current directory		*/
/*	return a newly allocated memory		*/
/*						*/
/************************************************/
char *rou_mkbase(char *newbase)

{
char *ptr;

ptr=(char *)0;
switch (newbase[0]) {
  case	'.'	: {
    char curdir[512];
    char newpath[1000];

    if (getcwd(curdir,sizeof(curdir))!=(char *)0) {
      (void) snprintf(newpath,sizeof(newpath),"%s/%s",curdir,newbase);
      if (chdir(newpath)<0) {
        (void) alert(0,"Unable to reach directory <%s>",newpath);
	break;	/*immediat exit	*/
        }
      if (getcwd(newpath,sizeof(newpath))==(char *)0)
        (void) snprintf(newpath,sizeof(newpath),"Unknown directory! (bug?)");
      if (chdir(curdir)<0) {
        (void) alert(0,"Unable to reach directory <%s>",curdir);
	break;	/*immediat exit	*/
        }
      ptr=strdup(newpath);
      } 
    } 
    break;
  case	'/'	:
    ptr=strdup(newbase);
    break;
  default	:
    break;
  }
if (ptr==(char *)0) {
  (void) alert(0,"Unable to assign new base '%s', immediat exit",newbase);
  (void) exit(-1);
  }
return ptr;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to make a mentor type name	*/
/*						*/
/************************************************/
char *rou_mkname(char *fullname,int max,DIRENUM dir,char *subdir,
		 const char *filename,char *ext)

{
char *sysbase;
char *apvers;

sysbase=" ";	/*something impossible		*/
apvers=getapvers();
switch (dir) {
  case (d_null)		:
    sysbase="";
    apvers=""; 
    break;
  case (d_tmp)		:
    sysbase="/var/tmp";
    apvers=CLEMENT"/"; 
    break;
  case (d_crash)	:
    sysbase="/var/crash";
    break;
  case (d_tls)		:
    sysbase="/etc/pki";
    break;
  case (d_etc)		:
    sysbase="/etc";
    break;
  case (d_spool)	:
    sysbase="/var/spool";
    break;
  case (d_lock)		:
    sysbase="/var/lock";
    break;
  case (d_log)		:
    sysbase="/var/spool";
    break;
  case (d_web)		:
    sysbase="/var/spool";
    break;
  case (d_bin)		:
    sysbase="/usr/bin";
    apvers=""; 
    break;
  case (d_varlib)	:
    sysbase="/var/lib";
    break;
  case (d_usrlib)	:
    sysbase="/usr/lib";
    break;
  }
(void) snprintf(fullname,max,"%s%s/%s%s%s%s",
			      base,sysbase,apvers,subdir,filename,ext);
return fullname;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to convert a string to a code,*/
/*	remaining of the string is copy at the	*/
/*	beginning.				*/
/*	Return a pointer to table, never a NULL	*/
/*	Pointer unless the table itself is NULL	*/
/*						*/
/************************************************/
VOCTYP *rou_getvoca(const VOCTYP *table,char *str)

{
VOCTYP *voc;

voc=(VOCTYP *)0;
if (table!=(VOCTYP *)0) {
  int i;
  char *sptr;
 
  sptr=str;
  while (isspace(*sptr)!=0)
    sptr++;
  for (i=0;table[i].key!=(char *)0;i++) {
    int max;

    max=strlen(table[i].key);
    if (strncasecmp(table[i].key,sptr,max)==0) {
      if ((sptr[max]!='\000')&&(isalnum(sptr[max])!=0))
        continue;/*it is not the right word	*/
      sptr+=max;
      while (isspace(*sptr)!=0)
	sptr++;
      (void) memmove(str,sptr,strlen(sptr)+1);
      voc=(VOCTYP *)(table+i);
      break;
      }
    }
  if (voc==(VOCTYP *)0)
   voc=(VOCTYP *)(table+i);/*return the "unknown" info	*/
  }
return voc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get the current system time*/
/*						*/
/************************************************/
time_t rou_systime()

{
return time((long *)0);
}
/*

*/
/************************************************/
/*						*/
/*	Return normalised date as in 20041231	*/
/*						*/
/************************************************/
u_long rou_normdate(time_t timestamp)

{
struct tm *ttime;

ttime=localtime(&timestamp);
return (((u_long)(ttime->tm_year+1900)*10000)+
		 (ttime->tm_mon+1)*100)+
		  ttime->tm_mday; 
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the local time in	*/
/*	HH:MM:SS format.			*/
/*	Time is stored in a STATIC memeory area	*/
/*						*/
/************************************************/
char *rou_getstrtime(time_t curtime)

{
static char timeloc[30];

struct tm *ttime;

(void) strcpy(timeloc,"");
ttime=localtime(&curtime);
(void) snprintf(timeloc,sizeof(timeloc),
                        "%02d:%02d:%02d",
                        ttime->tm_hour,ttime->tm_min,ttime->tm_sec);
return timeloc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the local time in	*/
/*	YYYY-MM-DD format.			*/
/*	Time is stored in a STATIC memeory area	*/
/*						*/
/************************************************/
char *rou_getstrdate(time_t curtime)

{
static char dateloc[40];

struct tm *ttime;

(void) strcpy(dateloc,"");
ttime=localtime(&curtime);
(void) snprintf(dateloc,sizeof(dateloc),"%04d-%02d-%02d",
                        ttime->tm_year+1900,ttime->tm_mon+1,ttime->tm_mday);
return dateloc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the local time in	*/
/*	YYYY-MM-DD HH:MM:SS format.		*/
/*	Variable is returned in static area.	*/
/*						*/
/************************************************/
char *rou_getstrdatetime(time_t curtime)

{
static char timeloc[40];

struct tm *ttime;

(void) strcpy(timeloc,"");
ttime=localtime(&curtime);
(void) snprintf(timeloc,sizeof(timeloc),
                        "%04d-%02d-%02d %02d:%02d:%02d",
                        ttime->tm_year+1900,ttime->tm_mon+1,ttime->tm_mday,
                        ttime->tm_hour,ttime->tm_min,ttime->tm_sec);
return timeloc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to transform the local system	*/
/*	time in ASCII time.			*/
/*	Stored in STATIC memory area.		*/
/*						*/
/************************************************/
char *rou_ascsystime(time_t curtime)

{
#define TFORM	"%a, %d %b %Y %H:%M:%S"

static char asctemps[100];

struct tm *tminfo;
char asct[80];

tminfo=localtime(&curtime);
(void) strftime(asct,sizeof(asct),TFORM,tminfo);
(void) snprintf(asctemps,sizeof(asctemps),"%s %05ld",asct,
					  ((tminfo->tm_gmtoff/3600)*100)+
					  ((tminfo->tm_gmtoff%3600)/60));
return asctemps;
}
/*
^L
*/
/************************************************/
/*						*/
/*      Procedure to add a string to pointer	*/
/*	holding another string, needed memory	*/
/*	is dynamicaly allocated.		*/
/*						*/
/************************************************/
char *rou_memstrcat(char *msg,char *fmt,...)

{
char *tmp;
va_list args;

va_start(args,fmt);
if (msg==(char *)0)
  msg=(char *)calloc(10,sizeof(char));
tmp=rou_dynavsprintf(fmt,args);
msg=(char *)realloc(msg,strlen(msg)+strlen(tmp)+5);
(void) strcat(msg,tmp);
(void) free(tmp);
va_end(args);
return msg;
}
/*
^L
*/
/************************************************/
/*						*/
/*      Procedure to count the number of vector	*/
/*	within a vector list.			*/
/*						*/
/************************************************/
int rou_vecsize(void **lst)

{
int num;

num=0;
if (lst!=(void **)0)
  for (num=0;lst[num]!=(void *)0;num++);
return num;
}
/*
^L
*/
/************************************************/
/*						*/
/*      Procedure to add a pointer to a pointer	*/
/*	list.					*/
/*						*/
/************************************************/
void **rou_addveclst(void **lst,void *info)

{
int num;

num=0;
if (lst!=(void **)0) {
  num=rou_vecsize(lst); 
  lst=(void **)realloc(lst,(num+2)*sizeof(void *));
  }
else
  lst=(void **)calloc(1,2*sizeof(void *));
lst[num]=info;
lst[num+1]=(void *)0;
return lst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to clean a string and return	*/
/*	a null pointer.				*/
/*						*/
/************************************************/
char *rou_freestr(char *str)

{
if (str!=(char *)0) {
  (void) free(str);
  str=(char *)0;
  }
return str;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a	*/
/*	pointer to a list of string.		*/
/*						*/
/************************************************/
char **rou_freestrlst(char **strlst)

{
if (strlst!=(char **)0) {
  int i;

  for (i=0;strlst[i]!=(char *)0;i++)
    strlst[i]=rou_freestr(strlst[i]);
  (void) free(strlst);
  strlst=(char **)0;
  }
return strlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure extract line from a file,	*/
/*	forget about any line starting with	*/
/*	carcom character (if carcom character	*/
/*	not null).				*/
/*	Doesn't return the '\r' and '\n' 	*/
/*	character.				*/
/*						*/
/************************************************/
char *rou_getstr(FILE *fichier,char *str,int taille,char carcom)

{
char *strloc;

(void) memset(str,'\000',taille);
while ((strloc=fgets(str,taille,fichier))!=(char *)0) {
  char *ptrloc;

  if (carcom!='\000') {
    if (str[0]==carcom)
      continue;
    ptrloc=str;
    while ((ptrloc=strchr(ptrloc,carcom))!=(char *)0) {
      if (*(ptrloc-1)=='\\') {
        (void) memmove(ptrloc-1,ptrloc,strlen(ptrloc)+1);	
        ptrloc++;
        }
      else {
        *ptrloc='\000';
        break;
        }
      }
    }
  ptrloc=strloc+strlen(strloc);
  while (ptrloc!=(char *)0) {
    ptrloc--;
    switch (*ptrloc) {
      case '\n'     :
      case '\r'     :
        *ptrloc='\000';
        break;
      default       :
        ptrloc=(char *)0;
	break;
      }
    }
  break;
  }
return strloc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to add a file contents to	*/
/*	the env area.				*/
/*						*/
/************************************************/
void rou_addtoenv(const char *filename)

{
FILE *fichier;

if ((fichier=fopen(filename,"r"))!=(FILE *)0) {
  char line[1000];

  while (rou_getstr(fichier,line,sizeof(line),'#')!=(char *)0) {
    char *ptr;
    char var[100];
    char contents[200];

    (void) memset(var,'\000',sizeof(var));
    (void) memset(contents,'\000',sizeof(contents));
    if (strlen(line)==0)	/*empty line	*/
      continue;
    if (sscanf(line,"%100[^=]=%199s",var,contents)!=2) {
      (void) alert(0,"subrou.c,addtoenv, <%s> invalid format",line);
      continue;
      }
    if (contents[0]=='"') {
      register char *pstr;

      pstr=contents+1; 
      (void) memmove(contents,pstr,strlen(pstr)+1);
      if ((ptr=strchr(contents,'"'))!=(char *)0)
        *ptr='\000';
      } 
    (void) sprintf(line,"%s=%s",var,contents);
    ptr=strdup(line);
    if (putenv(ptr)<0) {
      (void) alert(0,"subrou.c,addtoenv, unable to add '%s' "
		     "to env (error=<%s>)",line,strerror(errno));
      }
    } 
  }
else
  (void) alert(0,"subrou.c,addtoenv, unable to open file '%s' "
		 "(error=<%s>)",filename,strerror(errno));
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to add local config to env	*/
/*						*/
/************************************************/
void rou_loadconf(char *application)

{
char filename[1000];

(void) rou_mkname(filename,sizeof(filename),d_etc,"",application,".conf");
(void) rou_addtoenv(filename);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return allways 0 (no fork)	*/
/*	if foreground is set to true		*/
/*						*/
/************************************************/
pid_t rou_fork(int foregrnd)

{
pid_t child;

child=(pid_t)0;
if (foregrnd==false)
  child=fork();
return child;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to exit only if foreground is */
/*	set to false;				*/
/*						*/
/************************************************/
void rou_exit(int foregrnd,int status)

{
if (foregrnd==false) {
  (void) exit(status);
  }
}
/*

*/
/************************************************/
/*						*/
/*      procedure d'attente de l'arret du 	*/
/*	processu 'child'.			*/
/*						*/
/************************************************/
int rou_waitforchild(pid_t chldpid,char *cmt)

{
int done;

done=false;
switch (waitpid(chldpid,(int *)0,0)) {
  case -1		:
    switch (errno) {
      case ECHILD	:	/*Process with no other child   */
        break;			/*do not care			*/
      default       	:
        (void) alert(0,"%s Waitpid error=<%s>",cmt,strerror(errno));
        break;
      }
    break;
  case  0		:
    (void) alert(0,"%s no child to wait",cmt);
    break;
  default	:
    done=true;
    break;
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to terminate all child process*/
/*	refered in a process list. if not	*/
/*	successful 'delay'  after SIGTERM try,	*/
/*	a SIGKILL signal is applied.		*/
/*						*/
/************************************************/
void rou_childkill(pid_t *process,int num,int delay)

{
if (process!=(pid_t *)0) {
  int i;

  for (i=0;i<num;i++) {
    if (process[i]<=1)	/*no way to kill admin process	*/
      continue;
    (void) alarm(delay);
    (void) kill(process[i],SIGTERM);
    (void) rou_waitforchild(process[i],"subrou.c:rou_childkill,");
    (void) alarm(0);
    if (kill(process[i],SIGCHECK)==0) {
      (void) kill(process[i],SIGKILL);
      (void) tracing(TRALOG,2,"sending SIGKILL to process '%05d'",process[i]);
      }
    }
  (void) rou_nozombie();
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to change the current ID to	*/
/*	a application working ID.		*/
/*						*/
/************************************************/
uid_t rou_getappid(char *appnom,int group)

{
#define MAXPATH 300

uid_t id;
char *path;
struct stat bufstat;

id=(uid_t)0;
path=(char *)calloc(1,MAXPATH+2);
(void) rou_mkname(path,MAXPATH,d_bin,"",appnom,"");
if (stat(path,&bufstat)<0) {
  (void) alert(0,"%s Unable to get <%s> stat (bug?)",
		 "basys.c:rou_getappid",path);
  }
else {
  if (group==false)
    id=bufstat.st_uid;
  else
    id=bufstat.st_gid;
  }
(void) free(path);
return id;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to change the current ID to	*/
/*	a mentor working ID.			*/
/*						*/
/************************************************/
void rou_chgid(char *appnom)

{
#define	OPID	"subrou.c:rou_chgid,"
#define	CHGMSG \
  "%s Unable to %s to '%d' (error=<%s> (cur u/g id='%d/%d',eid='%d/%d') bug?)"

struct rlimit limites;

if (getuid()==0) {	/*is changing ID possible	*/
  uid_t uid;
  uid_t gid;

  gid=rou_getappid(appnom,true);
  uid=rou_getappid(appnom,false);
  if (setegid(gid)<0)
    (void) alert(0,CHGMSG,OPID,"setegid",gid,
				strerror(errno),
				getuid(),getgid(),
				geteuid(),getegid());
  if (seteuid(uid)<0)
    (void) alert(0,CHGMSG,OPID,"seteuid",uid,
				strerror(errno),
				getuid(),getgid(),
				geteuid(),getegid());
  }
if (getrlimit(RLIMIT_CORE,&limites)<0) {
  (void) alert(0,"getrlimit error='%s'",strerror(errno));
  }
limites.rlim_cur=limites.rlim_max;
if (setrlimit(RLIMIT_CORE,&limites)<0) {
  (void) alert(0,"setrlimit error='%s'",strerror(errno));
  }
(void) prctl(PR_SET_DUMPABLE,1,0,0,0);/*to allow core-dump	*/
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to fully change current ID,	*/
/*	(needed before doing a 'system()' call).*/
/*						*/
/************************************************/
void rou_fullid()

{
int euid;
int egid;

egid=getegid();
euid=geteuid();
(void) setegid(getgid());
(void) seteuid(getuid());
(void) setgid(egid);
(void) setuid(euid);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to set a string to lower case	*/
/*	only.					*/
/*						*/
/************************************************/
char *rou_strtolower(char *str)

{
if (str!=(char *)0) {
  int i;

  for (i=0;str[i]!='\000';i++)
    str[i]=(char)tolower((int)str[i]);
  }
return str;
}
/*
^L
*/
/************************************************/
/*						*/
/*      Procedure to return the current time	*/
/*	expressed with a millisecond precision.	*/
/*						*/
/************************************************/
u_int64_t rou_getmillisec()

{
u_int64_t millisec;
struct timeval newtime;

millisec=(u_int64_t)0;
(void) gettimeofday(&newtime,(struct timezone *)0);
millisec=newtime.tv_sec;	/*/4194304= 2^22*/
millisec*=1000;
millisec+=(newtime.tv_usec/1000);
return millisec;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to make create a unique name	*/
/*	can be used to store process data	*/
/*	name return is dynamically allocated.	*/
/*						*/
/************************************************/
char *rou_uniquename()

{
char *name;

name=(char *)calloc(1,60);
(void) sprintf(name,"%05d%08lx",getpid(),(u_long)rou_getmillisec());
return name;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait for waiting second	*/
/*	at max, quit if quickout or reload are	*/
/*	set to true.				*/
/*						*/
/************************************************/
int rou_passetime(int waiting)

{
int passed;
int i;

passed=true;
for (i=0;i<waiting;i++) {
  (void) sleep(1);
  if ((quickout==true)||(sigreload==true)) {
    passed=false;
    break;
    }
  }
return passed;
}
/*
^L
*/
/************************************************/
/*						*/
/*	Procedure to check if a child process	*/
/*	is still alive.				*/
/*						*/
/************************************************/
int rou_checkprocess(pid_t pid)

{
int status;

status=false;
switch(pid) {
  case (pid_t)0	:	/*0 means no process	*/
    status=false;
    break;
  case (pid_t)1	:	/*init process always OK*/
    status=true;
    break;
  default	:	/*standard process	*/
    if (kill(pid,SIGCHECK)==0)
      status=true;
    break;
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to create all the directory	*/
/*	needed by a filename.			*/
/*						*/
/************************************************/
int rou_createdirs(char *dirname)

{
struct stat bufstat;
int newdir;

newdir=false;
if (dirname[strlen(dirname)-1]=='/')
  dirname[strlen(dirname)-1]='\000';
if (stat(dirname,&bufstat)==0) {
  if (S_ISDIR(bufstat.st_mode)==0) {
    (void) code_fault("subrou.c:rou_createdirs, file <%s> is NOT a "
		      "directory <%s> (bug!?)");
    }
  }
else {
  char *ptr;
  char dirloc[300];

  (void) strncpy(dirloc,dirname,sizeof(dirloc));
  if ((ptr=strrchr(dirloc,'/'))!=(char *)0) {
    *ptr='\000';
    ptr++;
    if (strlen(ptr)>0)
      newdir|=rou_createdirs(dirloc);
    }
  if (mkdir(dirname,0751)<0) {
    (void) code_fault("subrou.c:rou_createdirs, unable to create "
		      "directory <%s> (error=<%s>)",dirname,strerror(errno));
    }
  else
    newdir=true;
  }
return newdir;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to check if file has been	*/
/*	updated since 'tempo' time.		*/
/*	return status:				*/
/*	-1	: file missing			*/
/*	 0	: file is not new		*/
/*	 1	: file is new			*/
/*						*/
/************************************************/
int rou_isnewfile(char *filename,time_t tempo)

{
int status;
int i;

status=-1;
for (i=0;i<10;i++) {
  struct stat bufstat;

  if (stat(filename,&bufstat)==0) {
    status=0;
    if (bufstat.st_mtime>tempo)
      status=1; 
    break;
    }
  (void) usleep(1000);	/*waiting file to be updated	*/
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to append a file contents to	*/
/*	a file previously open.			*/
/*						*/
/************************************************/
int rou_appenfile(FILE *fout,char *filename)

{
#define	OPAP	"subrou.c:rou_appenfile,"

int done;
FILE *fin;
int phase;
int proceed;

done=false;
fin=(FILE *)0;
phase=0;
proceed=true;
while (proceed==true) {
  switch (phase) {
    case  0	:	/*opening file		*/
      if ((fin=fopen(filename,"r"))==(FILE *)0) {
	(void) alert(0,"%s unable to open appened file <%s> (error=<%s>)",
			OPAP,filename,strerror(errno));
        proceed=false;
	}
      break;
    case 1	:	/*appening file		*/
      if (fin!=(FILE *)0) {
	int got;
	char buffer[1024];

        do {
	  got=fread(buffer,sizeof(char),sizeof(buffer),fin);
	  if (got>0)
	    if (fwrite(buffer,sizeof(char),got,fout)<got) {
	      (void) alert(0,"%s to write data from <%s> (error=<%s>)",
			      OPAP,filename,strerror(errno));
	      }
	  } while (got==sizeof(buffer));
        }
      break;
    case 2	:	/*closing file		*/
      (void) fclose(fin);
      done=true;
      break;
    default	:	/*SAFE guard		*/
      proceed=false;
      break;
    }
  phase++;
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to allow exited child process	*/
/*	to leave the zombie stat.		*/
/*						*/
/************************************************/
void rou_nozombie()

{
while (waitpid(-1,(int *)0,WNOHANG)>0);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to display a message		*/
/*	either on stderr or in the log file	*/
/*						*/
/************************************************/
void tracing(const int report,const int dlevel,const char *fmt,...)

{
if (debug>=dlevel)
  {
  va_list args;
  register int dummycnt;
  char *strloc;

  va_start(args,fmt);
  dummycnt=vasprintf(&strloc,fmt,args);
  if ((foreground==true)||((verbose!=false)&&(report==TRALOG)))
    (void) fprintf(stderr,"%s %s\n",rou_getstrtime(rou_systime()),strloc);
  else {
    static int already;

    if (already==false) {
      already=true;	/*syslog is not reentrant	*/
      (void) syslog(report,"%s\n",strloc);
      already=false;
      }
    }
  (void) free(strloc);
  va_end(args);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to display argv in debug log	*/
/*	file.					*/
/*						*/
/************************************************/
void argvtracing(const int report,const int dlevel,
		 const char *fmt,char *argv[])

{
if (debug>=dlevel) {
  int i;
  char *tmp;

  tmp=(char *)calloc(1,20);
  for (i=0;argv[i]!=(char *)0;i++) {
    tmp=(char *)realloc(tmp,strlen(tmp)+strlen(argv[i])+10);
    (void) strcat(tmp,argv[i]);
    (void) strcat(tmp," ");
    }
  (void) tracing(report,dlevel,fmt,tmp);
  (void) free(tmp);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to display an alert in the	*/
/*	debug log.				*/
/*						*/
/************************************************/
void alert(const int dlevel,const char *fmt,...)

{
if (debug>=dlevel)
  {
  va_list args;
  register int dummycnt;
  char *strloc;

  va_start(args,fmt);
  dummycnt=vasprintf(&strloc,fmt,args);
  (void) tracing(TRALOG,dlevel,"Alert! %s",strloc);
  (void) free(strloc);
  va_end(args);
  }
}
/*

*/
/************************************************/
/*						*/
/*	procedure to set the new journal edition*/
/*	return 0 if successfull -1 otherwise.	*/
/*						*/
/************************************************/
int setjournal(int numjrl,DIRENUM dir,char *today)

{
int status;

status=-1;
if (numjrl<MXJRL) {
  JRLTYP *jr;

  jr=jrl+numjrl;
  (void) flushjournal(numjrl);
  jr->dir=dir;
  jr->edition=rou_freestr(jr->edition);
  jr->edition=strdup(today);
  status=0;
  }
return status;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to compose a journal name,	*/
/*	getjourname will return a name to be	*/
/*	stored in LOGSDIR within the base	*/
/*	application.				*/
/*						*/
/************************************************/
char *getjourname(char *journame,int taille,DIRENUM dir,
		  char *edition,long curday)

{
char ext[30];

(void) snprintf(ext,sizeof(ext),"-%ld.jrl",curday);
(void) rou_mkname(journame,taille,dir,(char *)getjlog(dir),edition,ext);
return journame;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to close all journal, flushing*/
/*	all backlog news.			*/
/*						*/
/************************************************/
void closejournals()

{
int i;

for (i=0;i<MXJRL;i++) {
  JRLTYP *jr;

  jr=jrl+i;
  if (jr->edition!=(char *)0) {
    (void) flushjournal(i);
    jr->edition=rou_freestr(jr->edition);
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	procedure to store event in the logs	*/
/*	book.					*/
/*						*/
/************************************************/
void journalling(int logflush,int numjrl,char *frm,...)

{
va_list args;
JRLTYP *jr;

jr=jrl+numjrl;
if (logflush==true)
  (void) flushjournal(numjrl);
va_start(args,frm);
if ((frm!=(char *)0)&&(strlen(frm)>0)) {
  register int dummycnt;
  PRESSTYP *prs;
  char *strloc;

  dummycnt=vasprintf(&strloc,frm,args);
  prs=calloc(1,sizeof(PRESSTYP));
  prs->stamp=rou_systime();
  prs->news=strdup(strloc);
  jr->pressbook=(PRESSTYP **)rou_addveclst((void **)jr->pressbook,(void *)prs);
  if (logflush==true)
    (void) flushjournal(numjrl);
  (void) free(strloc);
  }
va_end(args);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to display an core_dump	*/
/*	message and terminate application	*/
/*						*/
/************************************************/
void core_dump(const char *fmt,...)

{
#define MXSTEP 30

va_list args;
struct rlimit limites;
char dirdump[1000];
char str1[1000];
char str2[2000];

if (getrlimit(RLIMIT_CORE,&limites)<0) {
  (void) alert(0,"getrlimit error='%s'",strerror(errno));
  }
limites.rlim_cur=limites.rlim_max;
if (setrlimit(RLIMIT_CORE,&limites)<0) {
  (void) alert(0,"setrlimit error='%s'",strerror(errno));
  }
(void) prctl(PR_SET_DUMPABLE,1,0,0,0);/*to allow core-dump	*/
va_start(args,fmt);
(void) rou_mkname(dirdump,sizeof(dirdump),d_crash,"","","");
if (chdir(dirdump)<0) {
  char command[2000];

  (void) snprintf(dirdump,sizeof(dirdump),"%s/coredump","/tmp");
  (void) snprintf(command,sizeof(command),"mkdir -p %s",dirdump);
  if (system(command)<0)
    (void) alert(0,"Unable to execute command <%s>",command);
  if (chdir(dirdump)<0)
    (void) abort();
  }
(void) vsnprintf(str2,sizeof(str2),fmt,args);
if (jrl[0].edition!=(char *)0) {
  (void) journalling(true,APJRL,"");
  (void) journalling(false,APJRL,"%s [was pid=%d] core_dump request! "
			         "condition: <%s>",appname,getpid(),str2);
  (void) journalling(true,APJRL,"%s [was pid=%d] Terminated in <%s>",
		     		appname,getpid(),dirdump);
  }
(void) alert(0,"Fatal error <%s>",str2);
(void) snprintf(str1,sizeof(str1),"going to CORE DUMP (in %score.%d)\n",
				   dirdump,getpid());
(void) alert(0,str1);
va_end(args);
(void) sleep(5);	/*to avoid crash avalanche	*/
(void) abort();		/*doing the abort		*/
			/*Theoriticaly unreachable	*/
while (true) {
  (void) sleep(1);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to generate a core dump in	*/
/*	case of fatal code fault.		*/
/*	message and terminate application	*/
/*						*/
/************************************************/
void code_fault(const char *fmt,...)

{
va_list args;
char strloc[1000];

va_start(args,fmt);
(void) vsnprintf(strloc,sizeof(strloc),fmt,args);
(void) core_dump(strloc);
va_end(args);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to find and limit space to	*/
/*	be used as status information available	*/
/*	from proc (ps ax)			*/
/*						*/
/************************************************/
void preptitle(int argc,char *argv[],char *env[])

{
extern char **environ;

char *lastend;

lastend=(char *)0;
(void) cleantitle();
title=(TITLTYP *)calloc(1,sizeof(TITLTYP));
if (argv!=(char **)0) {
  int i;

  title->title=argv[0];
  for (i=1;argv[i]!=(char *)0;i++) {
    lastend=argv[i]+strlen(argv[i]);
    argv[i]=(char *)0;
    }
  /*(void) memset(argv,'\000',argc*sizeof(argv[0]));*/
  }
if (env!=(char **)0) {
  char **newenv;
  int i;

  newenv=(char **)0;
  for (i=0;env[i]!=(char *)0;i++) {
    lastend=env[i]+strlen(env[i]);
    newenv=(char **)rou_addveclst((void **)newenv,(void *)strdup(env[i]));
    }
  environ=newenv;
  }
title->max=lastend-title->title;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free used title memory	*/
/*						*/
/************************************************/
void cleantitle()

{
if (title!=(TITLTYP *)0) {
  environ=rou_freestrlst(environ);
  (void) free(title);
  title=(TITLTYP *)0;
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to update title information.	*/
/*	title information is available via the	*/
/*	"ps" command.				*/
/*						*/
/************************************************/
void settitle(const char *fmt,...)

{
va_list args;

va_start(args,fmt);
if ((title!=(TITLTYP *)0)&&(title->title!=(char *)0)) {
  (void) memset(title->title,'\000',title->max);
  (void) vsnprintf(title->title,title->max,fmt,args);
  }
va_end(args);
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to put the main process in 	*/
/*	background mode				*/
/*						*/
/************************************************/
void divedivedive(int foregrnd)

{
if (rou_fork(foregrnd)!=0) {
  /*waiting  for ballast to be full :-}}	*/
  (void) usleep(1000);
  (void) exit(0);
  }
}
