/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	UNIDEV:					*/
/*	Take Care of all access to devices	*/
/*	could be system dependent.		*/
/*						*/
/************************************************/
#include	<stdlib.h>
#include	<unistd.h>
#include	<syslog.h>
#include	<string.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<netdb.h>
#include	<ctype.h>
#include	<sys/ioctl.h>
#include	<net/if.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include	"unidev.h"
#include	"subrou.h"
/*

*/
/************************************************/
/*						*/
/*	Procedure to duplicate a device lst 	*/
/*	contents.				*/
/*						*/
/************************************************/
static LSTTYP *duplst(LSTTYP *lst)

{
LSTTYP *neu;

neu=(LSTTYP *)0;
if (lst!=(LSTTYP *)0) {
  neu=(LSTTYP *)calloc(1,sizeof(LSTTYP)); 
  neu->handle=lst->handle;
  neu->proto=lst->proto;
  neu->port=lst->port;
  neu->iteration=lst->iteration;
  neu->process=(pid_t *)calloc(neu->iteration,sizeof(pid_t));
  (void) memcpy(neu->process,lst->process,neu->iteration*sizeof(pid_t));
  neu->ipnum=strdup(lst->ipnum);
  }
return neu;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to set the listening address	*/
/*						*/
/************************************************/
static int setaddress(char *ipname,struct in_addr *addr)


{
#define OPSE	"unidev.c:setaddress,"

int done;
int phase;
int proceed;
char ipnum[50];

done=false;
phase=0;
proceed=true;
(void) strcpy(ipnum,"");
while (proceed==true) {
  switch (phase) {
    case 0	:	/*checking ipname	*/
      if ((ipname==(char *)0)||(strlen(ipname)==0)) {
	(void) alert(0,"%s no host ip specified",OPSE);
        }
      (void) snprintf(ipnum,sizeof(ipnum),"%s",ipname);
      break;
    case 1	:	/*is ipname * (ANY)	*/
      if (strcmp(ipname,"*")==0) {
	addr->s_addr=htonl(INADDR_ANY);
	phase=999;
        }
      break;
    case 2	:	/*is ipname an hostname	*/
      if (isdigit(ipname[0])==0) {
        struct hostent *h;

        if ((h=(struct hostent *)gethostbyname(ipname))!=(struct hostent *)0){
           struct in_addr addrparam;

          (void) memcpy(&addrparam,h->h_addr_list[0],h->h_length);
          (void) strncpy(ipnum,inet_ntoa(addrparam),sizeof(ipnum));
	  }
        }
      break;
    case 3	:	/*we have now an IP num	*/
      if ((inet_aton(ipnum,addr))==0) {
	(void) alert(0,"%s ipnum=<%s> can't be converted");
	proceed=false;
        }
      break;
    default	:	/*SAFE Guard		*/
      done=true;
      proceed=false;
      break;
    }
  phase++;
  }
return done;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate a previously 	*/
/*	existing device definition in a list	*/
/*						*/
/************************************************/
static LSTTYP *searchprv(LSTTYP **lst,LSTTYP *prv)

{
LSTTYP *loc;

loc=(LSTTYP *)0;
if (lst!=(LSTTYP **)0) {
  int i;

  for (i=0;lst[i]!=(LSTTYP *)0;i++) {
    if (lst[i]->proto!=prv->proto)
      continue;
    if (lst[i]->port!=prv->port)
      continue;
    if (strcmp(lst[i]->ipnum,prv->ipnum)!=0)
      continue;
    loc=lst[i];
    break;
    }
  }
return loc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate an ANY_ADDR within	*/
/*	the device list, return a list of	*/
/*	any devices.				*/
/*						*/
/************************************************/
static LSTTYP **grabany(LSTTYP **lst)

{
LSTTYP **lstany;

lstany=(LSTTYP **)0;
if (lst!=(LSTTYP **)0) {
  int i;


  for (i=0;lst[i]!=(LSTTYP *)0;i++) {
    if (strcmp(lst[i]->ipnum,"*")==0) {
      LSTTYP *ke;

      ke=duplst(lst[i]);
      lstany=(LSTTYP **)rou_addveclst((void **)lstany,(void *)ke);
      lst[i]->iteration=0;
      }
    }
  }
return lstany;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to locate an ANY_ADDR within	*/
/*	the device list an merge iteration	*/
/*	on it.					*/
/*						*/
/************************************************/
static LSTTYP **mergelst(LSTTYP **lst)

{
LSTTYP **lstany;

if ((lstany=grabany(lst))!=(LSTTYP **)0) {
  int i;

  for (i=0;lstany[i]!=(LSTTYP *)0;i++) {
    int j;
    int niter;
    LSTTYP *any;

    any=lstany[i];
    niter=0;
    for (j=0;lst[j]!=(LSTTYP *)0;j++) {
      if (any->proto!=lst[j]->proto)
	continue;
      if (any->port!=lst[j]->port)
	continue;
      if (any->iteration==0)
	continue;
      niter=any->iteration+lst[j]->iteration;
      any->process=(pid_t *)realloc(any->process,niter*sizeof(pid_t));
      (void) memcpy(any->process+any->iteration,lst[j]->process,
		    lst[j]->iteration*sizeof(pid_t));
      any->iteration=niter; 
      lst[j]->iteration=0;
      }
    }
  /*taking care of remaining in the list	*/
  for (i=0;lst[i]!=(LSTTYP *)0;i++) {
    LSTTYP *ke;

    ke=(LSTTYP *)0;
    if (lst[i]->iteration==0)
      continue;
    ke=duplst(lst[i]);
    lstany=(LSTTYP **)rou_addveclst((void **)lstany,(void *)ke);
    }
  lst=dev_freelst(lst);
  lst=lstany;
  }
return lst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to accept a new handle from	*/
/*	the list of connect request.		*/
/*						*/
/************************************************/
static int getnewhandle(LSTTYP *lst)

{
#define	GTPF	"unidev.c:getnewhandle,"

int newhandle;
socklen_t taille;
struct sockaddr addr;
int flags;

newhandle=-1;
taille=sizeof(addr);
if ((newhandle=accept(lst->handle,&addr,&taille))<0) {
  switch (errno)  {
    case EAGAIN		:
      /*well process not quick enough the handle was already	*/
      /*accepted by another clement???.				*/
      break;
    case EINVAL		:	/*no socket avail anymore	*/
      quickout=true;		/*lets get out fast		*/
      break;
    default		:
      (void) alert(0,"%s Unexpected error on IP/PORT <%s/%d> errno=<%s>",
		      GTPF,lst->ipnum,lst->port,strerror(errno));
      break;
    }
  return newhandle;
  }
if ((flags=fcntl(newhandle,F_GETFL,0))<0)
  (void) code_fault("%s Unable to get socket descripteur (error='%s')",
		     GTPF,strerror(errno));
if ((flags=fcntl(newhandle,F_SETFL,flags|O_NONBLOCK|O_ASYNC))<0)
  (void) code_fault("%s Unable to set socket descripteur (error='%s')",
		     GTPF,strerror(errno));
return newhandle;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan binding info and	*/
/*	extract (if present) the prototocol	*/
/*	definition.				*/
/*	Available protocole are:		*/
/*	p_stmp:	SMTP Protocol (SUBSET)		*/
/*	p_chkid: CHKID Protocol			*/
/*						*/
/*	If the protocol info is not available	*/
/*	return p_smtp.				*/
/*						*/
/************************************************/
static PROTYP overproto(PROTYP proto,char *bindinfo)

{
char *ptr;

if ((ptr=strchr(bindinfo,'%'))!=(char *)0) {
  static const char *lstpro[]={PROSMTP,PROCHKID,(char *)0};
  PROTYP proloc;
  int i;

  *ptr='\000';
  ptr++;
  proloc=pro_unknown;
  for (i=0;lstpro[i]!=(char *)0;i++) {
    if (strcasecmp(bindinfo,lstpro[i])==0) {
      proloc=(PROTYP)i;
      break;
      }
    }
  if (proloc==pro_unknown) {
    (void) alert(0,"%s Unable to understand protocol part in <%s%%%s> "
		   "(forcing '%d')","unidev.c:overproto,",bindinfo,ptr,proto);
    proloc=proto;
    }
  (void) memmove(bindinfo,ptr,strlen(ptr)+1);
  proto=proloc;
  }
return proto;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to scan binding info from	*/
/*	a string, return an LSTTYP* structure	*/
/*						*/
/************************************************/
static LSTTYP *scaninfo(PROTYP proto,char *bindinfo)

{
LSTTYP *lstloc;
char strip[50];
int port;
int iteration;

lstloc=(LSTTYP *)0;
iteration=1;
proto=overproto(proto,bindinfo);
if (sscanf(bindinfo,"%49[^,],%d,%d",strip,&port,&iteration)>=2) {
  lstloc=(LSTTYP *)calloc(1,sizeof(LSTTYP)); 
  lstloc->handle=-1;
  lstloc->proto=proto;
  lstloc->port=port;
  lstloc->iteration=iteration;
  if (iteration>0) {
    lstloc->process=(pid_t *)calloc(iteration,sizeof(pid_t));
    }
  lstloc->ipnum=strdup(strip);
  }
else 
  (void) alert(0,"unidev.c:scaninfo, Unable to scan bindinfo <%s>",bindinfo);
return lstloc;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to have a list of all		*/
/*	ethernet device available.		*/
/*						*/
/************************************************/
static struct ifreq *devethlist()

{
#define	OPBI	"unidev.c:dev_ethlist,"
struct ifreq *list;
int handle;

list=(struct ifreq *)0;
if ((handle=socket(PF_INET,SOCK_DGRAM,0))<0) {
  (void) alert(0,"%s Unable to open a socket (error='%s')",
		 OPBI,strerror(errno));
  }
else {
#define MXDEV	600

  struct ifconf ifc;
  int taille;
 
  taille=sizeof(struct ifreq); 
  ifc.ifc_req=(struct ifreq *)calloc(MXDEV,taille);
  taille*=MXDEV;
  ifc.ifc_len=taille;
  if (ioctl(handle,SIOCGIFCONF,(char *)&ifc)<0) {
    (void) alert(0,"%s Unable to access eth device (error='%s')",
		   OPBI,strerror(errno));
    (void) free(ifc.ifc_req);
    }
  else {
    if (ifc.ifc_len<taille) {
      list=ifc.ifc_req;
      }
    else {
      (void) core_dump("%s too many device in PF_NET",OPBI);
      }
    }
  (void) close(handle);
  }
return list;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory used by a 	*/
/*	LSTTYP information.			*/
/*						*/
/************************************************/
static LSTTYP *devonefree(LSTTYP *lst)

{
if (lst!=(LSTTYP *)0) {
  if (lst->ipnum!=(char *)0)
     (void) free(lst->ipnum);
   if (lst->process!=(pid_t *)0)
     (void) free(lst->process);
  (void) free(lst);
  lst=(LSTTYP *)0;
  }
return lst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to close one waiting channel	*/
/*	an terminating all related process.	*/
/*						*/
/************************************************/
static void devoneclose(LSTTYP *lst)

{
if (shutdown(lst->handle,SHUT_RDWR)<0) {
  (void) alert(0,"unidev.c:devoneclose, shutdown error on IP/PORT <%s/%d> "
		 "(error='%s')",lst->ipnum,lst->port,strerror(errno));
  }
if (lst->iteration>0) {
  int n;
  pid_t *pidlst;
  int i;

  n=0;
  pidlst=(pid_t *)calloc(lst->iteration,sizeof(pid_t));
  for (i=0;i<lst->iteration;i++) {
    if (lst->process[i]!=(pid_t)0) {
      pidlst[n]=lst->process[i];
      n++;
      }
    }
  if (n>0)
    (void) rou_childkill(pidlst,n,10);
  (void) free(pidlst);
  }
(void) close(lst->handle);
lst->handle=-1;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to build a brand new list of	*/
/*	ip/port/iteration to listen on.		*/
/*						*/
/************************************************/
LSTTYP **buildlst(PROTYP proto,char *bindinfo)

{
#define	DPORT	5025	/*default waiting port	*/

LSTTYP **lst;
char *scan;

lst=(LSTTYP **)0;
scan=(char *)0;
if (bindinfo==(char *)0) {
  struct ifreq *ethlist;
  
  if ((ethlist=devethlist())!=(struct ifreq *)0) {
    int i;

    bindinfo=(char *)calloc(1,10);
    for (i=0;ethlist[i].ifr_name[0]!='\000';i++) {
      struct sockaddr_in *addr;
      struct in_addr addrret;
      char iface[50];

      if (strcmp(ethlist[i].ifr_name,"lo")==0)
        continue;
      addr=(struct sockaddr_in *)&(ethlist[i].ifr_addr);
      addrret.s_addr=addr->sin_addr.s_addr;
      (void) snprintf(iface,sizeof(iface),"%s,%d,1",inet_ntoa(addrret),DPORT);
      bindinfo=realloc(bindinfo,strlen(bindinfo)+strlen(iface)+3);
      if (strlen(bindinfo)>0)
        (void) strcat(bindinfo,"/");
      (void) strcat(bindinfo,iface);
      }
    (void) free(ethlist);
    }
  else {
    char last[100];
    /*no eth device found! going on lo		*/
    /*proto smtp, port 5025, one binding only	*/
    (void) snprintf(last,sizeof(last),"%s,%d,1","127.0.0.1",DPORT);
    bindinfo=strdup(last);
    }
  }
else 
  bindinfo=strdup(bindinfo);
scan=bindinfo;
while (strlen(scan)>0) {
  LSTTYP *lstloc;
  char *ptr;
 
  lstloc=(LSTTYP *)0; 
  if ((ptr=strchr(scan,':'))!=(char *)0)
    *ptr='\000';
  if ((lstloc=scaninfo(proto,scan))!=(LSTTYP *)0) {
    LSTTYP *prv;

    if ((prv=searchprv(lst,lstloc))!=(LSTTYP *)0) {
      int niter;

      niter=prv->iteration+lstloc->iteration;
      prv->process=(pid_t *)realloc(prv->process,niter*sizeof(pid_t));
      (void) memset(prv->process+prv->iteration,'\000',
		    lstloc->iteration*sizeof(pid_t));
      prv->iteration=niter;
      lstloc=devonefree(lstloc);
      }
    else {
      lst=(LSTTYP **)rou_addveclst((void **)lst,(void *)lstloc);
      }
    }
  scan="";
  if (ptr!=(char *)0) {
    ptr++;
    scan=ptr;
    }
  }
lst=mergelst(lst);
(void) free(bindinfo);
return lst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to establish the list of	*/
/*	ip/port/iteration to listen on.		*/
/*						*/
/************************************************/
LSTTYP **dev_getlst(LSTTYP **prv,PROTYP proto,char *bindinfo)

{
LSTTYP **newlst;

if ((newlst=buildlst(proto,bindinfo))!=(LSTTYP **)0) {
  if (prv!=(LSTTYP **)0) {
    LSTTYP **mrglst;
    int i;

    mrglst=(LSTTYP **)0;
    for (i=0;newlst[i]!=(LSTTYP *)0;i++) {
      LSTTYP *old;

      if ((old=searchprv(prv,newlst[i]))!=(LSTTYP *)0) {
	int iter;

	iter=newlst[i]->iteration;
	if (old->iteration!=iter) {

	  if (old->iteration<iter) {
	    int taille;
	    int j;

	    taille=sizeof(pid_t)*iter;
	    old->process=(pid_t *)realloc((void *)old->process,taille);
	    for (j=old->iteration;j<iter;j++)
	      old->process[j]=(pid_t)0;
	    }
	  else {
	    pid_t *ptr;
  
	    ptr=old->process+iter;
	    (void) rou_childkill(ptr,old->iteration-iter,5);
            }
	  }
	old->iteration=iter;
        newlst[i]->iteration=0;
        }
      }
    for (i=0;prv[i]!=(LSTTYP *)0;i++) {
      if (searchprv(newlst,prv[i])==(LSTTYP *)0) {
        /*removing unneeded anymore	*/
        (void) devoneclose(prv[i]);
        prv[i]=devonefree(prv[i]);
        }
      else {
	/*keeping good connection	*/
        mrglst=(LSTTYP **)rou_addveclst((void **)mrglst,(void *)prv[i]);
	}
      }
    (void) free(prv);
    for (i=0;newlst[i]!=(LSTTYP *)0;i++) {
      if (newlst[i]->iteration==0) {
        newlst[i]=devonefree(newlst[i]);
	}
      else {
	/*brand new connection detected	*/
        mrglst=(LSTTYP **)rou_addveclst((void **)mrglst,(void *)newlst[i]);
	}
      }
    (void) free(newlst);
    newlst=mrglst;
    }
  }
return newlst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to free memory related to a	*/
/*	binded devices list.			*/
/*						*/
/************************************************/
LSTTYP **dev_freelst(LSTTYP **lst)

{
if (lst!=(LSTTYP **)0) {
  int i;

  for (i=0;lst[i]!=(LSTTYP *)0;i++) {
    lst[i]=devonefree(lst[i]);
    }
  (void) free(lst);
  lst=(LSTTYP **)0;
  }
return lst;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to bind a list of interface	*/
/*	to a specific port.			*/
/*						*/
/************************************************/
void dev_mullisten(LSTTYP **lst)

{
#define MXSTRICK 1
#define	MULOPEN	"unidev.c:dev_mullisten,"

if (lst!=(LSTTYP **)0) {
  uid_t gid;
  uid_t uid;
  int i;

  gid=getegid();	/*let be back to root if needed	*/
  uid=geteuid(); 	/*to open device on < 1024 port	*/
  if (setegid(getgid())<0) {
    (void) alert(0,"%s Unable to set the Egid to '%d'",MULOPEN,getgid());
    }
  if (seteuid(getuid())<0) {
    (void) alert(0,"%s Unable to set the Euid to '%d'",MULOPEN,getuid());
    }
  for (i=0;(lst[i]!=(LSTTYP *)0);i++) {
    int handle;
    struct sockaddr_in sosin;
    int strick;

    if ((lst[i]->lasttry+5)>rou_systime())
      continue;		/*binding try done less than 5 sec ago	*/
    handle=lst[i]->handle;
    sosin.sin_family=AF_INET;
    sosin.sin_port=htons(lst[i]->port);
    memset(sosin.sin_zero,'\000',sizeof(sosin.sin_zero));
    if (setaddress(lst[i]->ipnum,&(sosin.sin_addr))==false)
      continue;
    for (strick=0;(handle<0)&&(strick<MXSTRICK);strick++) {
      int flags;
      struct linger slinger;
      char buf[30];

      if ((handle=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP))<0)
        (void) code_fault("%s Unable to open socket (error='%s')",
			  MULOPEN,strerror(errno));
      if (setsockopt(handle,SOL_SOCKET,SO_REUSEADDR,buf,sizeof(buf))<0)
        (void) code_fault("%s Unable to set socket option (error='%s')",
			  MULOPEN,strerror(errno));
      slinger.l_onoff=true;
      slinger.l_linger=5;	/*5 second timeout on close	*/
      if (setsockopt(handle,SOL_SOCKET,SO_LINGER,&slinger,sizeof(slinger))<0)
        (void) code_fault("%s Unable to set socket so_inger (error='%s')",
			  MULOPEN,strerror(errno));
      if ((flags=fcntl(handle,F_GETFL,0))<0)
        (void) code_fault("%s Unable to get socket descripteur (error='%s')",
			  MULOPEN,strerror(errno));
      if (fcntl(handle,F_SETFL,flags|O_NONBLOCK|O_ASYNC)<0)
        (void) code_fault("%s Unable to set socket in NONBLOCK mode "
			  "(error='%s')",MULOPEN,strerror(errno));
      if (bind(handle,(struct sockaddr *)&sosin,sizeof(sosin))<0) {
    	(void) tracing(TRALOG,0,"%s Unable to bind IP/PORT <%s/%d> on "
				"(error='%s')",
                                 MULOPEN,lst[i]->ipnum,lst[i]->port,
				 strerror(errno));
	(void) close(handle);
	handle=-1;
	}
      else {
        if (listen(handle,5)<0) {
	  (void) tracing(TRALOG,0,"%s Unable to listen port '%d' on "
				  "IP '%s' (error='%s')",
                                   MULOPEN,lst[i]->port,lst[i]->ipnum,
				   strerror(errno));
	  (void) close(handle);
	  handle=-1;
 	  }
	}
      if (handle<0)
	(void) sleep(1);
      else {
	(void) tracing(TRALOG,1,"Listening on IP/PORT <%s/%d>",
				 lst[i]->ipnum,lst[i]->port);
        break;
	}
      }
    lst[i]->lasttry=rou_systime();
    lst[i]->handle=handle;
    }
  (void) setegid(gid);
  (void) seteuid(uid);
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to remove bindinf from	*/
/*	interface and specific port.		*/
/*						*/
/************************************************/
void dev_mulclose(LSTTYP **lst)

{
if (lst!=(LSTTYP **)0) {
  int i;

  for (i=0;lst[i]!=(LSTTYP *)0;i++) {
    if (lst[i]->handle<0)
      continue;
    (void) devoneclose(lst[i]);
    }
  }
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to wait a connection from a	*/
/*	client, return NULL if quickout flag	*/
/*	is set, a FILE pointer otherwise.	*/
/*						*/
/************************************************/
int dev_acceptfile(LSTTYP *lst)

{
#define	ACPTF	"unidev.c:dev_acceptfile,"
int newhandle;

newhandle=-1;
if (lst!=(LSTTYP *)0) {
  int proceed;

  proceed=true;
  while (proceed==true) {
    fd_set reading;
    struct timeval relax;

    proceed=false;
    if (lst->handle<0) 
      break;
    FD_ZERO(&reading);
    FD_SET(lst->handle,&reading);
    relax.tv_sec=5;	/*2 sec relax time	*/
    relax.tv_usec=0;
    switch (select(lst->handle+1,&reading,(fd_set *)0,(fd_set *)0,&relax)) {
      case	-1	:
	switch (errno) {
	  case EINTR	:	/*received a signal, lets see...*/
	    break;		/*getting out fast		*/
	  default	:
	    (void) code_fault("%s select error '%s'",ACPTF,strerror(errno));
	    break;
	  }
        break;
      case      0	:	/*checking if parent is lost	*/
        quickout=!(rou_checkprocess(getppid()));
	break;
      default		: 
        newhandle=getnewhandle(lst);
	break;
      }
    /*we may have NO channel open, lets exit anyway		*/
    }
  }
return newhandle;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to get a name related to an	*/
/*	IP address.				*/
/*	Return a newly allocated memory which	*/
/*	contents is the needed information.	*/
/*						*/
/*	MODE:					*/
/*		pr_loc  -> local reverse address*/
/*		pr_name -> remote reverse name	*/
/*		pr_ipnum-> remote IP number	*/
/*						*/
/************************************************/
char *dev_getaddrname(int handle,PEERTYP peer,char *unknown)

{
char *rvname;
socklen_t taille;
int status;
struct sockaddr_in intaddr;
char ipnumber[50];

taille=sizeof(intaddr);
status=-1;
rvname=unknown;
(void) strcpy(ipnumber,"");
switch(peer) {
  case pr_locname	:
  case pr_locip		:
    status=getsockname(handle,(struct sockaddr *)&intaddr,&taille);
    break;
  case pr_remname	:
  case pr_remip		:
    status=getpeername(handle,(struct sockaddr *)&intaddr,&taille);
    break;
  default	:
    break;
  }
if (status<0) {
  switch (errno) {
    case ENOTCONN	:	/*other side just vanished	*/
      break;
    default		:
      (void) alert(0,"undev.c:dev_getaddrname, Unable to find reverse (error='%s')",
		      strerror(errno));
      break;
    }
  }
else {
  struct hostent *hinf;

  switch (peer) {
    case pr_locname	:
    case pr_remname	:
      hinf=gethostbyaddr((const char *)&intaddr.sin_addr.s_addr,
		          sizeof(intaddr.sin_addr.s_addr),AF_INET);
      if (hinf!=(struct hostent *)0) 
        rvname=hinf->h_name;
      break;
    case pr_remip	:
    case pr_locip	:
      rvname=inet_ntoa(intaddr.sin_addr);
      break;
    }
  }
return strdup(rvname);
}
