/************************************************/
/*						*/
/*      Copyright:				*/
/*	 Jean-Marc Pigeon <jmp@safe.ca>	 2009	*/
/*						*/
/************************************************/
/* This program is free software; you can 	*/
/* redistribute it and/or modify it under the 	*/
/* terms of the GNU General Public License as	*/
/* published by the Free Software Foundation	*/
/* version 2 of the License			*/
/*						*/
/* This program is distributed in the hope that */
/* it will be useful, but WITHOUT ANY WARRANTY; */
/* without even the implied warranty of		*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR	*/
/* PURPOSE.  See the GNU General Public License	*/
/* for more details.				*/
/*						*/
/* You should have received a copy of the GNU	*/
/* General Public License along with this 	*/
/* program; if not, write to the Free Software	*/
/* Foundation, Inc., 51 Franklin Street,	*/
/* Fifth Floor, Boston, MA  02110-1301, USA.	*/
/************************************************/
/*						*/
/*	UNIEML:					*/
/*	Take Care of all access low level 	*/
/*	E-mail pertinence level handling.	*/
/*						*/
/************************************************/
#include	<strings.h>
#include	"uniper.h"

static	char	*strlng[]={
		"dflt",		/*dflt language	*/
		"English",	/*English	*/
		"Francais",	/*French	*/
		(char *)0
		};
/*

*/
/************************************************/
/*						*/
/*	Procedure to return the language code	*/
/*	according the string submitted.		*/
/*						*/
/************************************************/
LNGTYP per_getlng(char *strlang)

{
LNGTYP lng;

lng=l_unknown;
if (strlang!=(char *)0) {
  int i;

  for (i=0;strlng[i]!=(char *)0;i++) {
    if (strcasecmp(strlng[i],strlang)==0) {
      lng=(LNGTYP)i;
      break;
      }
    }
  }
return lng;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return a static string 	*/
/*	according language used and pertinence	*/
/*	level.					*/
/*						*/
/************************************************/
const char *per_getstrper(LNGTYP lng,PERTYP pertinence)

{
static const char *lstrperti[][(int)p_unknown]={
        /*English version       */
        {"nocheck","nocare","scrap","spam","caution",
         "contact","known","reply","nofilter"},
        /*French version        */
        {"nocheck","Direct","Rebut","Pourriel","Attention",
         "Contact","Connu","Confiance","Sans_filtre"}
        };

const char *strperti;

strperti=(char *)0;
switch (lng) {
  case  l_french	:
    strperti=lstrperti[1][(int)pertinence];
    break;
  case  l_dflt		:
  case  l_english	:
  default		:
    strperti=lstrperti[0][(int)pertinence];
    break;
  }
return strperti;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return a static string 	*/
/*	according language value		*/
/*						*/
/************************************************/
const char *per_getstrlng(LNGTYP lng)

{
const char *str;

str=strlng[(int)l_english];
switch (lng) {
  case l_dflt		:
  case l_english	:
  case l_french		:
    str=strlng[(int)lng];
    break;
  default		:
    break;
  }
return str;
}
/*

*/
/************************************************/
/*						*/
/*	Procedure to return a static string 	*/
/*	according language value and URL	*/
/*	requested.				*/
/*						*/
/************************************************/
const char *per_getstrurl(LNGTYP lng,URLTYP url)

{
#define URL1 "<A HREF=\"http://clement.%s/goodmail.php?email=%s&id=%s&lang=%s\">"

static const char *lstrurl[][(int)l_unknown]={
        /*English version       */
	{
	"<HTML><BODY>"
	"This E-mail seems to be a SPAM, you can check the E-mail attachment and<BR>"
	"if you do not consider it as SPAM, you can acknowledge the originator<BR>"
	"to accept next E-mail sending from this originator.<BR>"
	URL1
	"To acknowledge the originator, please click</A><BR>"
	"</BODY></HTML>"
	},
        /*French version        */
	{
	"<HTML><BODY>"
	"Ce courrier semble &#234tre un POURRIEL, "
	"vous pouvez v&#233;rifier le contenu du fichier attach&#233; et<BR>"
	"s'il est pertinent, vous pouvez valider les "
	"envoits ult&#233;rieur de ce correpondant.<BR>"
	URL1
	"Pour valider ce correspondant cliquer cet hyper-lien</A><BR>" 
	"</BODY></HTML>"
	}
        };

const char *strurl;

strurl=(char *)0;
switch (lng) {
  case  l_french	:
    strurl=lstrurl[1][(int)url];
    break;
  case  l_dflt		:
  case  l_english	:
  default		:
    strurl=lstrurl[0][(int)url];
    break;
  }
return strurl;
}
